package org.gridschool.unicore.plugins.hello;

import java.awt.BorderLayout;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.client.panels.FileExportPanel;
import com.pallas.unicore.client.panels.FileImportPanel;
import com.pallas.unicore.client.panels.JPAPanel;

/**
 * Minimal implementation of a JPAPanel, the graphical respresentation of a
 * plugin container.
 */
public class HelloJPAPanel extends JPAPanel {
    static final long serialVersionUID = 1553160802311526855L;

    // plugin container that this panel represents
    private HelloContainer container;

    // a text field to specify task name
    private JTextField nameTextField;

    //	panel to define file imports
    private FileImportPanel importPanel;

    // panel to define file exports
    private FileExportPanel exportPanel;

    /**
     * Constructor
     * 
     * @param parentFrame
     *            reference to main JFrame for modal sub dialogs
     * @param container
     *            container that this panel represents
     */
    public HelloJPAPanel(JFrame parentFrame, HelloContainer container) {
        super(parentFrame, container);
        this.container = container;

        // build gui
        buildComponents();
    }

    /**
     * Build GUI components
     */
    private void buildComponents() {
        // build text field for task name
        nameTextField = new JTextField(20);
        nameTextField.setToolTipText("Enter identifier for this task");

        JPanel namePanel = new JPanel();
        namePanel.add(new JLabel("Name: "));
        namePanel.add(nameTextField);

        // build pre defined import and export panels
        importPanel = new FileImportPanel(container);
        importPanel.setBorder(new TitledBorder("File Imports"));

        exportPanel = new FileExportPanel(container);
        exportPanel.setBorder(new TitledBorder("File Exports"));

        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.add(importPanel, "Imports");
        tabbedPane.add(exportPanel, "Exports");

        add(namePanel, BorderLayout.NORTH);
        add(tabbedPane, BorderLayout.CENTER);
        setBorder(new TitledBorder("Hello World Plug-in"));

    }

    /**
     * Apply values from GUI to container
     */
    public void applyValues() {
        logger.info("Applying values...");

        // replace spaces in name with underscores and
        // put name into container
        nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
        container.setName(nameTextField.getText());

        // pass event to import and export panels
        importPanel.applyValues();
        exportPanel.applyValues();
    }

    /**
     * Fill values from container into gui
     */
    public void resetValues() {
        logger.info("Resetting values...");

        if (container.getIdentifier() != null && container.getName() != null) {
            nameTextField.setText(container.getName());
        }
        else {
            nameTextField.setText("");
        }

        // pass event to import and export panels
        importPanel.resetValues();
        exportPanel.resetValues();
    }

    /**
     * This method is executed whenever the panel becomes visible. Here can be
     * checked for instance if resources have changed
     * 
     * @param vsiteChanged
     *            true, if vsite has changed since last update or reset
     */
    public void updateValues(boolean vsiteChanged) {
        logger.info("Updating values, vsiteChanged: " + vsiteChanged);

        if (vsiteChanged) {
            // if vsite changed, pass event to import and export panels
            importPanel.updateValues();
            exportPanel.updateValues();
        }
    }
}