/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.gridschool.unicore.plugins.povray;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import com.pallas.unicore.client.panels.Applyable;
import com.pallas.unicore.client.panels.ImagePanel;
import com.sixlegs.image.png.PngImage;

/**
 * Additional panel for JMC outcome area to display the rendered image
 * 
 * @author Ralf Ratering
 * @version $Id: RenderPanel.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class RenderPanel extends JPanel implements Applyable {

    private static Logger logger = Logger.getLogger("com.pallas.unicore.client.plugins.povray");
    final static long serialVersionUID = -8104512832433094363L;
    private File[] files;
    private Vector images = new Vector();
    private JTextField filenameTextField;
    private ImagePanel imagePanel;
    private NextAction nextAction = new NextAction();
    private PrevAction prevAction = new PrevAction();
    private int currentIndex;

    /**
     * Constructor
     */
    public RenderPanel() {
        buildComponents();
    }

    /**
     * This method gets executed whenever the panel contents will be applied to
     * an object
     */
    public void applyValues() {
        logger.fine("Applying povray render panel...");
    }

    private void buildComponents() {
        setLayout(new BorderLayout());
        imagePanel = new ImagePanel(false);
        add(new JScrollPane(imagePanel), BorderLayout.CENTER);
        JPanel filePanel = new JPanel();
        filePanel.add(new JLabel("Filename:"));
        filenameTextField = new JTextField(40);
        filePanel.add(filenameTextField);
        nextAction.setEnabled(false);
        prevAction.setEnabled(false);
        JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 5, 5));
        buttonPanel.add(new JButton(nextAction));
        buttonPanel.add(new JButton(prevAction));
        JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.add(filePanel, BorderLayout.CENTER);
        northPanel.add(buttonPanel, BorderLayout.EAST);
        add(northPanel, BorderLayout.NORTH);
    }

    /**
     * Load image in png format and repaint
     * 
     * @throws IOException
     *             if loading failed
     */
    private void loadImages() throws IOException {
        images.clear();
        for (int i = 0; i < files.length; i++) {
            logger.info("Loading image: " + files[i].getAbsolutePath());
            PngImage png = new PngImage(files[i].getAbsolutePath());
            Image img = Toolkit.getDefaultToolkit().createImage(png);
            images.add(img);
        }
    }

    private void selectImage(int index) {
        imagePanel.setImage((Image) images.elementAt(index));
        filenameTextField.setText(files[index].getAbsolutePath());
        repaint();
        currentIndex = index;
        nextAction.setEnabled(currentIndex < (files.length - 1));
        prevAction.setEnabled(currentIndex > 0);
    }

    /**
     * This method will be executed whenever a panel is filled with values
     */
    public void resetValues() {
        logger.info("Resetting povray render panel...");
        try {
            loadImages();
            selectImage(0);
        }
        catch (IOException e) {
            logger.log(Level.SEVERE, "Could not load image.", e);
        }
    }

    /**
     * Sets the input file
     * 
     * @param files
     *            input image files in png format
     */
    public void setFiles(File[] files) {
        this.files = files;
    }

    /**
     * Called when panel gets visible. Do nothing right now.
     */
    public void updateValues() {
        logger.info("Updating povray render panel...");
    }

    private class NextAction extends AbstractAction {

        public NextAction() {
            super(">|");
        }

        public void actionPerformed(ActionEvent e) {
            if (currentIndex < files.length) {
                selectImage(currentIndex + 1);
            }
        }
    }

    private class PrevAction extends AbstractAction {

        public PrevAction() {
            super("|<");
        }

        public void actionPerformed(ActionEvent e) {
            if (currentIndex > 0) {
                selectImage(currentIndex - 1);
            }
        }
    }
}