/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.UIManager;

/**
 * CombobBox for selecting one of all available fonts. If preview is switched on
 * every font uses itself to render the comboBox entry.
 * 
 * @author Ralf Ratering
 * @version $Id: FontSelectionComboBox.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class FontSelectionComboBox extends JComboBox {

	/**
	 * @author Ralf Ratering
	 * @version $Id: FontSelectionComboBox.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
	 */
	private class FontRenderer extends DefaultListCellRenderer {

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			setComponentOrientation(list.getComponentOrientation());
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			if (value instanceof Icon) {
				setIcon((Icon) value);
				setText("");
			}
			else {
				setIcon(null);
				setText((value == null) ? "" : value.toString());
			}
			setEnabled(list.isEnabled());
			String selectedFont = (String) value;
			if (preview) {
				for (int i = 0; i < fonts.length; i++) {
					if (selectedFont.equals(fonts[i].getFontName())) {
						setFont(new Font(fonts[i].getFontName(), Font.PLAIN, 12));
						continue;
					}
				}
			}
			else {
				setFont(list.getFont());
			}
			setBorder((cellHasFocus) ? UIManager
					.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
			return this;
		}
	}

	public static void main(String args[]) {
		JFrame frame = new JFrame("Test font selector");
		frame.getContentPane().add(new FontSelectionComboBox());
		frame.pack();
		frame.show();
	}
	private Font[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment()
			.getAllFonts();
	private boolean preview = false;

	public FontSelectionComboBox() {
		for (int i = 0; i < fonts.length; i++) {
			addItem(fonts[i].getFontName());
		}
		setRenderer(new FontRenderer());
	}

	public FontSelectionComboBox(boolean preview) {
		this();
		this.preview = preview;
	}
}