/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * @author Ralf Ratering
 * @version $Id: GenericFileFilter.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GenericFileFilter extends FileFilter {

	/**
	 * Get the extension of a file.
	 * 
	 * @return The Extension value
	 */
	public static String getExtension(File f) {
		String name = f.getName();
		int index = name.lastIndexOf('.');
		if (index < 0) {
			return "";
		}
		return name.substring(index, name.length());
	}
	private String ending, description;

	/**
	 * Constructor for the GenericFileFilter object
	 */
	public GenericFileFilter() {
		this("");
	}

	/**
	 * Constructor for the GenericFileFilter object
	 *  
	 */
	public GenericFileFilter(String ending) {
		this.ending = ending;
		description = "All files";
		if (!ending.equals("")) {
			description += " ending with " + ending;
		}
	}

	/**
	 * Constructor for the GenericFileFilter object
	 *  
	 */
	public GenericFileFilter(String ending, String description) {
		this.ending = ending;
		this.description = description;
	}

	/**
	 * If f is a directory or ending matches file return true
	 * 
	 * @return true if file ending matches or file is directory
	 */
	public boolean accept(File f) {
		if (ending.equals("")) {
			return true;
		}
		return f.isDirectory() || ending.equals(getExtension(f));
	}

	public boolean equals(Object object) {
		if (object == null) {
			return false;
		}
		GenericFileFilter filter2 = (GenericFileFilter) object;
		if (filter2.ending.equals(this.ending)
				&& filter2.description.equals(this.description)) {
			return true;
		}
		return false;
	}

	public String getDescription() {
		return description;
	}

	public int hashCode() {
		int result = 17;
		result = 37 * result + ending.hashCode();
		result = 37 * result + description.hashCode();
		return result;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String toString() {
		if (ending.equals("")) {
			return getDescription();
		}
		return getDescription() + " (*" + ending + ")";
	}
}