/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import java.awt.GridBagConstraints;
import java.awt.Insets;

/**
 * Some layout utitlities
 * 
 * @author Ralf Ratering
 * @version $Id: LayoutTools.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class LayoutTools {

	/**
	 * Creates a GridBagConstraints object with all of its fields set to the
	 * passed-in arguments.
	 * 
	 * @param x -
	 *            x specifies the cell at the left of the component's display
	 *            area, where the leftmost cell has gridx=0.
	 * @param y -
	 *            y specifies the cell at the top of the component's display
	 *            area, where the topmost cell has gridy=0.
	 * @param gridwidth -
	 *            gridwith specifies the number of cells in a row for the
	 *            component's display area.
	 * @param weightx -
	 *            weigty specifies how to distribute extra horizontal space.
	 * @param scalable -
	 *            true, if the component resize horizontally; false, if the
	 *            component do not resize.
	 * @return - a GridbagConstraints objects with initial values
	 */
	public static GridBagConstraints makegbc(int x, int y, int gridwidth,
			int weightx, boolean scalable) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = 1;
		gbc.weightx = weightx;
		gbc.anchor = GridBagConstraints.WEST;
		//specifies the external padding of the component, the minimum amount
		//of space between the component and the edges of its display area.
		gbc.insets = new Insets(5, 5, 5, 5);
		if (scalable) {
			gbc.fill = GridBagConstraints.HORIZONTAL;
		}
		else {
			gbc.fill = GridBagConstraints.NONE;
		}
		return gbc;
	}

	/**
	 * Constructor for the LayoutTools object
	 */
	private LayoutTools() {
	}
}