/*
 *paste FZJ BSD licence here.....
 */
package com.pallas.unicore.client.dialogs;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.Key;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.logging.Logger;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * a dialog allowing to export a key to a new PKCS12 or JKS keystore<br>
 *
 * @author schuller
 * @version $Id: ExportKeyDialog.java,v 1.1 2004/06/23 13:41:58 bschuller Exp $
 */
public class ExportKeyDialog extends JDialog {
	
	protected static Logger logger = Logger.getLogger("com.pallas.unicore.client");	
	private String alias, password;
	
	private JTextField aliasField;
	private JPasswordField passwordField;
	private JPasswordField exportPwdField;
	private JPasswordField exportPwdConfirmField;
	private JRadioButton pkcsBtn;

	/**
	 * creates a new dialog
	 * @param parent
	 * @param title
	 */	
	public ExportKeyDialog(JFrame parent, String title) {
		super(parent, title, true);
		initComponents();
	}
	
	/**
	 * Displays this dialog.
	 */
	public void show() {
		setLocationRelativeTo(getParent());
		super.show();
	}

	//-- init GUI
	private void initComponents() {
		GridBagConstraints gc=new GridBagConstraints();
		GridBagLayout gb=new GridBagLayout();
		gc.insets=new Insets(5,5,5,5);
		mainPanel = new javax.swing.JPanel(gb);
		
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});
		JButton exportButton =new JButton("Export key...");
		exportButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				if (check()){ 
					if(exportKey()==true){
						exitForm(null);
					}
					
				}
				else{
					UserMessages.error("Please check the export password and the confirmation!");	
				}
			}
		});
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				exitForm(null);
			}
		});
		aliasField=new JTextField(30);
		passwordField=new JPasswordField(30);
		exportPwdField=new JPasswordField(30);
		exportPwdConfirmField=new JPasswordField(30);
		
		JPanel oldKSPanel=new JPanel(gb);
		oldKSPanel.setBorder(new TitledBorder("Select key to export:"));
		
		addToGridBag(new JLabel("Alias:"),aliasField,gc,gb,oldKSPanel);
		addToGridBag(new JLabel("Keystore Password:"),passwordField,gc,gb,oldKSPanel);
		
		addSingleToGridBag(oldKSPanel, gc,gb,mainPanel);
		
		JPanel newKSPanel=new JPanel(gb);
		newKSPanel.setBorder(new TitledBorder("New keystore:"));
		addToGridBag(new JLabel("Export Password:"),exportPwdField,gc,gb,newKSPanel);
		addToGridBag(new JLabel("Confirm Password:"),exportPwdConfirmField,gc,gb,newKSPanel);
		
		ButtonGroup fGroup=new ButtonGroup();
		pkcsBtn =new JRadioButton("PKCS12 format (recommended)");
		pkcsBtn.setSelected(true);
		JRadioButton jksBtn =new JRadioButton("JKS format");
		fGroup.add(pkcsBtn);
		fGroup.add(jksBtn);
		
		addSingleToGridBag(pkcsBtn, gc,gb, newKSPanel);
		addSingleToGridBag(jksBtn, gc,gb, newKSPanel);
		
		addSingleToGridBag(newKSPanel, gc,gb,mainPanel);
		
		//--buttons
		JPanel btnPanel=new JPanel(new GridLayout(1, 2, 10, 10));
		btnPanel.add(exportButton);
		btnPanel.add(cancelButton);
		
		addSingleToGridBag(btnPanel, gc,gb,mainPanel);
		mainPanel.add(btnPanel,gc);
		
		this.getContentPane().add(mainPanel);
		pack();
	}
	
	private void exitBtnActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
	}
	
	/** hide the dialog */
	private void exitForm(java.awt.event.WindowEvent evt) {
		setVisible(false);
	}
	
	/** Do some consistency checks on the supplied export password: 
	 * export password and confirmation must be given and equal 
	 */
	private boolean check(){
		boolean OK=true;
		char[] newPwd=exportPwdField.getPassword();
		if(newPwd.length==0) OK=false;
		else{
			char[] confirm=exportPwdConfirmField.getPassword();
			if (confirm.length==0 || confirm.length!=newPwd.length) OK=false;
			else{
				for(int i=0;i<confirm.length;i++){
					if(confirm[i]!=newPwd[i]) {
						OK=false;
						break;
					}
				}
			}
		}
		return OK;	
	}
	
	/**
	*export the key
	*/
	private boolean exportKey(){
		File sel=null;
		KeyStore newKeystore=null;
		Key key=null;
		SelectorDialog sd=ResourceManager.getSelectorDialog(null,null);
		sd.setFileSelectionMode(SelectorDialog.FILES_ONLY);
		sd.setTitle("Select a file to export the key to");
		if (sd.showDialog() == SelectorDialog.APPROVE_OPTION) {
			sel = sd.getSelectedFile();
		}
		else return false;
		
		KeyStoreManager ksm=ResourceManager.getKeystoreManager();
		
		alias=aliasField.getText();
		password=new String(passwordField.getPassword());
		
		//choose new keystore format & provider
		String format= pkcsBtn.isSelected()==true ? "PKCS12" : "JKS";
		String provider= pkcsBtn.isSelected()==true ? "BC" : "SUN";
		logger.info("Creating new keystore in "+format+" format.");
		try{	
			newKeystore=KeyStore.getInstance(format,provider);
			key=ksm.getKeyEntry(alias, password.toCharArray());
			if(key==null) throw new NullPointerException("No such key entry in keystore!");
		}catch(NullPointerException e){e.printStackTrace();
			UserMessages.error("No such key found in keystore.\nPlease check the alias.");
			return false;
		}
		catch(Exception e){
			UserMessages.error("Could not open keystore. \nPlease check the password.");
		}
		
		try{
			X509Certificate[] certs=ksm.getCertificateByAlias(alias);
			newKeystore.load(null, null);
			newKeystore.setKeyEntry(alias, key, exportPwdField.getPassword(),certs);
			newKeystore.store(new FileOutputStream(sel),exportPwdField.getPassword());
			UserMessages.info("Key was successfully exported to \n"+sel.getAbsolutePath());
			return true;
			
		}catch(IOException e){
			UserMessages.error("Could not write new keystore file.\nReason: "+e.getMessage());
			return false;
		}
		catch(Exception e){e.printStackTrace();
			UserMessages.error("Error handling keystore.\nReason: "+e.getMessage());
			return false;
		}
		
	}
	
	//--helper
	private void addToGridBag(
			JLabel label,
			JComponent component,
			GridBagConstraints constraints,
			GridBagLayout gridbag,
			JPanel panel) {
		constraints.anchor = GridBagConstraints.WEST;
		constraints.gridheight=1; //(re)set to default, since this constraints object is being reused!
		constraints.gridwidth = GridBagConstraints.RELATIVE;
		constraints.weightx=0;
		gridbag.setConstraints(label, constraints);
		panel.add(label);
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.EAST;
		// ------------------------------------------------
		constraints.weightx = 0.8;
		// ------------------------------------------------
		gridbag.setConstraints(component, constraints);
		panel.add(component);
	}
	
	private void addSingleToGridBag(
			
			JComponent component,
			GridBagConstraints constraints,
			GridBagLayout gridbag,
			JPanel panel) {
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill=GridBagConstraints.HORIZONTAL;
		constraints.gridheight=1; //(re)set to default, since this constraints object is being reused!
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.EAST;
		constraints.weightx = 1;
		gridbag.setConstraints(component, constraints);
		panel.add(component);
	}

	// Variables declaration 
	
	private javax.swing.JPanel mainPanel;
	
	
	// End of variables declaration
	
	

	
}
