/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;

/**
 * Dialog to enter a password.
 * 
 * @author Ralf Ratering
 * @version $Id: PasswordDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class PasswordDialog extends GenericDialog {

	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 * Minimal main
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();
		PasswordDialog dialog = new PasswordDialog(frame,
				"TEST: PasswordDialog");
		dialog.show();
	}
	private char[] password;
	private JPasswordField passwordField = new JPasswordField(15);

	/**
	 * Constructor builds gui and sets a title
	 * 
	 * @param title
	 *            dialog title
	 * @param parent
	 *            dialog parent - important for modal dialogs
	 */
	public PasswordDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);
		buildComponents();
		pack();
	}

	/**
	 * Takes current textfield entry as password
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean applyValues() {
		password = passwordField.getPassword();
		return true;
	}

	/**
	 * Add a password field to the parent dialog
	 */
	private void buildComponents() {
		JLabel passwordLabel = new JLabel(res.getString("ENTER_PASSWORD"));
		JPanel passwordPanel = new JPanel();
		passwordPanel.add(passwordLabel);
		passwordPanel.add(passwordField);
		passwordPanel.setBorder(new EmptyBorder(20, 5, 10, 5));
		getContentPane().add(passwordPanel, BorderLayout.CENTER);
	}

	/**
	 * Sets password to null
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		password = null;
		return true;
	}

	/**
	 * Get the result from the dialog
	 * 
	 * @return null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getPassword() {
		return password;
	}

	/**
	 * Put focus into password field
	 */
	public void show() {
		// focus request does not work for windows, see jdk bug 4199374
		passwordField.requestFocus();
		super.show();
	}

	/**
	 * Display it...
	 *  
	 */
	public int showDialog() {
		passwordField.requestFocus();
		return super.showDialog();
	}
}