/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import com.pallas.unicore.client.panels.ResourceEditPanel;
import com.pallas.unicore.extensions.NamedResourceSet;

/**
 * Dialog that displays site resources for a vsite
 * 
 * @author Ralf Ratering
 * @version $Id: ResourceEditDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ResourceEditDialog extends GenericDialog {

	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	private ResourceEditPanel editPanel;

	public ResourceEditDialog(JFrame owner, NamedResourceSet request,
			NamedResourceSet siteResources) {
		super(owner, "", true, OK_CANCEL);
		editPanel = new ResourceEditPanel(request, siteResources);
		JScrollPane scrollPane = new JScrollPane(editPanel);
		scrollPane.scrollRectToVisible(new Rectangle(new Point(0, 0)));
		// add to main panel
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		setStatusMessage("Editing resource set " + request.getName());
		String title = "UNICORE: Resource Editor <" + request.getName() + ">";
		if (siteResources.getVsite() != null) {
			title += " at site " + siteResources.getVsite();
		}
		setTitle(title);
		pack();
	}

	protected boolean applyValues() {
		editPanel.applyValues();
		return true;
	}

	protected boolean cancelValues() {
		return true;
	}

	/**
	 * Overwrite show() method to position dialog
	 */
	public void show() {
		setSize(800, 600);
		setLocationRelativeTo(getParent());
		super.show();
	}
}