/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Dialog that displays logging information
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: ShowLogDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ShowLogDialog extends JDialog {

	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.dialogs");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	private JButton closeButton = new JButton(res.getString("CLOSE"));
	private String log;
	private JButton saveButton = new JButton(res.getString("SAVE"));
	private JTextArea textArea = new JTextArea();

	public ShowLogDialog(JFrame parentFrame, String log, String title) {
		// build a modal dialog
		super(parentFrame, title, false);
		this.log = log;
		buildComponents();
		pack();
	}

	private void buildComponents() {
		textArea.setLineWrap(false);
		textArea.setText(log);
		JScrollPane scrollPane = new JScrollPane(textArea,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.scrollRectToVisible(new Rectangle(new Point(0, 0)));
		ButtonListener buttonListener = new ButtonListener();
		closeButton.addActionListener(buttonListener);
		saveButton.addActionListener(buttonListener);
		JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 20, 20));
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);
		JPanel southPanel = new JPanel();
		southPanel.add(buttonPanel);
		getContentPane().add(southPanel, "South");
		getContentPane().add(scrollPane, "Center");
	}

	/**
	 * Display file chooser and write logging info to file
	 */
	private void saveLog() {
		String logDir = ResourceManager.getUserDefaults().getLogDirectory();
		SelectorDialog d = ResourceManager.getSelectorDialog(new File(logDir),
				null);
		d.setTitle("UNICORE: Save text information");
		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			PrintWriter pw;
			try {
				pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
				pw.println(log);
				pw.close();
			}
			catch (IOException ioe) {
				logger.log(Level.SEVERE, "I/O error", ioe);
			}
		}
	}

	public void setEditable(boolean editable) {
		textArea.setEditable(editable);
	}

	public void setTextFont(Font font) {
		textArea.setFont(font);
	}

	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			Object source = evt.getSource();
			if (source == closeButton) {
				dispose();
			}
			else if (source == saveButton) {
				saveLog();
			}
		}
	}
}