/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;

import javax.swing.JFrame;

/**
 *  Dialog to enter URL in a textfield and check it syntactically
 *
 *@author     Ralf Ratering
 *@version    $Id: URLDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class URLDialog extends TextfieldDialog {
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		URLDialog dialog =
			new URLDialog(frame, "Enter Url: ", "TEST: URLDialog", true);
		dialog.show();

	}
	private URL url;

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title      dialog title
	 *@param  parent     dialog parent - important for modal dialogs
	 *@param  modal      true if dialog is modal
	 *@param  labelText  Description of Parameter
	 */
	public URLDialog(
		JFrame parent,
		String labelText,
		String title,
		boolean modal) {
		super(parent, labelText, title, modal);
	}

	/**
	 *  Takes current textfield entry and tries to build a URL out of it
	 *
	 *@return    true if URL was syntactically correct
	 */
	protected boolean applyValues() {
		try {
			url = new URL(getText());
		} catch (MalformedURLException e) {
			setStatusMessage(res.getString("SYNTAX_ERROR"));
			url = null;
			return false;
		}
		return true;
	}

	/**
	 *  Constantly check URL for syntax
	 */
	protected void caretMoved() {
		try {
			URL test = new URL(getText());
			setStatusMessage("URL ok.");
			okButton.setEnabled(true);
			getRootPane().setDefaultButton(okButton);
		} catch (MalformedURLException e) {
			setStatusMessage(res.getString("SYNTAX_ERROR"));
			okButton.setEnabled(false);
		}
	}

	/**
	 *  Get the resulting URL from the dialog
	 *
	 *@return    null if cancel was pressed and a URL if ok was pressed
	 */
	public URL getURL() {
		return url;
	}
}
