/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.editor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import com.pallas.unicore.client.controls.GenericFileFilter;
import com.pallas.unicore.client.dialogs.CSRDialog;
import com.pallas.unicore.client.dialogs.ExportKeyDialog;
import com.pallas.unicore.client.dialogs.ChangeAliasDialog;
import com.pallas.unicore.client.dialogs.ChangePasswordDialog;
import com.pallas.unicore.client.dialogs.DefinePasswordDialog;
import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.dialogs.KeyStoreDetailsDialog;
import com.pallas.unicore.client.dialogs.PasswordDialog;
import com.pallas.unicore.client.dialogs.ServiceURLDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.util.DistinguishedNameFields;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.resourcemanager.ServiceManager;
import com.pallas.unicore.security.CertificateUtility;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.utility.Base64;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorComparator;

/**
 * @author Thomas Kentemich
 * @author Kirsten Foerster
 * @author Ralf Ratering
 * @version $Id: KeyStoreEditor.java,v 1.2 2004/06/23 13:41:58 bschuller Exp $
 */
public class KeyStoreEditor extends JPanel {

	private final static String DEFAULT_STRING = "(Default)";
	public final static String IDENTITY_CHANGED_EVENT = "IDENTITY_CHANGED",
			PLUGINS_OUTDATED_EVENT = "PLUGINS_OUTDATED_EVENT",
			CACHE_OUTDATED_EVENT = "CACHE_OUTDATED_EVENT";
	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.editor");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.editor.ResourceStrings");
	// CAUTION: Change this if test grid CA changes...
	public final static String TEST_CA = "Pallas test CA certificate";
	private JMenuItem addCertificateItem;
	private JMenuItem addTestCertificateItem;
	private ButtonListener buttonListener;
	// certificate list stuff
	private Vector caCertificates;
	private JList caCertList;
	private DefaultListModel caCertModel;
	private JButton caDeleteButton;
	// some buttons
	private JButton caDetailsButton;
	private JPanel certificatePanel;
	private JButton checkCertificateButton;
	private JButton checkCertificateButton2;
	private JButton defaultIdentityButton;
	private GenericFileFilter derFilter = new GenericFileFilter(".der",
			"Certificates in DER format");
	private JFrame detailsFrame;
	private JMenuItem exitItem;
	private JButton exportPublicKeyButton;
	/**
	 * File Chooser
	 */
	private SelectorDialog fileChooser;
	private JMenuItem generateCSRItem;
	private JMenuItem exportKeyItem;
	private JButton changeAliasButton;
	private JPanel keyPanel;
	private KeyStoreManager keystoreManager;
	private String keyStoreName = "UNKNOWN";
	private String keyStoreType = "UNKNOWN";
	private JMenuItem mergeKeystoreItem;
	private boolean newKeystore = false;
	private boolean removedKey = false;
	private String oldDefaultAlias, defaultAlias;
	private Vector oldKeys, oldTrusted, oldRejected;
	private GenericFileFilter pemFilter = new GenericFileFilter(".pem",
			"Certificates in PEM format");
	private GenericFileFilter pkcs12Filter = new GenericFileFilter(".p12",
			"Keystores in PKCS12 format");
	private Vector rejectedCertificates;
	private JList rejectedCertList;
	private DefaultListModel rejectedCertModel;
	private JButton rejectedDeleteButton;
	private JButton rejectedDetailsButton;
	private JPanel rejectedPanel;
	// file menu entries
	private JMenuItem saveItem;
	private JMenuItem setPasswordItem;
	private StatusBar statusBar = new StatusBar();
	private boolean testKeyEntryExists = false;
	private Vector userCertificates;
	private JList userCertList;
	private DefaultListModel userCertModel;
	private JButton userDeleteButton;
	private JButton userDetailsButton;

	public KeyStoreEditor() {
		this(null);
	}

	/**
	 * Constructor for the KeyStoreEditor object
	 * 
	 * @deprecated the client contains only one keystore manager belonging to
	 *             the single keystore
	 */
	public KeyStoreEditor(KeyStoreManager keystoreManager) {
		if (keystoreManager == null) {
			ResourceManager.initKeystoreManager(ResourceManager
					.getUserDefaults().getKeyStoreName());
			keystoreManager = ResourceManager.getKeystoreManager();
		}
		this.keystoreManager = keystoreManager;
		rememberOldEntries();
		buildComponents();
		updateValues();
	}

	private void addTrustedCertificate() {
		initFileChooser();
		fileChooser.setTitle("UNICORE: Import a certificate");
		fileChooser.removeFileFilters();
		fileChooser.addFileFilter(derFilter);
		fileChooser.addFileFilter(pemFilter);
		fileChooser.setSelectedFileFilter(pemFilter);
		int result = fileChooser.showDialog();
		// clean up filters
		fileChooser.removeFileFilters();
		if (result != JFileChooser.APPROVE_OPTION) {
			return;
		}
		final File certFile = fileChooser.getSelectedFile();
		if (certFile.exists()) {
			statusBar.setStatusMessage(res.getString("IMPORTING_CERT"));
			statusBar.startProgressBar();
			Thread defaultRunner = new Thread() {

				public void run() {
					try {
						X509Certificate cert = (X509Certificate) CertificateUtility
								.importTrustedCertifcate(certFile);
						logger.info("Importing cert for: "
								+ ((X509Certificate) cert).getSubjectDN()
										.getName());
						X509Certificate oriCert = keystoreManager
								.getCertificateByKey(cert.getPublicKey());
						if (oriCert == null) {
							logger.info("Adding new trusted certificate.");
							keystoreManager.addTrustedCertificate(cert, false);
						}
						else {
							String alias = keystoreManager
									.getAliasFromCertifcate(oriCert);
							if (keystoreManager.isKeyEntry(alias)) {
								logger.info("Replacing key certificate: "
										+ alias);
								keystoreManager.replaceKeyCertificate(alias,
										cert, false);
								Vector keyEntries = keystoreManager
										.getCachedKeyEntries();
								if (keyEntries.size() == 1) {
									defaultAlias = alias;
								}
								else {
									if (JOptionPane
											.showConfirmDialog(
													getFrame(),
													alias
															+ res
																	.getString("INSTALLED_AS_DEFAULT"),
													res
															.getString("KEYSTORE_EDITOR_TITLE"),
													JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
										defaultAlias = alias;
									}
								}
							}
							else {
								logger.info("Replacing trusted certifcate: "
										+ alias);
								keystoreManager.addTrustedCertificate(cert,
										false);
							}
						}
					}
					catch (Exception e) {
						statusBar.stopProgressBar();
						statusBar.setStatusMessage("[" + keyStoreName + "] "
								+ keyStoreType);
						logger.log(Level.SEVERE, "", e);
						UserMessages.error(res.getString("FAILED_TO_LOAD_CERT")
								+ certFile, e.getLocalizedMessage());
					}
					ResourceManager.getConnectionManager()
							.removeAllConnections();
					updateValues();
					statusBar.stopProgressBar();
					statusBar.setStatusMessage("[" + keyStoreName + "] "
							+ keyStoreType);
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				}
			};
			defaultRunner.start();
		}
	}

	/**
	 * Build gui elements
	 */
	private void buildComponents() {
		JTabbedPane tabbedPane = new JTabbedPane();
		buttonListener = new ButtonListener();
		buildKeyPanel();
		buildTrustedPanel();
		buildRejectedPanel();
		tabbedPane.add(keyPanel, res.getString("KEY_ENTRIES"));
		tabbedPane.add(certificatePanel, res.getString("TRUSTED_CERT"));
		tabbedPane.add(rejectedPanel, res.getString("REJECTED_CERT"));
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEtchedBorder());
		add(createMenubar(), BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);
		add(statusBar, BorderLayout.SOUTH);
		statusBar.setStatusMessage("No keystore loaded.");
	}

	/**
	 * Define a new password and build an initially empty keystore.
	 * 
	 * @return true if building keystore succeeded
	 */
	private boolean buildEmptyKeystore(String provider, String type) {
		// define new password
		DefinePasswordDialog definePasswordDialog = new DefinePasswordDialog(
				(JFrame) getFrame(), res.getString("PASSWORD_TITLE"));
		definePasswordDialog.setStatusMessage(res.getString("DEFINE PASSWORD"));
		definePasswordDialog.show();
		char[] passwd = definePasswordDialog.getPassword();
		try {
			if (passwd == null) {
				throw new Exception(
						"Building initial keystore aborted by user.");
			}
			keystoreManager.buildEmptyKeystore(keyStoreName, passwd, passwd);
			passwd = null;
			definePasswordDialog = null;
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
			UserMessages.warning(res.getString("NO_KEYSTORE_BUILT"), e
					.getMessage());
			return false;
		}
		return true;
	}

	private void buildKeyPanel() {
		keyPanel = new JPanel(new BorderLayout());
		// User Certificate
		userCertModel = new DefaultListModel();
		userCertList = new JList(userCertModel);
		userCertList.addListSelectionListener(new ListListener());
		userCertList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		userCertList.setVisibleRowCount(5);
		JScrollPane userCertScrollPane = new JScrollPane(userCertList);
		userCertScrollPane.setViewportView(userCertList);
		//  User certificate
		userDeleteButton = new JButton(res.getString("REMOVE"));
		userDeleteButton.addActionListener(buttonListener);
		userDeleteButton.setToolTipText(res.getString("REMOVE_TIP"));
		userDetailsButton = new JButton(res.getString("DETAILS"));
		userDetailsButton.addActionListener(buttonListener);
		userDetailsButton.setToolTipText(res.getString("SHOW_TIP"));
		defaultIdentityButton = new JButton(res.getString("SET_DEFAULT"));
		defaultIdentityButton.setToolTipText(res.getString("SET_DEFAULT_TIP"));
		defaultIdentityButton.addActionListener(buttonListener);
		exportPublicKeyButton = new JButton(res.getString("EXPORT_PUBLIC"));
		exportPublicKeyButton.addActionListener(buttonListener);
		exportPublicKeyButton
				.setToolTipText(res.getString("EXPORT_PUBLIC_TIP"));
		checkCertificateButton = new JButton(res.getString("CHECK_CRL"));
		checkCertificateButton.addActionListener(buttonListener);
		checkCertificateButton.setToolTipText(res.getString("CHECK_CRL_TIP"));
		changeAliasButton = new JButton(res.getString("CHANGE_ALIAS"));
		changeAliasButton.addActionListener(buttonListener);
		changeAliasButton.setToolTipText(res.getString("CHANGE_ALIAS_TIP"));
		JPanel userButtonPanel = new JPanel(new GridLayout(3, 2, 10, 10));
		userButtonPanel.add(userDeleteButton);
		userButtonPanel.add(userDetailsButton);
		userButtonPanel.add(defaultIdentityButton);
		userButtonPanel.add(checkCertificateButton);
		userButtonPanel.add(exportPublicKeyButton);
		userButtonPanel.add(changeAliasButton);
		keyPanel.add(userCertScrollPane, BorderLayout.CENTER);
		keyPanel.add(userButtonPanel, BorderLayout.SOUTH);
	}

	void buildRejectedPanel() {
		rejectedPanel = new JPanel(new BorderLayout());
		// CA Certificate
		rejectedCertModel = new DefaultListModel();
		rejectedCertList = new JList(rejectedCertModel);
		rejectedCertList.addListSelectionListener(new ListListener());
		rejectedCertList.setVisibleRowCount(5);
		rejectedCertList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		JScrollPane rejectedCertScrollPane = new JScrollPane(rejectedCertList);
		rejectedCertScrollPane.setViewportView(rejectedCertList);
		rejectedDetailsButton = new JButton(res.getString("DETAILS"));
		rejectedDetailsButton.addActionListener(buttonListener);
		rejectedDetailsButton.setToolTipText(res.getString("DETAILS_TIP"));
		rejectedDeleteButton = new JButton(res.getString("REMOVE"));
		rejectedDeleteButton.addActionListener(buttonListener);
		rejectedDeleteButton.setToolTipText(res.getString("REMOVE_CERT"));
		JPanel rejectedButtonPanel = new JPanel(new GridLayout(2, 2, 10, 10));
		rejectedButtonPanel.add(rejectedDetailsButton);
		rejectedButtonPanel.add(rejectedDeleteButton);
		//        rejectedPanel.setBorder(new
		// TitledBorder(res.getString("TRUSTED_CERT")));
		rejectedPanel.add(rejectedCertScrollPane, BorderLayout.CENTER);
		rejectedPanel.add(rejectedButtonPanel, BorderLayout.SOUTH);
	}

	void buildTrustedPanel() {
		certificatePanel = new JPanel(new BorderLayout());
		// CA Certificate
		caCertModel = new DefaultListModel();
		caCertList = new JList(caCertModel);
		caCertList.addListSelectionListener(new ListListener());
		caCertList.setVisibleRowCount(5);
		caCertList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		JScrollPane caCertScrollPane = new JScrollPane(caCertList);
		caCertScrollPane.setViewportView(caCertList);
		caDetailsButton = new JButton(res.getString("DETAILS"));
		caDetailsButton.addActionListener(buttonListener);
		caDetailsButton.setToolTipText(res.getString("DETAILS_TIP"));
		caDeleteButton = new JButton(res.getString("REMOVE"));
		caDeleteButton.addActionListener(buttonListener);
		caDeleteButton.setToolTipText(res.getString("REMOVE_CERT"));
		checkCertificateButton2 = new JButton(res.getString("CHECK_CRL"));
		checkCertificateButton2.addActionListener(buttonListener);
		checkCertificateButton2.setToolTipText(res.getString("CHECK_CRL_TIP"));
		JPanel caButtonPanel = new JPanel(new GridLayout(2, 2, 10, 10));
		caButtonPanel.add(caDeleteButton);
		caButtonPanel.add(caDetailsButton);
		caButtonPanel.add(checkCertificateButton2);
		//       certificatePanel.setBorder(new
		// TitledBorder(res.getString("TRUSTED_CERT")));
		certificatePanel.add(caCertScrollPane, BorderLayout.CENTER);
		certificatePanel.add(caButtonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Display a dialog to change the keystore password
	 */
	private void changeKeystorePassword() {
		ChangePasswordDialog dialog = new ChangePasswordDialog(
				(JFrame) getFrame(), res.getString("CHANGE_PASSWORD_TITLE"));
		dialog.show();
		if (dialog.getNewPassword() != null && dialog.getOldPassword() != null) {
			try {
				keystoreManager.changePassword(dialog.getOldPassword(), dialog
						.getNewPassword());
			}
			catch (Exception e) {
				UserMessages.error(res.getString("COULD_NOT_CHANGE_PASSWD"), e
						.getLocalizedMessage());
			}
		}
		dialog = null;
	}

	/**
	 * Create the menubar for the app. By default this pulls the definition of
	 * the menu from the associated resource file.
	 */
	protected JMenuBar createMenubar() {
		// file menu
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		saveItem = new JMenuItem("Save", 'S');
		initMenuItem(saveItem, new SaveAction(), KeyEvent.VK_S);
		fileMenu.add(saveItem);
		fileMenu.addSeparator();
		exitItem = new JMenuItem("Exit", 'X');
		ExitAction exitAction = new ExitAction();
		initMenuItem(exitItem, exitAction, KeyEvent.VK_X);
		fileMenu.add(exitItem);
		JMenu actionMenu = new JMenu("Actions");
		actionMenu.setMnemonic('A');
		mergeKeystoreItem = new JMenuItem("Import Keystore...", 'M');
		initMenuItem(mergeKeystoreItem, new MergeAction(), KeyEvent.VK_M);
		mergeKeystoreItem
				.setToolTipText("Merge contents of another keystore (PKCS12/JKS/JCEKS)");
		actionMenu.add(mergeKeystoreItem);
		addCertificateItem = new JMenuItem("Import Certificate...", 'I');
		initMenuItem(addCertificateItem, new AddCertificateAction(),
				KeyEvent.VK_I);
		addCertificateItem
				.setToolTipText("Import a certificate in PEM or DER format");
		actionMenu.add(addCertificateItem);
		addTestCertificateItem = new JMenuItem("Import Test Certificates...",
				'T');
		initMenuItem(addTestCertificateItem, new AddTestCertificateAction(),
				KeyEvent.VK_T);
		addTestCertificateItem.setToolTipText("Import TestGrid Certificates");
		actionMenu.add(addTestCertificateItem);
		actionMenu.addSeparator();
		generateCSRItem = new JMenuItem("Generate Certification Request...",
				'C');
		initMenuItem(generateCSRItem, new CSRAction(), KeyEvent.VK_C);
		generateCSRItem.setToolTipText("Generate a certification request");
		actionMenu.add(generateCSRItem);
		
		exportKeyItem = new JMenuItem("Export key entry...",
		'C');
		initMenuItem(exportKeyItem, new ExportKeyAction(), KeyEvent.VK_K);
		exportKeyItem.setToolTipText("Export a key to a new keystore in PKCS12 or JKS format");
		actionMenu.add(exportKeyItem);
		
		actionMenu.addSeparator();
		setPasswordItem = new JMenuItem("Set Password...", 'P');
		initMenuItem(setPasswordItem, new PasswordAction(), KeyEvent.VK_P);
		actionMenu.add(setPasswordItem);
		JMenuBar menubar = new JMenuBar();
		menubar.add(fileMenu);
		menubar.add(actionMenu);
		return menubar;
	}

	private boolean defaultAliasChanged() {
		return !oldDefaultAlias.equals(defaultAlias);
	}

	private void deleteSelectedCertificate(JList list) {
		Object[] values = list.getSelectedValues();
		if (values.length == 0) {
			return;
		}
		boolean removeDefault = false;
		for (int i = 0; i < values.length; i++) {
			String alias = (String) values[i];
			if (alias.indexOf(DEFAULT_STRING) != -1) {
				alias = alias.substring(0, alias.indexOf(DEFAULT_STRING));
				this.defaultAlias = "";
				removeDefault = true;
			}
			try {
				if (keystoreManager.isKeyEntry(alias)) {
					removedKey = true;
				}
				keystoreManager.removeEntry(alias);
			}
			catch (Exception e) {
				logger
						.log(Level.WARNING, "Could not remove alias: " + alias,
								e);
			}
		}
		updateValues();
		Vector keyEntries = keystoreManager.getCachedKeyEntries();
		if (removeDefault && keyEntries.size() == 1) {
			UserMessages.info(res.getString("SINGLE_ENTRY_AS_DEFAULT"));
			defaultAlias = (String) keyEntries.elementAt(0);
			removeDefault = false;
			updateValues();
		}
		if (removeDefault) {
			UserMessages.warning(res.getString("DEFAULT_IDENTITY_UNKNOWN"));
		}
	}

	/**
	 * Changes the alias name of the selected entry
	 */
	private void changeAlias() {
		Object[] values = userCertList.getSelectedValues();
		if (values.length == 0) {
			return;
		}
		boolean changeDefault = false;
		String newAlias = "";
		for (int i = 0; i < values.length; i++) {
			String alias = (String) values[i];
			if (alias.indexOf(DEFAULT_STRING) != -1) {
				alias = alias.substring(0, alias.indexOf(DEFAULT_STRING));
				this.defaultAlias = "";
				changeDefault = true;
			}
			ChangeAliasDialog dialog = new ChangeAliasDialog(
					(JFrame) getFrame(), "Change Alias", alias);
			dialog.show();
			if (dialog.getAlias() != null) {
				newAlias = dialog.getAlias();
				try {
					newAlias = keystoreManager.changeAlias(alias, newAlias);
					removedKey = true;
				}
				catch (Exception e) {
					UserMessages.error("Could not remove alias: " + alias, e
							.getLocalizedMessage());
					logger.log(Level.WARNING, "Could not remove alias: "
							+ alias, e);
				}
			}
			dialog = null;
		}
		Vector keyEntries = keystoreManager.getCachedKeyEntries();
		if (changeDefault) {
			// default alias changed
			setAsDefault(newAlias);
		}
		updateValues();
	}

	public void exitKeystore() {
		if (defaultAliasChanged() || keystoreChanged() || removedKey) {
			if (JOptionPane.showConfirmDialog(getFrame(),
					"Save modified Keystore?", "Keystore Editor: Exit",
					JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				save();
			}
			else {
				// reload keystore
				try {
					keystoreManager.reloadKeystore();
					ResourceManager.getUserDefaults().setDefaultIdentity(
							oldDefaultAlias);
				}
				catch (Exception e) {
					logger.log(Level.WARNING, "Reloading old keystore failed.",
							e);
				}
			}
			removedKey = false;
		}
		for (Container p = getParent(); p != null; p = p.getParent()) {
			if (p instanceof JDialog) {
				((JDialog) p).dispose();
				break;
			}
		}
	}

	private void exportPublicKey() {
		String alias = (String) userCertList.getSelectedValue();
		if (alias == null) {
			return;
		}
		setStatusMessage(res.getString("EXPORTING_KEY"));
		initFileChooser();
		fileChooser.removeFileFilters();
		fileChooser.addFileFilter(derFilter);
		fileChooser.addFileFilter(pemFilter);
		fileChooser.setSelectedFileFilter(pemFilter);
		int returnVal = fileChooser.showDialog();
		if (returnVal != JFileChooser.APPROVE_OPTION) {
			fileChooser.removeFileFilters();
			return;
		}
		try {
			String name = fileChooser.getSelectedFile().getCanonicalPath();
			// build .der ending
			if (!name.endsWith(".pem")) {
				name = name + ".pem";
			}
			if (alias.equals(this.defaultAlias + DEFAULT_STRING)) {
				alias = this.defaultAlias;
			}
			// export the public key
			keystoreManager.exportUserCert(alias, name);
		}
		catch (Exception e) {
			UserMessages.error(res.getString("COULD_NOT_EXPORT_CERT") + alias,
					e.getMessage());
		}
		fileChooser.removeFileFilters();
	}
	
	//export public+private keypair to new PKCS12 or JKS keystore
	private void exportKey(){ 
		ExportKeyDialog ekd=new ExportKeyDialog((JFrame)getFrame(), "Export a key entry");
		ekd.show();
	}
	
	private void generateCSR() {
		Vector keyEntries = new Vector(keystoreManager.getCachedKeyEntries());
		CSRDialog csrDialog = new CSRDialog((JFrame) getFrame(), res
				.getString("CSR_DIALOG_TITLE"));
		csrDialog.show();
		final String dnString = csrDialog.getDNString();
		if (dnString != null) {
			statusBar.setStatusMessage("Generating certification request...");
			statusBar.startProgressBar();
			try {
				byte[] encodedCSR = keystoreManager.generateCSR(dnString, 1024);
				updateValues();
				if (encodedCSR == null) {
					UserMessages.error(res.getString("COULD_NOT_EXPORT_CERT"));
					setStatusMessage("Exporting CSR");
					return;
				}
				setStatusMessage("Exporting CSR");
				initFileChooser();
				int returnVal = fileChooser.showDialog();
				if (returnVal != JFileChooser.APPROVE_OPTION) {
					statusBar.stopProgressBar();
					statusBar.setStatusMessage("[" + keyStoreName + "] "
							+ keyStoreType);
					return;
				}
				try {
					String name = fileChooser.getSelectedFile()
							.getCanonicalPath();
					FileOutputStream fos = new FileOutputStream(name);
					String certStart = "-----BEGIN CERTIFICATE REQUEST-----\n";
					String certEnd = "\n-----END CERTIFICATE REQUEST-----\n";
					fos.write(certStart.getBytes());
					fos.write(Base64.encode(encodedCSR, true));
					fos.write(certEnd.getBytes());
					fos.close();
					setStatusMessage("OK.");
				}
				catch (Exception e) {
					statusBar.stopProgressBar();
					statusBar.setStatusMessage("[" + keyStoreName + "] "
							+ keyStoreType);
					UserMessages.error(res.getString("COULD_NOT_EXPORT_CSR")
							+ e.getMessage());
				}
			}
			catch (Exception e) {
				statusBar.stopProgressBar();
				statusBar.setStatusMessage("[" + keyStoreName + "] "
						+ keyStoreType);
				UserMessages.error(res
						.getString("COULD_NOT_GENERATE_CERT_REQUEST"), e
						.getMessage());
			}
			statusBar.stopProgressBar();
			statusBar
					.setStatusMessage("[" + keyStoreName + "] " + keyStoreType);
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		else {
			System.err.println("Cancel!");
			setStatusMessage("OK");
			return;
		}
		VectorComparator newKeyEntries = new VectorComparator(keystoreManager
				.getCachedKeyEntries());
		Vector csr = newKeyEntries.excludes(keyEntries);
		UserMessages.info((String) csr.elementAt(0)
				+ res.getString("IS_SAVED_TO_KEYSTORE"));
		if (newKeyEntries.size() == 1) {
			this.defaultAlias = (String) keyEntries.elementAt(0);
			updateValues();
			return;
		}
		updateValues();
		String alias = (String) csr.elementAt(0);
		if (JOptionPane.showConfirmDialog(this.getFrame(), alias
				+ res.getString("INSTALLED_AS_DEFAULT"), res
				.getString("KEYSTORE_EDITOR_TITLE"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
			this.defaultAlias = alias;
			updateValues();
		}
	}

	/**
	 * Find the hosting frame, for the file-chooser dialog.
	 */
	protected Frame getFrame() {
		for (Container p = getParent(); p != null; p = p.getParent()) {
			if (p instanceof Frame) {
				return (Frame) p;
			}
		}
		return null;
	}

	private void importTestCert() {
		Certificate[] caCerts = null;
		String userAlias = null;
		try {
			ServiceURLDialog urlDialog = null;
			urlDialog = new ServiceURLDialog((JFrame) getFrame(), "URL:",
					"Enter URL to CA web service", true);
			urlDialog
					.setText("http://testgrid.unicorepro.com:5900/unicore/services/CAService?WSDL");
			urlDialog.show();
			if (urlDialog.getText() == null) {
				return;
			}
			caCerts = ServiceManager.getCACertificate(urlDialog.getText());
			X509Certificate userCert = (X509Certificate) ServiceManager
					.getSignedRequest(urlDialog.getText());
			if (userCert == null) {
				return;
			}
			userAlias = CertificateUtility.getCommonName(userCert);
			for (int i = 0; i < caCerts.length; i++) {
				keystoreManager.addTrustedCertificate(caCerts[i], false);
			}
			userAlias = keystoreManager.replaceKeyCertificate(userAlias,
					userCert, false);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					"Error in communication with Certificate Service. ", e);
			UserMessages.warning(res.getString("CERT_SERVICE_FAILED"));
		}
		Vector keyEntries = keystoreManager.getCachedKeyEntries();
		if (keyEntries.size() == 1) {
			defaultAlias = (String) keyEntries.elementAt(0);
		}
		updateValues();
		if (keyEntries.size() > 1) {
			if (JOptionPane.showConfirmDialog(this.getFrame(), userAlias
					+ res.getString("INSTALLED_AS_DEFAULT"), res
					.getString("KEYSTORE_EDITOR_TITLE"),
					JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				this.defaultAlias = userAlias;
				updateValues();
			}
		}
	}

	private void initFileChooser() {
		// only build the file chooser once...
		if (fileChooser == null) {
			fileChooser = ResourceManager.getSelectorDialog(null, null);
		}
	}

	/**
	 * Helper method initializes menu items
	 *  
	 */
	protected void initMenuItem(JMenuItem item, Action action) {
		item.addActionListener(action);
		action.addPropertyChangeListener(new ActionChangedListener(item));
		item.setEnabled(action.isEnabled());
	}

	/**
	 * Helper method initializing menu item including a CTRL-Shortcut
	 */
	protected void initMenuItem(JMenuItem item, Action action, int keyCode) {
		item.setAccelerator(KeyStroke.getKeyStroke(keyCode,
				ActionEvent.CTRL_MASK));
		initMenuItem(item, action);
	}

	private boolean keystoreChanged() {
		if (!oldKeys.equals(keystoreManager.getCachedKeyEntries())) {
			return true;
		}
		if (!oldTrusted.equals(keystoreManager.getCachedTrustedCertificates())) {
			return true;
		}
		if (!oldRejected.equals(keystoreManager.getRejectedEntries())) {
			return true;
		}
		return false;
	}

	private void mergeKeystore() {
		if (this.keystoreManager == null) {
			UserMessages.error(res.getString("NO_KEYSTORE_DEFINED"));
			return;
		}
		// open the keystore to merge
		String newKeystoreName = null;
		KeyStoreManager newKeystoreManager = null;
		boolean opened = false;
		// enter password dialog
		PasswordDialog pwdDlg = new PasswordDialog((JFrame) getFrame(),
				"Enter Password");
		statusBar.startProgressBar();
		initFileChooser();
		fileChooser.setTitle("UNICORE: Import Keystore");
		fileChooser.removeFileFilters();
		fileChooser.addFileFilter(pkcs12Filter);
		fileChooser.setSelectedFileFilter(pkcs12Filter);
		int result = fileChooser.showDialog();
		if (result != JFileChooser.APPROVE_OPTION) {
			fileChooser.removeFileFilters();
			statusBar.stopProgressBar();
			return;
		}
		File secondFile = fileChooser.getSelectedFile();
		fileChooser.removeFileFilters();
		if (secondFile.exists()) {
			int count = 3;
			try {
				newKeystoreName = secondFile.getCanonicalPath();
			}
			catch (IOException ioe) {
				statusBar.stopProgressBar();
				// should not happen
				logger.log(Level.SEVERE, "", ioe);
				return;
			}
			// three tries to enter correct password
			while (count > 0 && !opened) {
				switch (count) {
					case 3 :
						pwdDlg.setStatusMessage(res.getString("ENTER_PASSWORD")
								+ newKeystoreName);
						break;
					case 2 :
						pwdDlg.setStatusMessage(res
								.getString("WRONG_PASSWORD_2_TRIES"));
						break;
					case 1 :
						pwdDlg.setStatusMessage(res
								.getString("WRONG_PASSWORD_LAST_TRY"));
						break;
				}
				int value = pwdDlg.showDialog();
				if (value == GenericDialog.CANCEL_OPTION) {
					statusBar.stopProgressBar();
					return;
				}
				count--;
				try {
					newKeystoreManager = openKeystore(newKeystoreName, pwdDlg
							.getPassword());
				}
				catch (FileNotFoundException fnfe) {
					// something wrong fith the certificate file
					logger.log(Level.SEVERE, "", fnfe);
					UserMessages.error(
							res.getString("FAILED_TO_LOAD_KEYSTORE"), fnfe
									.getLocalizedMessage());
					opened = false;
					break;
				}
				catch (IOException ioe) {
					// wrong password!!! Why IOException? Grrrr.
					continue;
				}
				catch (Exception e) {
					// big trouble
					logger.log(Level.SEVERE, "", e);
					UserMessages.error(
							res.getString("FAILED_TO_LOAD_KEYSTORE"), e
									.getLocalizedMessage());
					opened = false;
					break;
				}
				if (newKeystoreManager != null) {
					// System.out.println("Opened keystore to merge.");
					opened = true;
				}
			}
		}
		if (!opened) {
			statusBar.stopProgressBar();
			UserMessages.error(res
					.getString("UNABLE_TO_OPEN_KEYSTORE_FOR_MERGING")
					+ newKeystoreName);
			return;
		}
		setStatusMessage(res.getString("WAIT_WHILE_UPDATING"));
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
		ImportUserCertThread importThread = new ImportUserCertThread(
				newKeystoreManager, pwdDlg.getPassword());
		importThread.start();
		updateValues();
		Vector keyEntries = keystoreManager.getCachedKeyEntries();
		Vector newKeyEntries = newKeystoreManager.getCachedKeyEntries();
		Vector mergeKeyEntries = new Vector();
		for (int i = 0; i < newKeyEntries.size(); i++) {
			Object entry = newKeyEntries.elementAt(i);
			if (!keyEntries.contains(entry)) {
				mergeKeyEntries.addElement(entry);
			}
		}
		if (keyEntries.size() >= 1 && newKeyEntries.size() >= 1) {
			if (mergeKeyEntries.size() == 1) {
				String alias = (String) newKeyEntries.elementAt(0);
				if (JOptionPane.showConfirmDialog(this.getFrame(), alias
						+ res.getString("INSTALLED_AS_DEFAULT"), res
						.getString("KEYSTORE_EDITOR_TITLE"),
						JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
					this.defaultAlias = alias;
					updateValues();
				}
			}
		}
		if (mergeKeyEntries.size() > 1) {
			String[] aliases = new String[mergeKeyEntries.size()];
			for (int i = 0; i < mergeKeyEntries.size(); i++) {
				aliases[i] = (String) mergeKeyEntries.elementAt(i);
			}
			String alias = (String) JOptionPane.showInputDialog(
					this.getFrame(), res
							.getString("INSTALLED_ONE_ENTRY_AS_DEFAULT"), res
							.getString("KEYSTORE_EDITOR_TITLE"),
					JOptionPane.QUESTION_MESSAGE, null, aliases, aliases[0]);
			if (alias != null) {
				this.defaultAlias = alias;
				updateValues();
			}
		}
		pwdDlg = null;
		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	private KeyStoreManager openKeystore(String keystoreName, char[] password)
			throws Exception {
		KeyStoreManager keystoreManager = null;
		boolean opened = false;
		if (password != null) {
			// if cancel was pressed password will be null -> return null
			try {
				keystoreManager = KeyStoreManager.getInstance(null, "JKS",
						keystoreName);
				keystoreManager.loadKeyStore(password, password);
				this.keyStoreType = "JKS";
			}
			catch (Exception firstException) {
				if (firstException.getMessage().startsWith("Keystore")) {
					// autsch!! wrong password
					return null;
				}
				try {
					keystoreManager = KeyStoreManager.getInstance("SunJCE",
							"JCEKS", keystoreName);
					keystoreManager.loadKeyStore(password, password);
					this.keyStoreType = "JCEKS";
				}
				catch (Exception secondException) {
					if (secondException.getMessage().startsWith("Keystore")) {
						// autsch!! wrong password
						return null;
					}
					// try PKCS12 format
					keystoreManager = KeyStoreManager.getInstance(null,
							"PKCS12", keystoreName);
					keystoreManager.loadKeyStore(password, password);
					this.keyStoreType = "PKCS12";
				}
			}
			opened = true;
		}
		if (!opened) {
			UserMessages.error(res.getString("FAILED_TO_LOAD_KEYSTORE"));
			return null;
		}
		return keystoreManager;
	}

	private void rememberOldEntries() {
		// rememember old default entry
		oldDefaultAlias = ResourceManager.getUserDefaults()
				.getDefaultIdentity();
		defaultAlias = oldDefaultAlias;
		oldKeys = (Vector) keystoreManager.getCachedKeyEntries().clone();
		oldTrusted = (Vector) keystoreManager.getCachedTrustedCertificates()
				.clone();
		oldRejected = (Vector) keystoreManager.getRejectedEntries().clone();
	}

	private void save() {
		try {
			keystoreManager.reWriteKeyStore();
			if (removedKey) {
				UserMessages
						.warning("If you have configured different identities for\naccessing sites, these identities may be no longer valid.\nRun Settings->Set Identity");
			}
			updateListeners();
		}
		catch (Exception e) {
			logger.log(Level.WARNING, "Keystore is not complete", e);
			UserMessages.error("Warning occured while saving keystore.\n"
					+ e.getMessage());
		}
	}

	private void setAsDefault(String alias) {
		if (alias.indexOf(DEFAULT_STRING) >= 0) {
			return;
		}
		for (int i = 0; i < userCertModel.getSize(); i++) {
			String entry = (String) userCertModel.elementAt(i);
			if (entry.equals(defaultAlias + DEFAULT_STRING)) {
				userCertModel.setElementAt(defaultAlias, i);
			}
			else if (entry.equals(alias)) {
				userCertModel.setElementAt(alias + DEFAULT_STRING, i);
			}
		}
		defaultAlias = alias;
	}

	private void setKeystoreName(String keyStoreName) {
		this.keyStoreName = keyStoreName;
	}

	protected void setStatusMessage(String message) {
		statusBar.setStatusMessage(message);
	}

	private void showSelectedCertificate(JList list) {
		String alias = (String) list.getSelectedValue();
		if (alias == null) {
			return;
		}
		if (alias.indexOf(DEFAULT_STRING) != -1) {
			// remove default part from string
			alias = alias.substring(0, alias.indexOf(DEFAULT_STRING));
		}
		try {
			X509Certificate[] x509 = keystoreManager
					.getCertificateByAlias(alias);
			if (x509 != null) {
				KeyStoreDetailsDialog dialog = new KeyStoreDetailsDialog(
						x509[0], this.getFrame(), alias, true);
				dialog.show();
			}
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "", ex);
			UserMessages.error(alias + res.getString("COULD_NOT_READ_CERT"), ex
					.getMessage());
		}
	}

	/**
	 * Enable and disable the buttons.
	 */
	private void updateButtons() {
		if (!keystoreManager.hasOpenKeystore()
				|| !ResourceManager.isKeystoreAvailable()) {
			caDeleteButton.setEnabled(false);
			caDetailsButton.setEnabled(false);
			rejectedDeleteButton.setEnabled(false);
			rejectedDetailsButton.setEnabled(false);
			checkCertificateButton2.setEnabled(false);
			userDetailsButton.setEnabled(false);
			defaultIdentityButton.setEnabled(false);
			checkCertificateButton.setEnabled(false);
			userDeleteButton.setEnabled(false);
			exportPublicKeyButton.setEnabled(false);
			changeAliasButton.setEnabled(false);
			return;
		}
		if (caCertList.getSelectedValue() == null) {
			caDeleteButton.setEnabled(false);
			caDetailsButton.setEnabled(false);
			checkCertificateButton2.setEnabled(false);
		}
		else {
			if (this.caCertList.getSelectedIndices().length == 1) {
				caDeleteButton.setEnabled(true);
				caDetailsButton.setEnabled(true);
				checkCertificateButton2.setEnabled(false);
			}
			else {
				caDeleteButton.setEnabled(true);
				caDetailsButton.setEnabled(false);
				checkCertificateButton2.setEnabled(false);
			}
		}
		if (rejectedCertList.getSelectedValue() == null) {
			rejectedDeleteButton.setEnabled(false);
			rejectedDetailsButton.setEnabled(false);
		}
		else {
			if (this.rejectedCertList.getSelectedIndices().length == 1) {
				rejectedDeleteButton.setEnabled(true);
				rejectedDetailsButton.setEnabled(true);
			}
			else {
				rejectedDeleteButton.setEnabled(true);
				rejectedDetailsButton.setEnabled(false);
			}
		}
		if (userCertList.getSelectedValue() == null) {
			userDetailsButton.setEnabled(false);
			defaultIdentityButton.setEnabled(false);
			checkCertificateButton.setEnabled(false);
			userDeleteButton.setEnabled(false);
			exportPublicKeyButton.setEnabled(false);
			changeAliasButton.setEnabled(false);
		}
		else {
			if (this.userCertList.getSelectedIndices().length == 1) {
				userDetailsButton.setEnabled(true);
				defaultIdentityButton.setEnabled(true);
				checkCertificateButton.setEnabled(false);
				userDeleteButton.setEnabled(true);
				exportPublicKeyButton.setEnabled(true);
			}
			else {
				userDetailsButton.setEnabled(false);
				defaultIdentityButton.setEnabled(false);
				checkCertificateButton.setEnabled(false);
				userDeleteButton.setEnabled(true);
				exportPublicKeyButton.setEnabled(false);
			}
			changeAliasButton.setEnabled(true);
		}
	}

	private void updateListeners() {
		boolean cacheUpdating = false;
		if (defaultAliasChanged()) {
			ResourceManager.getUserDefaults().setDefaultIdentity(defaultAlias);
			ResourceManager.getUserDefaults().writeToFile();
			oldDefaultAlias = defaultAlias;
			firePropertyChange(IDENTITY_CHANGED_EVENT, null, null);
			firePropertyChange(CACHE_OUTDATED_EVENT, null, null);
			cacheUpdating = true;
		}
		if (keystoreChanged()) {
			firePropertyChange(PLUGINS_OUTDATED_EVENT, null, null);
			// if cache was not updated because of identity change,
			// do it now
			if (!cacheUpdating) {
				firePropertyChange(CACHE_OUTDATED_EVENT, null, null);
			}
		}
		rememberOldEntries();
	}

	private void updateMenu() {
		if (!keystoreManager.hasOpenKeystore()
				|| !ResourceManager.isKeystoreAvailable()) {
			saveItem.setEnabled(false);
			mergeKeystoreItem.setEnabled(false);
			addCertificateItem.setEnabled(false);
			addTestCertificateItem.setEnabled(false);
			setPasswordItem.setEnabled(false);
			generateCSRItem.setEnabled(false);
		}
		else {
			saveItem.setEnabled(true);
			mergeKeystoreItem.setEnabled(true);
			addCertificateItem.setEnabled(true);
			setPasswordItem.setEnabled(true);
			generateCSRItem.setEnabled(true);
			testKeyEntryExists = false;
			for (int i = 0; i < userCertList.getModel().getSize(); i++) {
				String alias = (String) userCertList.getModel().getElementAt(i);
				if (alias.indexOf(DEFAULT_STRING) != -1) {
					// remove default part from string
					alias = alias.substring(0, alias.indexOf(DEFAULT_STRING));
				}
				try {
					X509Certificate[] x509 = keystoreManager
							.getCertificateByAlias(alias);
					if (x509 != null) {
						DistinguishedNameFields issuer = new DistinguishedNameFields(
								x509[0].getIssuerDN());
						String issuerCommonName = issuer.getCommonName(false);
						if (issuerCommonName.equals(TEST_CA)
								&& keystoreManager.isKeyEntry(alias)) {
							testKeyEntryExists = true;
							break;
						}
					}
				}
				catch (Exception ex) {
					logger.log(Level.SEVERE, "", ex);
					UserMessages.error(alias
							+ res.getString("COULD_NOT_READ_CERT"), ex
							.getMessage());
				}
			}
			addTestCertificateItem.setEnabled(!testKeyEntryExists);
		}
	}

	private void updateValues() {
		userCertModel.removeAllElements();
		caCertModel.removeAllElements();
		rejectedCertModel.removeAllElements();
		userCertificates = keystoreManager.getCachedKeyEntries();
		if (userCertificates != null) {
			for (int i = 0; i < userCertificates.size(); i++) {
				String alias = (String) userCertificates.elementAt(i);
				if (alias.equals(defaultAlias)) {
					// use oldDefaultAlias instead of alias to get upper case
					// letter again,
					// which are not present in keystore alias
					alias = defaultAlias + DEFAULT_STRING;
				}
				userCertModel.addElement(alias);
			}
		}
		caCertificates = keystoreManager.getCachedTrustedCertificates();
		if (caCertificates != null) {
			for (int i = 0; i < caCertificates.size(); i++) {
				String alias = (String) caCertificates.elementAt(i);
				caCertModel.addElement(alias);
			}
		}
		rejectedCertificates = keystoreManager.getRejectedEntries();
		if (rejectedCertificates != null) {
			for (int i = 0; i < rejectedCertificates.size(); i++) {
				String alias = (String) rejectedCertificates.elementAt(i);
				rejectedCertModel.addElement(alias);
			}
		}
		updateButtons();
		updateMenu();
		if (ResourceManager.isKeystoreAvailable()) {
			keyStoreName = keystoreManager.getKeyStoreName();
			keyStoreType = keystoreManager.getKeyStoreType();
			statusBar.setStatusMessage("[" + keystoreManager.getKeyStoreName()
					+ "] " + keystoreManager.getKeyStoreType());
		}
	}

	private class ActionChangedListener implements PropertyChangeListener {

		JMenuItem menuItem;

		ActionChangedListener(JMenuItem mi) {
			super();
			this.menuItem = mi;
		}

		public void propertyChange(PropertyChangeEvent e) {
			String propertyName = e.getPropertyName();
			if (e.getPropertyName().equals(Action.NAME)) {
				String text = (String) e.getNewValue();
				menuItem.setText(text);
			}
			else if (propertyName.equals("enabled")) {
				Boolean enabledState = (Boolean) e.getNewValue();
				menuItem.setEnabled(enabledState.booleanValue());
			}
		}
	}

	class AddCertificateAction extends AbstractAction {

		AddCertificateAction() {
			super("certificate");
		}

		public void actionPerformed(ActionEvent e) {
			addTrustedCertificate();
		}
	}

	class AddTestCertificateAction extends AbstractAction {

		AddTestCertificateAction() {
			super("test certificate");
		}

		public void actionPerformed(ActionEvent e) {
			importTestCert();
		}
	}

	private class ButtonListener implements ActionListener {

		public void actionPerformed(java.awt.event.ActionEvent e) {
			Object src = e.getSource();
			if (src == userDetailsButton) {
				showSelectedCertificate(userCertList);
			}
			else if (src == caDetailsButton) {
				showSelectedCertificate(caCertList);
			}
			else if (src == rejectedDetailsButton) {
				showSelectedCertificate(rejectedCertList);
			}
			else if (src == rejectedDeleteButton) {
				deleteSelectedCertificate(rejectedCertList);
			}
			else if (src == userDeleteButton) {
				deleteSelectedCertificate(userCertList);
			}
			else if (src == caDeleteButton) {
				deleteSelectedCertificate(caCertList);
			}
			else if (src == defaultIdentityButton) {
				setAsDefault((String) userCertList.getSelectedValue());
			}
			else if (src == exportPublicKeyButton) {
				exportPublicKey();
			}
			else if (src == changeAliasButton) {
				changeAlias();
			}
		}
	}

	class CSRAction extends AbstractAction {

		CSRAction() {
			super("csr");
		}

		public void actionPerformed(ActionEvent e) {
			generateCSR();
		}
	}

	class ExportKeyAction extends AbstractAction {

		ExportKeyAction() {
			super("keyexport");
		}

		public void actionPerformed(ActionEvent e) {
			exportKey();
		}
	}

	class ExitAction extends AbstractAction {

		ExitAction() {
			super("exit");
		}

		public void actionPerformed(ActionEvent e) {
			exitKeystore();
		}
	}

	private class ImportUserCertThread extends Thread {

		private KeyStoreManager newKeystoreManager;
		private char[] password;

		public ImportUserCertThread(KeyStoreManager newKeystoreManager,
				char[] password) {
			this.newKeystoreManager = newKeystoreManager;
			this.password = password;
		}

		public void run() {
			// System.out.println("This is import thread:" +
			// Thread.currentThread());
			setStatusMessage(res.getString("IMPORTING_KEYSTORE"));
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			Vector keys = newKeystoreManager.getCachedKeyEntries();
			Vector trustedCertificates = newKeystoreManager
					.getCachedTrustedCertificates();
			Vector rejectedCertificates = newKeystoreManager
					.getRejectedEntries();
			try {
				keystoreManager.mergeKeystore(newKeystoreManager, keys,
						password);
				keystoreManager.mergeKeystore(newKeystoreManager,
						trustedCertificates, password);
				keystoreManager.mergeKeystore(newKeystoreManager,
						rejectedCertificates, password);
				Vector keyEntries = keystoreManager.getCachedKeyEntries();
				if (keyEntries.size() == 1) {
					defaultAlias = (String) keyEntries.elementAt(0);
				}
				updateValues();
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "", e);
				statusBar.stopProgressBar();
				UserMessages.error(res.getString("MERGING_KEYSTORE_FAILED"), e
						.getLocalizedMessage());
			}
			statusBar.stopProgressBar();
			statusBar
					.setStatusMessage("[" + keyStoreName + "] " + keyStoreType);
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	}

	/**
	 * Listen to list selections
	 */
	private class ListListener implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent e) {
			// check if this is one of multiple selection events
			if (e.getValueIsAdjusting()) {
				return;
			}
			if (e.getSource() == caCertList) {
				updateButtons();
			}
			else if (e.getSource() == userCertList) {
				updateButtons();
			}
			else if (e.getSource() == rejectedCertList) {
				updateButtons();
			}
		}
	}

	class MergeAction extends AbstractAction {

		MergeAction() {
			super("merge");
		}

		public void actionPerformed(ActionEvent e) {
			mergeKeystore();
		}
	}

	class PasswordAction extends AbstractAction {

		PasswordAction() {
			super("password");
		}

		public void actionPerformed(ActionEvent e) {
			changeKeystorePassword();
		}
	}

	class SaveAction extends AbstractAction {

		SaveAction() {
			super("save");
		}

		SaveAction(String name) {
			super(name);
		}

		public void actionPerformed(ActionEvent e) {
			save();
			statusBar
					.setStatusMessage("[" + keyStoreName + "] " + keyStoreType);
		}
	}

	class StatusBar extends JPanel {

		private JTextField message;
		private JProgressBar progressBar;

		public StatusBar() {
			Border border = BorderFactory.createLoweredBevelBorder();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			message = new JTextField(30);
			message.setBorder(border);
			message.setToolTipText("Status messages");
			this.add(message);
			this.add(Box.createHorizontalGlue());
			progressBar = new JProgressBar(0, 1);
			progressBar.setBorder(border);
			this.add(progressBar);
		}

		public void setProgressString(final String in) {
			Runnable runner = new Runnable() {

				public void run() {
					progressBar.setStringPainted(true);
					progressBar.setString(in);
				}
			};
			SwingUtilities.invokeLater(runner);
		}

		public void setStatusMessage(final String messageString) {
			Runnable runner = new Runnable() {

				public void run() {
					message.setText(messageString);
				}
			};
			SwingUtilities.invokeLater(runner);
		}

		public void startProgressBar() {
			Runnable runner = new Runnable() {

				public void run() {
					progressBar.setStringPainted(true);
					progressBar.setValue(0);
					progressBar.setIndeterminate(true);
				}
			};
			SwingUtilities.invokeLater(runner);
		}

		public void stopProgressBar() {
			Runnable runner = new Runnable() {

				public void run() {
					progressBar.setStringPainted(false);
					progressBar.setValue(0);
					progressBar.setIndeterminate(false);
				}
			};
			SwingUtilities.invokeLater(runner);
		}
	}
}