/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.explorer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.util.Arrays;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * @author Ralf Ratering
 * @version $Id: DirNode.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class DirNode extends DefaultMutableTreeNode implements Transferable {

	private static DataFlavor flavor;
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.explorer");

	public static DataFlavor getFlavor() {
		return flavor;
	}
	private Icon icon;
	private File[] listing;
	private boolean listingEnabled = false;
	private DataFlavor[] supportedFlavors = {flavor};
	private boolean upToDate = false;
	private FileSystemView view;

	public DirNode(File file, FileSystemView _view) {
		super(file);
		view = _view;
		// per default do not list floppy drives
		listingEnabled = !view.isFloppyDrive(file);
		icon = view.getSystemIcon(file);
		try {
			flavor = new DataFlavor(Class
					.forName("com.pallas.unicore.client.explorer.DirNode"),
					"DirNode");
		}
		catch (ClassNotFoundException e) {
			logger.log(Level.SEVERE, "Did not find DirNode class.", e);
		}
	}

	public boolean equals(Object object) {
		DirNode node2 = (DirNode) object;
		return getUserObject().equals(node2.getUserObject());
	}

	public File getDir() {
		return (File) getUserObject();
	}

	public Icon getIcon() {
		return icon;
	}

	public File[] getListing() {
		if (listing == null) {
			File file = (File) userObject;
			if (listingEnabled) {
				listing = view.getFiles(file, false);
				Arrays.sort(listing);
			}
			else {
				logger.info("NO LISTING FOR: " + file);
				listing = new File[0];
			}
		}
		return listing;
	}

	/**
	 * Transferable interface.
	 *  
	 */
	public synchronized Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException {
		if (flavor.equals(flavor)) {
			return this;
		}
		else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 * Transferable interface
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		return (supportedFlavors);
	}

	public int hashCode() {
		return getUserObject().hashCode();
	}

	/**
	 * Transferable interface.
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return (flavor.equals(flavor));
	}

	public boolean isListingEnabled() {
		return listingEnabled;
	}

	public boolean isUpToDate() {
		return upToDate;
	}

	public void setListing(File[] listing) {
		this.listing = listing;
	}

	public void setListing(Vector listVector) {
		this.listing = new File[listVector.size()];
		for (int i = 0; i < listVector.size(); i++) {
			listing[i] = (File) listVector.elementAt(i);
		}
	}

	public void setListingEnabled(boolean listingEnabled) {
		this.listingEnabled = listingEnabled;
	}

	public void setUpToDate(boolean upToDate) {
		this.upToDate = upToDate;
	}

	public String toString() {
		return view.getSystemDisplayName(getDir());
	}
}