/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.explorer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;
import org.unicore.Vsite;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystemView;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @version $Id: SelectorDialog.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class SelectorDialog extends JDialog {

	public static int APPROVE_OPTION = JFileChooser.APPROVE_OPTION,
			CANCEL_OPTION = JFileChooser.CANCEL_OPTION,
			DIRECTORIES_ONLY = JFileChooser.DIRECTORIES_ONLY,
			FILES_ONLY = JFileChooser.FILES_ONLY,
			FILES_AND_DIRECTORIES = JFileChooser.FILES_AND_DIRECTORIES;
	private static int dividerLocation;
	private static boolean initialized = false;
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.explorer");
	private static Point position;
	private static Dimension size;

	public static void main(String[] args) {
		JFrame frame = new JFrame("Selector Dialog Test");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		String startDir = System.getProperty("user.home");
		//String startDir ="C:/src/java/euklid/src/euklid/Euklid.java";
		SelectorDialog selector = new SelectorDialog(frame, new File(startDir),
				new Vector(), true);
		selector.setTitle("SelectorDialog");
		int result = selector.showDialog();
		System.err.println("SELECTED: " + selector.getSelectedFile()
				+ ", Code: " + result);
		System.exit(0);
	}
	private JPanel centerPanel;
	private SelectorPanel currentPanel;
	protected FileSelectedListener fileSelectedListener = new FileSelectedListener();
	private int fileSelectionMode;
	private boolean localEnabled = true;
	private Hashtable panelTable;
	private boolean preemptiveMode;
	private int returnCode;
	private File selectedFile;
	private JComboBox vsiteBox;
	private Vector vsites;

	public SelectorDialog(Frame owner) {
		this(owner, (File) null, (Vsite) null, new UserDefaults()
				.getPreemptiveFileSelectorMode());
	}

	protected SelectorDialog(Frame owner, boolean preemptiveMode) {
		this(owner, (File) null, (Vsite) null, preemptiveMode);
	}

	public SelectorDialog(Frame owner, File currentDir) {
		this(owner, currentDir, new Vector(), new UserDefaults()
				.getPreemptiveFileSelectorMode());
	}

	protected SelectorDialog(Frame owner, File currentDir,
			boolean preemptiveMode) {
		this(owner, currentDir, new Vector(), preemptiveMode);
	}

	public SelectorDialog(Frame owner, File selectedFile, Vector vsites) {
		this(owner, selectedFile, vsites, new UserDefaults()
				.getPreemptiveFileSelectorMode());
	}

	protected SelectorDialog(Frame owner, File selectedFile, Vector vsites,
			boolean preemptiveMode) {
		super(owner, true);
		this.selectedFile = selectedFile;
		this.vsites = vsites;
		this.preemptiveMode = preemptiveMode;
		init();
	}

	public SelectorDialog(Frame owner, File selectedFile, Vsite vsite) {
		this(owner, selectedFile, vsite, new UserDefaults()
				.getPreemptiveFileSelectorMode());
	}

	protected SelectorDialog(Frame owner, File selectedFile, Vsite vsite,
			boolean preemptiveMode) {
		super(owner, true);
		this.selectedFile = selectedFile;
		this.preemptiveMode = preemptiveMode;
		vsites = new Vector();
		if (vsite != null) {
			vsites.add(vsite);
		}
		init();
	}

	public void addFileFilter(FileFilter filter) {
		currentPanel.addFileFilter(filter);
	}

	protected void buildComponents() {
		panelTable = new Hashtable();
		Object startLocation = null;
		// analyze, if we want to start remote
		if (selectedFile instanceof RemoteFile) {
			RemoteFile file = (RemoteFile) selectedFile;
			currentPanel = buildPanel(file.getVsite());
			startLocation = file.getVsite();
		}
		else {
			currentPanel = buildPanel(FileStorage.NSPACE_STRING);
			startLocation = FileStorage.NSPACE_STRING;
		}
		JButton okButton = new JButton(new OkAction());
		okButton.setDefaultCapable(true);
		getRootPane().setDefaultButton(okButton);
		JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		buttonPanel.add(okButton);
		buttonPanel.add(new JButton(new CancelAction()));
		JPanel southPanel = new JPanel();
		southPanel.add(buttonPanel);
		centerPanel = new JPanel(new BorderLayout());
		centerPanel.add(currentPanel);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(centerPanel, BorderLayout.CENTER);
		getContentPane().add(southPanel, BorderLayout.SOUTH);
		// only add vsite box if vsites are available
		if (!vsites.isEmpty()) {
			vsiteBox = new JComboBox();
			vsiteBox.addItem(FileStorage.NSPACE_STRING);
			for (int i = 0; i < vsites.size(); i++) {
				vsiteBox.addItem(vsites.elementAt(i));
			}
			vsiteBox.setSelectedItem(startLocation);
			vsiteBox.setAction(new VsiteChangeAction());
			JLabel locationLabel = new JLabel("Location: ");
			locationLabel.setLabelFor(vsiteBox);
			JPanel northPanel = new JPanel();
			northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.X_AXIS));
			northPanel.add(locationLabel);
			northPanel.add(vsiteBox);
			getContentPane().add(northPanel, BorderLayout.NORTH);
		}
	}

	private SelectorPanel buildPanel(Object location) {
		SelectorPanel panel = (SelectorPanel) panelTable.get(location);
		if (panel != null) {
			return panel;
		}
		if (location.equals(FileStorage.NSPACE_STRING)) {
			panel = new SelectorPanel(selectedFile, preemptiveMode);
		}
		else if (location instanceof Vsite) {
			Vsite vsite = (Vsite) location;
			FileSystemView view = new RemoteFileSystemView(vsite, this);
			panel = new SelectorPanel(selectedFile, view, preemptiveMode);
		}
		panel.addPropertyChangeListener(SelectorPanel.FILE_SELECTED_PROPERTY,
				fileSelectedListener);
		panelTable.put(location, panel);
		return panel;
	}

	private boolean checkFileType(File file) {
		// if file not exists make sure the parent directory does.
		if (!file.exists()) {
			File parent = file.getParentFile();
			// in directory mode create file itself
			if (fileSelectionMode == DIRECTORIES_ONLY) {
				parent = file;
			}
			if (parent == null || !parent.exists()) {
				if (file instanceof RemoteFile) {
					JOptionPane
							.showMessageDialog(
									this,
									"Remote directory "
											+ parent
											+ " does not exist.\nPlease select an existing directory.",
									"UNICORE: Info",
									JOptionPane.INFORMATION_MESSAGE);
					return false;
				}
				if (JOptionPane
						.showConfirmDialog((JFrame) getParent(), "Directory "
								+ parent + " does not exist." + "\nCreate it?",
								"UNICORE: Create Directory?",
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
					// create dirs
					parent.mkdirs();
					return true;
				}
				// dir does not exist and we do not want to create it.
				return false;
			}
			// dir already exists
			return true;
		}
		if (fileSelectionMode == DIRECTORIES_ONLY && !file.isDirectory()) {
			JOptionPane.showMessageDialog(this, file + " is a file."
					+ "\nPlease, select a directory, not a file.",
					"UNICORE: Info", JOptionPane.INFORMATION_MESSAGE);
			return false;
		}
		if (fileSelectionMode == FILES_ONLY && !file.isFile()) {
			JOptionPane.showMessageDialog(this, file + " is a directory."
					+ "\nPlease, select a file, not a directory.",
					"UNICORE: Info", JOptionPane.INFORMATION_MESSAGE);
			return false;
		}
		return true;
	}

	public boolean getLocalEnabled() {
		return localEnabled;
	}

	public File getSelectedFile() {
		return selectedFile;
	}

	public FileFilter getSelectedFileFilter() {
		return currentPanel.getSelectedFileFilter();
	}

	public SelectorTable getSelectorTable() {
		return this.currentPanel.getSelectorTable();
	}

	private void init() {
		// make dialog closing appear as cancel
		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent evt) {
				returnCode = CANCEL_OPTION;
			}
		});
		for (int i = 0; i < vsites.size(); i++) {
			Vsite vsite = (Vsite) vsites.elementAt(i);
			if (!ResourceManager.isVsiteAvailable(vsite)) {
				vsites.remove(vsite);
			}
		}
		buildComponents();
		int xsize = 800;
		int ysize = 600;
		Dimension screen = getToolkit().getScreenSize();
		double xcenter = screen.getWidth() / 2;
		double ycenter = screen.getHeight() / 2;
		setSize(xsize, ysize);
		setLocation((int) (xcenter - xsize / 2), (int) (ycenter - ysize / 2));
		pack();
		currentPanel.setDividerLocation(250);
	}

	private void performOK() {
		returnCode = APPROVE_OPTION;
		selectedFile = currentPanel.getSelectedFile();
		if (checkFileType(selectedFile)) {
			setVisible(false);
		}
	}

	public void rememberLayout() {
		position = getLocation();
		size = getSize();
		dividerLocation = currentPanel.getDividerLocation();
	}

	public void removeFileFilters() {
		currentPanel.removeFileFilters();
	}

	public void resetLayout() {
		if (position == null || size == null) {
			return;
		}
		setLocation(position);
		setSize(size);
		currentPanel.setDividerLocation(dividerLocation);
	}

	public void setFilename(String filename) {
		currentPanel.setFilename(filename);
	}

	public void setFileSelectionMode(int fileSelectionMode) {
		this.fileSelectionMode = fileSelectionMode;
	}

	public void setLocalEnabled(boolean localEnabled) {
		this.localEnabled = localEnabled;
		if (vsiteBox == null) {
			return;
		}
		if (!localEnabled) {
			vsiteBox.removeItem(FileStorage.NSPACE_STRING);
			vsiteBox.setSelectedIndex(0);
		}
		else {
			vsiteBox.removeItem(FileStorage.NSPACE_STRING);
			vsiteBox.addItem(FileStorage.NSPACE_STRING);
		}
	}

	public void setSelectedFile(File selectedFile) throws FileNotFoundException {
		if (selectedFile == null) {
			selectedFile = FileSystemView.getFileSystemView()
					.getDefaultDirectory();
		}
		this.selectedFile = selectedFile;
		if (vsiteBox != null) {
			Object location = null;
			if (selectedFile instanceof RemoteFile) {
				location = ((RemoteFile) selectedFile).getVsite();
			}
			else {
				location = FileStorage.NSPACE_STRING;
			}
			vsiteBox.setSelectedItem(location);
		}
		currentPanel.setSelectedFile(selectedFile);
	}

	public void setSelectedFileFilter(FileFilter filter) {
		currentPanel.setSelectedFileFilter(filter);
	}

	public int showDialog() {
		if (initialized) {
			resetLayout();
		}
		else{
			pack();
			setSize(800, 600);
		}
		currentPanel.refresh();
		setVisible(true);
		rememberLayout();
		initialized = true;
		return returnCode;
	}

	private void updateCurrentPanel() {
		centerPanel.removeAll();
		centerPanel.add(currentPanel);
		currentPanel.revalidate();
		resetLayout();
	}

	private void updateFileFilters(SelectorPanel oldPanel,
			SelectorPanel newPanel) {
		Vector newFilters = oldPanel.getFileFilters();
		Vector oldFilters = newPanel.getFileFilters();
		for (int i = 0; i < newFilters.size(); i++) {
			FileFilter filter = (FileFilter) newFilters.elementAt(i);
			if (!oldFilters.contains(filter)) {
				newPanel.addFileFilter(filter);
			}
		}
		for (int i = 0; i < oldFilters.size(); i++) {
			FileFilter filter = (FileFilter) oldFilters.elementAt(i);
			if (!newFilters.contains(filter)) {
				newPanel.removeFileFilter(filter);
			}
		}
		newPanel.setSelectedFileFilter(oldPanel.getSelectedFileFilter());
	}

	class CancelAction extends AbstractAction {

		public CancelAction() {
			super("Cancel");
		}

		public void actionPerformed(ActionEvent e) {
			returnCode = CANCEL_OPTION;
			selectedFile = currentPanel.getSelectedFile();
			setVisible(false);
		}
	}

	private class FileSelectedListener implements PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent evt) {
			logger.finest("Property Changed: " + evt);
			if (evt.getPropertyName().equals(
					SelectorPanel.FILE_SELECTED_PROPERTY)) {
				logger.finest("File Selected Property Changed: " + evt);
				performOK();
			}
		}
	}

	class OkAction extends AbstractAction {

		public OkAction() {
			super("Ok");
		}

		public void actionPerformed(ActionEvent e) {
			performOK();
		}
	}

	class VsiteChangeAction extends AbstractAction {

		public void actionPerformed(ActionEvent e) {
			selectedFile = null;
			rememberLayout();
			SelectorPanel oldPanel = currentPanel;
			currentPanel = buildPanel(vsiteBox.getSelectedItem());
			updateFileFilters(oldPanel, currentPanel);
			updateCurrentPanel();
		}
	}
}