/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import com.pallas.unicore.client.graphics.Arrow;
import com.pallas.unicore.client.graphics.Selectable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Editor for DAGs
 * 
 * @author Ralf Ratering
 * @version $Id: DAGEditor.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class DAGEditor extends GraphDisplay {

	private class ClickListener implements MouseListener {

		public void mouseClicked(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
			DAGEditor.this.grabFocus();
			// get mouse click coords
			lastPoint = e.getPoint();
			// check if a vertex was selected
			for (int i = 0; i < graph.getNumberOfVertices(); i++) {
				Vertex vertex = graph.getVertex(i);
				if (vertex.getBounds().contains(lastPoint)) {
					selectObject(vertex);
					if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) > 0) {
						dragArrow = new Arrow(selectedObject.getCenter(), e
								.getPoint());
					}
					return;
				}
			}
			// check if an edge was selected
			Hashtable distances = new Hashtable();
			for (int i = 0; i < graph.getNumberOfEdges(); i++) {
				Edge edge = graph.getEdge(i);
				double distance = edge.getLine().ptSegDistSq(lastPoint);
				if (distance < 20) {
					distances.put(edge, new Double(distance));
				}
			}
			// now find the closest edge
			double minDist = 20.0;
			Edge closestEdge = null;
			Enumeration hitEdges = distances.keys();
			while (hitEdges.hasMoreElements()) {
				Edge edge = (Edge) hitEdges.nextElement();
				double distance = ((Double) distances.get(edge)).doubleValue();
				if (distance < minDist) {
					minDist = distance;
					closestEdge = edge;
				}
			}
			if (closestEdge != null) {
				selectObject(closestEdge);
				showPopupMenu(e);
				return;
			}
			// if we got here no object was hit
			deselectObject();
			grabFocus();
		}

		public void mouseReleased(MouseEvent e) {
			showPopupMenu(e);
			draggedHit = false;
			// delete drag arrow
			dragArrow = null;
			if (selectedObject instanceof Vertex
					&& selectedObject2 instanceof Vertex) {
				DAGVertex start = (DAGVertex) selectedObject;
				DAGVertex end = (DAGVertex) selectedObject2;
				if (!start.hasEdgeTo(end)) {
					Edge newEdge = new Edge(start, end);
					try {
						graph.addEdge(newEdge);
					}
					catch (Exception ex) {
						UserMessages.info(ex.getMessage());
					}
					if (autoArrangeOn) {
						arrangeGraph(xoffset, yoffset);
					}
				}
				selectedObject2.setSelected(false);
				selectedObject2 = null;
			}
			repaint();
		}
	}

	private class DragListener implements MouseMotionListener {

		public void mouseDragged(MouseEvent e) {
			if (selectedObject == null) {
				return;
			}
			// stop dragging at border
			Point point = e.getPoint();
			if (point.getX() > getWidth()) {
				point.setLocation(getWidth(), point.getY());
			}
			else if (point.getX() < 0) {
				point.setLocation(0, point.getY());
			}
			if (point.getY() > getHeight()) {
				point.setLocation(point.getX(), getHeight());
			}
			else if (point.getY() < 0) {
				point.setLocation(point.getX(), 0);
			}
			// left button->move object
			if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) > 0) {
				moveObject(point);
			}
			// right button -> draw dependency
			else if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) > 0) {
				if (!(selectedObject instanceof Vertex)) {
					return;
				}
				dragArrow.setLine(selectedObject.getCenter(), point);
				// check if a vertex was hit
				if (selectedObject2 != null) {
					selectedObject2.setSelected(false);
				}
				selectedObject2 = null;
				for (int i = 0; i < graph.getNumberOfVertices(); i++) {
					if (graph.getVertex(i).getBounds().contains(e.getPoint())) {
						if (graph.getVertex(i) != selectedObject) {
							selectedObject2 = graph.getVertex(i);
							selectedObject2.setSelected(true);
							draggedHit = true;
							break;
						}
					}
				}
				repaint();
			}
		}

		public void mouseMoved(MouseEvent e) {
		}
	}

	class IgnoreFailureAction extends AbstractAction {

		public IgnoreFailureAction() {
			super("Ignore task failure");
		}

		public void actionPerformed(ActionEvent e) {
			((ActionContainer) ((ContainerVertex) selectedObject)
					.getContainer()).setIgnoreFailure(ignoreFailedItem
					.isSelected());
			ResourceManager.getCurrentInstance().getJPATree().updateCurrentJob(
					false);
		}
	}

	private class PopupListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			removeEdge((Edge) selectedObject);
			repaint();
		}
	}
	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.graphs");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.graphs.ResourceStrings");

	/**
	 * The main program for the DAGEditor class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("DAG Editor");
		String filename = res.getString("TEST_ICON");
		Image image = Toolkit.getDefaultToolkit().createImage(filename);
		ImageIcon icon = new ImageIcon(image);
		DAGVertex v1 = new DAGVertex("Eins");
		DAGVertex v2 = new DAGVertex("Zwei");
		DAGVertex v3 = new DAGVertex("DreiDas ist aber ein langer Name");
		DAGVertex v4 = new DAGVertex("Vier");
		v1.setCenter(new Point(60, 70));
		v2.setCenter(new Point(100, 100));
		v3.setCenter(new Point(220, 300));
		v4.setCenter(new Point(60, 220));
		v1.setImageIcon(icon);
		v2.setImageIcon(icon);
		v3.setImageIcon(icon);
		v4.setImageIcon(icon);
		Edge e1 = new Edge(v1, v2);
		Edge e2 = new Edge(v1, v3);
		Edge e3 = new Edge(v1, v4);
		//Edge e4 = new Edge(v2, v1);
		Edge e5 = new Edge(v2, v3);
		Edge e6 = new Edge(v2, v4);
		//Edge e7 = new Edge(v3, v1);
		//Edge e8 = new Edge(v3, v2);
		Edge e9 = new Edge(v3, v4);
		//Edge e10 = new Edge(v4, v1);
		//Edge e11 = new Edge(v4, v2);
		//Edge e12 = new Edge(v4, v3);
		DAG graph = new DAG();
		graph.addVertex(v1);
		graph.addVertex(v2);
		graph.addVertex(v3);
		graph.addVertex(v4);
		try {
			graph.addEdge(e1);
			graph.addEdge(e2);
			graph.addEdge(e3);
			// graph.addEdge(e4);
			graph.addEdge(e5);
			graph.addEdge(e6);
			// graph.addEdge(e7);
			//graph.addEdge(e8);
			graph.addEdge(e9);
			//graph.addEdge(e10);
			//graph.addEdge(e11);
			//graph.addEdge(e12);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		DAGEditor graphEditor = new DAGEditor(graph);
		graphEditor.setAutoArrangeOn(true, 80, 80);
		JScrollPane scrollPane = new JScrollPane(graphEditor);
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);
		frame.setSize(400, 400);
		frame.show();
	}
	private boolean autoArrangeOn;
	private Arrow dragArrow;
	private boolean draggedHit = false;
	private JPopupMenu edgePopup;
	private JPopupMenu iconPopup;
	private JCheckBoxMenuItem ignoreFailedItem;
	private IgnoreFailureAction ignoreFailureAction = new IgnoreFailureAction();
	private Point lastPoint;
	private Selectable selectedObject;
	private Selectable selectedObject2;
	private int xoffset = 80;
	private int yoffset = 80;

	/**
	 * Constructor
	 * 
	 * @param graph
	 *            Input graph
	 */
	public DAGEditor(DAG graph) {
		super(graph);
		buildComponents();
		addMouseListener(new ClickListener());
		addMouseMotionListener(new DragListener());
	}

	/**
	 * Arrange graph icons
	 * 
	 * @param xoffset
	 *            x distance between icons in pixel
	 * @param yoffset
	 *            y distance between icons in pixel
	 */
	public void arrangeGraph(int xoffset, int yoffset) {
		DAG dag = (DAG) graph;
		dag.computeDepths();
		for (int i = 0; i <= dag.getDepth(); i++) {
			Vector nodes = dag.getNodesByDepth(i);
			for (int j = 0; j < nodes.size(); j++) {
				Vertex vertex = (Vertex) nodes.elementAt(j);
				vertex
						.setCenter(new Point((j + 1) * xoffset, (i + 1)
								* yoffset));
			}
		}
	}

	/**
	 * Build the gui components
	 */
	protected void buildComponents() {
		JMenuItem removeItem = new JMenuItem(res.getString("REMOVE"));
		removeItem.addActionListener(new PopupListener());
		edgePopup = new JPopupMenu();
		edgePopup.setInvoker(this);
		edgePopup.add(removeItem);
		ignoreFailedItem = new JCheckBoxMenuItem(ignoreFailureAction);
		iconPopup = new JPopupMenu();
		iconPopup.setInvoker(this);
		iconPopup.add(ignoreFailedItem);
	}

	/**
	 * Remove selection from object
	 */
	protected void deselectObject() {
		if (selectedObject != null) {
			selectedObject.setSelected(false);
		}
		selectedObject = null;
		repaint();
	}

	/**
	 * Check if icons are automatically arranged
	 * 
	 * @return true, if icons should be automatically arranged
	 */
	public boolean getAutoArrangeOn() {
		return autoArrangeOn;
	}

	/**
	 * Move an object and update panel size
	 * 
	 * @param point
	 *            mouse coordinate
	 */
	protected void moveObject(Point point) {
		Point oldCenter = selectedObject.getCenter();
		double tx = point.getX() + oldCenter.getX() - lastPoint.getX();
		double ty = point.getY() + oldCenter.getY() - lastPoint.getY();
		selectedObject.setCenter(new Point((int) tx, (int) ty));
		lastPoint = point;
		// updates panel size
		revalidate();
		// repaint while dragging
		repaint();
	}

	/**
	 * Overwrite paint method
	 * 
	 * @param g
	 *            Graphics context
	 */
	public void paint(Graphics g) {
		super.paint(g);
		if (dragArrow != null) {
			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(graph.getColor());
			g2.draw(dragArrow);
		}
	}

	/**
	 * Constructor for the removeEdge object
	 * 
	 * @param edge
	 *            Description of the Parameter
	 */
	protected void removeEdge(Edge edge) {
		graph.removeEdge(edge);
		if (autoArrangeOn) {
			arrangeGraph(xoffset, yoffset);
		}
	}

	/**
	 * Select an object
	 * 
	 * @param object
	 *            to be selected
	 */
	protected void selectObject(Selectable object) {
		// deselect old object
		if (selectedObject != null) {
			selectedObject.setSelected(false);
		}
		selectedObject = object;
		selectedObject.setSelected(true);
		repaint();
	}

	/**
	 * Automatically arrange graph icons
	 * 
	 * @param autoArrangeOn
	 *            true, if icons should be automatically arranged
	 * @param xoffset
	 *            x distance between icons in pixel
	 * @param yoffset
	 *            y distance between icons in pixel
	 */
	public void setAutoArrangeOn(boolean autoArrangeOn, int xoffset, int yoffset) {
		this.autoArrangeOn = autoArrangeOn;
		this.xoffset = xoffset;
		this.yoffset = yoffset;
		if (autoArrangeOn) {
			arrangeGraph(xoffset, yoffset);
			repaint();
		}
	}

	/**
	 * Show edgePopup menu
	 * 
	 * @param e
	 *            MouseEvent
	 */
	protected void showPopupMenu(MouseEvent e) {
		if (selectedObject instanceof Edge) {
			if (e.isPopupTrigger()) {
				edgePopup.show(this, e.getX(), e.getY());
			}
		}
		else if (selectedObject instanceof ContainerVertex) {
			if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) > 0) {
				// get mouse click coords
				lastPoint = e.getPoint();
				// check if a vertex was selected
				for (int i = 0; i < graph.getNumberOfVertices(); i++) {
					Vertex vertex = graph.getVertex(i);
					if (vertex.getBounds().contains(lastPoint)) {
						if (vertex.equals(selectedObject)) {
							ActionContainer container = ((ContainerVertex) selectedObject)
									.getContainer();
							ignoreFailedItem.setSelected(container
									.getIgnoreFailure());
							iconPopup.show(this, e.getX(), e.getY());
						}
					}
				}
			}
		}
		else {
			return;
		}
	}
}