/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Panel that displays a graph
 * 
 * @author Ralf Ratering
 * @version $Id: GraphDisplay.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class GraphDisplay extends JPanel {

	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.graphs");

	/**
	 * The main program for the GraphDisplay class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("Graph Display");
		String filename = "c:/pix/testIcon.gif";
		Image image = Toolkit.getDefaultToolkit().createImage(filename);
		ImageIcon icon = new ImageIcon(image);
		Vertex v1 = new Vertex("EinsHullaBulla");
		Vertex v2 = new Vertex("ZweiHullaBulla");
		Vertex v3 = new Vertex("DreiHullaBulla");
		Vertex v4 = new Vertex("Vie");
		v1.setCenter(new Point(60, 70));
		v2.setCenter(new Point(100, 100));
		v3.setCenter(new Point(220, 300));
		v4.setCenter(new Point(40, 220));
		v1.setImageIcon(icon);
		v2.setImageIcon(icon);
		v3.setImageIcon(icon);
		v4.setImageIcon(icon);
		Edge e1 = new Edge(v1, v2);
		Edge e2 = new Edge(v1, v3);
		Edge e3 = new Edge(v1, v4);
		Edge e4 = new Edge(v2, v1);
		Edge e5 = new Edge(v2, v3);
		Edge e6 = new Edge(v2, v4);
		Edge e7 = new Edge(v3, v1);
		Edge e8 = new Edge(v3, v2);
		Edge e9 = new Edge(v3, v4);
		Edge e10 = new Edge(v4, v1);
		Edge e11 = new Edge(v4, v2);
		Edge e12 = new Edge(v4, v3);
		Graph graph = new Graph();
		graph.addVertex(v1);
		graph.addVertex(v2);
		graph.addVertex(v3);
		graph.addVertex(v4);
		try {
			graph.addEdge(e1);
			graph.addEdge(e2);
			graph.addEdge(e3);
			graph.addEdge(e4);
			graph.addEdge(e5);
			graph.addEdge(e6);
			graph.addEdge(e7);
			graph.addEdge(e8);
			graph.addEdge(e9);
			graph.addEdge(e10);
			graph.addEdge(e11);
			graph.addEdge(e12);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		GraphDisplay graphDisplay = new GraphDisplay(graph);
		JScrollPane scrollPane = new JScrollPane(graphDisplay);
		frame.getContentPane().add(scrollPane);
		frame.setSize(400, 400);
		frame.show();
	}
	/**
	 * the graph
	 */
	protected Graph graph;

	/**
	 * Constructor for the GraphDisplay object
	 * 
	 * @param graph
	 *            graph to be displayed
	 */
	public GraphDisplay(Graph graph) {
		this.graph = graph;
	}

	/**
	 * Gets the graph attribute of the GraphDisplay object
	 * 
	 * @return The graph value
	 */
	public Graph getGraph() {
		return graph;
	}

	/**
	 * Overwrite method to keep panel at graph size
	 * 
	 * @return size of the graph on screen
	 */
	public Dimension getPreferredSize() {
		return graph.getSize();
	}

	/**
	 * Overwrite paint method
	 * 
	 * @param g
	 *            graphics context
	 */
	public void paint(Graphics g) {
		super.paint(g);
		// draw nodes
		for (int i = 0; i < graph.getNumberOfVertices(); i++) {
			Vertex vertex = graph.getVertex(i);
			int x = (int) vertex.getCenter().getX();
			int y = (int) vertex.getCenter().getY();
			ImageIcon icon = vertex.getImageIcon();
			if (icon == null) {
				g.drawRect(x, y, 40, 40);
			}
			else {
				// draw icon
				vertex.draw(g, this);
			}
		}
		// draw edges
		for (int i = 0; i < graph.getNumberOfEdges(); i++) {
			graph.getEdge(i).draw(g, this);
		}
	}
}