/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

/**
 * This interface is comparable to the interface Comparable. This interface
 * imposes a total ordering on the objects of each class which implements it.
 * The distinction to Comparable is that the ordering objects are elements of a
 * graph.
 * 
 * @author Kirsten Foerster
 * @version $Id: ITreeComparable.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public interface ITreeComparable {

	/**
	 * Compares two vertexs for ordering. For example a criterion of the
	 * serialization ist he route inside a graph.
	 * 
	 * @param oneObject -
	 *            inserting element
	 * @param anotherObject -
	 *            element inside a sorted list
	 * @return a negative integer or zero if the first object is less than the
	 *         second one. If the return value is anegative the first object
	 *         should be inserted directly, if it is zero it is not sure that
	 *         the first object is really less. Therefore, the actual position
	 *         of the list has to noticed. The other elements of the list must
	 *         be checked by furhter invocations of this method. It returns a
	 *         positive integer if the first object maybe should be inserted
	 *         after the second one. The greater the return value the superior
	 *         is the prioritaet to inserting the element at this position. The
	 *         succeeding position has to be noticed if the prioritaet is higher
	 *         than one of the previous comparisons; respectively if the current
	 *         prioritaet is equal to the top one of the comparisions.
	 */
	public int compareTo(Object oneObject, Object anotherObject);

	/**
	 * Checks if the transfering object has childrens.
	 * 
	 * @param oneObject
	 * @return the value true if the transfering parameter has no childrens;
	 *         otherwise false.
	 */
	public boolean noChildrens(Object oneObject);

	/**
	 * Checks if the DAGVertex object has parents.
	 * 
	 * @param oneObject
	 * @return the value true if the transfering parameter has no parents;
	 *         otherwise false.
	 */
	public boolean noParents(Object oneObject);
}