/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.If;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.trees.AjoCellRenderer;
import com.pallas.unicore.client.trees.AjoNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * View AJO tasks with corresponding NJS log snippets
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: AjoViewPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class AjoViewPanel extends JPanel implements Applyable {

	private AbstractAction action;
	private JTree tree;
	/**
	 * tree model
	 */
	private DefaultTreeModel model;
	private AjoNode root;
	private JScrollPane scrollPane;
	private JTextArea logArea;
	private JButton logSaveButton;
	/**
	 * selected node that has focus
	 */
	private PanelNode focussedNode;

	public AjoViewPanel(AbstractAction action) {
		this.action = action;
		buildComponents();
	}

	/**
	 * Implementation of Applyable
	 */
	public void resetValues() {
		Enumeration enum = root.depthFirstEnumeration();
		while (enum.hasMoreElements()) {
			((AjoNode) enum.nextElement()).updateActionStatus();
		}
		updatePanels((AbstractAction) focussedNode.getUserObject());
	}

	/**
	 * Empty implementation of Applyable
	 */
	public void updateValues() {
	}

	/**
	 * Empty implementation of Applyable
	 */
	public void applyValues() {
	}

	private void buildComponents() {
		root = new AjoNode(action);
		focussedNode = root;
		if (action instanceof AbstractJob) {
			root.setIcon(ResourceManager.getIcon(ResourceManager.ABSTRACTJOB));
		}
		else if (action instanceof ActionGroup) {
			root.setIcon(ResourceManager.getIcon(ResourceManager.ACTIONGROUP));
		}
		else {
			root.setIcon(ResourceManager.getIcon(ResourceManager.ABSTRACTTASK));
		}
		tree = new JTree(root);
		model = new DefaultTreeModel(root);
		model.setAsksAllowsChildren(true);
		tree.setModel(model);
		// some cosmetics
		tree.setCellRenderer(new AjoCellRenderer());
		tree.setEditable(false);
		ToolTipManager.sharedInstance().registerComponent(tree);
		putClientProperty("JTree.lineStyle", "Angled");
		// selections
		tree.addTreeSelectionListener(new MyTreeSelectionListener());
		// allow only selection of one node at a time
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		if (action instanceof ActionGroup) {
			addSubTasks((ActionGroup) action, root);
		}
		scrollPane = new JScrollPane(tree);
		JPanel tmp = new JPanel(new BorderLayout());
		tmp.add(scrollPane, BorderLayout.CENTER);
		// logs
		logArea = new JTextArea();
		JScrollPane logScrollPane = new JScrollPane(logArea);
		JPanel logPanel = new JPanel(new BorderLayout());
		logSaveButton = new JButton("Save");
		logSaveButton.addActionListener(new ButtonListener());
		JPanel logButtonPanel = new JPanel();
		logButtonPanel.add(logSaveButton);
		logPanel.add(logScrollPane, BorderLayout.CENTER);
		logPanel.add(logButtonPanel, BorderLayout.SOUTH);
		setLayout(new BorderLayout());
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tmp,
				logPanel);
		add(splitPane, BorderLayout.CENTER);
		// choose a nice initial divider location
		splitPane.setDividerLocation(200);
		// reset values to check if we already have
		// fetched the status
		resetValues();
	}

	private void addSubTasks(ActionGroup group, AjoNode root) {
		Enumeration actions = group.getActions();
		root.updateActionStatus();
		while (actions.hasMoreElements()) {
			AbstractAction next = (AbstractAction) actions.nextElement();
			AjoNode nextNode = new AjoNode(next);
			if (next instanceof ActionGroup) {
				nextNode.setIcon(ResourceManager
						.getIcon(ResourceManager.ACTIONGROUP));
			}
			else {
				nextNode.setIcon(ResourceManager
						.getIcon(ResourceManager.ABSTRACTTASK));
			}
			model.insertNodeInto(nextNode, root, root.getChildCount());
			nextNode.updateActionStatus();
			if (next instanceof ActionGroup) {
				nextNode.setIcon(ResourceManager
						.getIcon(ResourceManager.ACTIONGROUP));
				addSubTasks((ActionGroup) next, nextNode);
			}
			else if (next instanceof If) {
				If ifAction = (If) next;
				AbstractAction trueAction = ifAction.getTrueBranch();
				AbstractAction falseAction = ifAction.getFalseBranch();
				if (trueAction instanceof ActionGroup) {
					addSubTasks((ActionGroup) trueAction, nextNode);
				}
				else {
					AjoNode trueNode = new AjoNode(trueAction);
					trueNode.setIcon(ResourceManager
							.getIcon(ResourceManager.ABSTRACTTASK));
					model.insertNodeInto(trueNode, nextNode, nextNode
							.getChildCount());
					trueNode.updateActionStatus();
				}
				if (falseAction instanceof ActionGroup) {
					addSubTasks((ActionGroup) falseAction, nextNode);
				}
				else {
					AjoNode falseNode = new AjoNode(trueAction);
					falseNode.setIcon(ResourceManager
							.getIcon(ResourceManager.ABSTRACTTASK));
					model.insertNodeInto(falseNode, nextNode, nextNode
							.getChildCount());
					falseNode.updateActionStatus();
				}
			}
		}
	}

	/**
	 * Save stdout or stderr to file
	 * 
	 * @param text
	 *            text from stdout or stderr area
	 * @param title
	 *            title of file chooser dialog
	 */
	private void saveToFile(String text, String title) {
		SelectorDialog d = ResourceManager.getSelectorDialog(new File(
				ResourceManager.getUserDefaults().getLogDirectory()), null);
		d.setTitle(title);
		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			try {
				PrintWriter pw = new PrintWriter(new BufferedWriter(
						new FileWriter(file)));
				pw.println(text);
				pw.close();
			}
			catch (IOException ioe) {
				UserMessages.error("Saving file " + filename + " failed.", ioe
						.getMessage());
			}
		}
	}

	/**
	 * Get selected node and its corresponding objects
	 */
	private void updateSelection(TreePath newPath) {
		synchronized (root) {
			// if no node has been selected, automatically select last selected
			// nodes
			// and if it's the first time, select root node
			if (tree.isSelectionEmpty()) {
				focussedNode = root;
				tree.setSelectionPath(new TreePath(focussedNode.getPath()));
			}
			// set focus to last selected node
			if (newPath == null) {
				return;
			}
			focussedNode = (PanelNode) newPath.getLastPathComponent();
		}
	}

	private void updatePanels(AbstractAction what) {
		logArea.setText("");
		Vector sortedEntries = OutcomeManager.getOutcomeEntries(what);
		for (int i = 0; i < sortedEntries.size(); i++) {
			OutcomeEntry entry = (OutcomeEntry) sortedEntries.elementAt(i);
			if (entry.getOutcome() != null) {
				String header = "Task: " + entry.getOutcome().getId().getName();
				if (!entry.getIteration().equals("")) {
					header += ", Iteration: " + entry.getIteration();
				}
				logArea.append(header + "\n" + entry.getLog() + "\n");
			}
		}
	}

	/**
	 * Listener class listens to save button events
	 */
	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == logSaveButton) {
				saveToFile(logArea.getText(), "Save logs");
			}
		}
	}

	/**
	 * Listen to tree selections and call updateSelection method
	 */
	private class MyTreeSelectionListener implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent event) {
			updateSelection(event.getNewLeadSelectionPath());
			AbstractAction focussedObject = (AbstractAction) focussedNode
					.getUserObject();
			if (focussedObject != null) {
				updatePanels(focussedObject);
			}
		}
	}
}