/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.AbstractAction;
import org.unicore.Vsite;
import org.unicore.resources.Storage;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @version $Id: FileStoragePanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public abstract class FileStoragePanel extends FilePanel {

	class BrowseAction extends AbstractAction {

		public BrowseAction() {
			super("Browse", ResourceManager.getIcon(ResourceManager.OPEN));
		}

		public void actionPerformed(ActionEvent e) {
			if (table.getCellEditor() != null) {
				table.getCellEditor().stopCellEditing();
			}
			showBrowser();
		}
	}
	protected BrowseAction browseAction = new BrowseAction();
	protected String currentDirectory;
	protected Vector storages;
	protected Vector targetStorages;

	public FileStoragePanel(TaskContainer container, String currentDirectory) {
		super(container);
		this.currentDirectory = currentDirectory;
		popup.add(browseAction);
		toolbar.add(browseAction);
	}

	protected FileStorage getFileStorageAt(int row) {
		return (FileStorage) getFileOperationAt(row);
	}

	protected void removeFileStorage(FileStorage fileStorage) {
		removeFileOperation(fileStorage);
	}

	protected void replaceStorageResources(int col) {
		for (int i = 0; i < table.getRowCount(); i++) {
			Object storage = tableModel.getValueAt(i, col);
			// do not replace "Local"
			if (storage instanceof Storage) {
				//ResourceEntry siteStorage =
				// storages.findResourceEntry(((ResourceEntry)storage).getResource());
				NamedResourceSet siteResources = ResourceManager
						.getResourceSet(container.getVsite());
				Storage siteStorage = (Storage) siteResources
						.findResource((Storage) storage);
				// Only replace storages found at current Vsite; keep "wrong"
				// storage from other Vsite
				if (siteStorage != null) {
					tableModel.setValueAt(siteStorage, i, col);
				}
			}
		}
		// apply new entries
		applyValues();
	}

	protected abstract void showBrowser();

	/**
	 * Get current storage resources from ResourceManager
	 */
	public void updateStorageResources() {
		/*
		 * Vector alsoResourceEntries = new Vector();
		 * alsoResourceEntries.add(ResourceEntry.LOCAL);
		 * storages.update(container.getVsite(), alsoResourceEntries);
		 */
		if (storages == null) {
			storages = new Vector();
		}
		storages.removeAllElements();
		storages.add(FileStorage.NSPACE_STRING);
		NamedResourceSet resourceSet = ResourceManager.getResourceSet(container
				.getVsite());
		if (resourceSet == null) {
			return;
		}
		resourceSet.addStorageResources(storages);
		Vector cloned = (Vector) storages.clone();
		for (int i = 0; i < cloned.size(); i++) {
			if (cloned.elementAt(i) instanceof org.unicore.resources.Spool
					|| cloned.elementAt(i) instanceof org.unicore.resources.AlternativeUspace) {
				storages.remove(cloned.elementAt(i));
			}
		}
		cloned.clear();
	}

	/**
	 * Get current storage resources from ResourceManager
	 */
	public Vector updateStorageResources(Vsite vsite) {
		Vector storages = new Vector();
		NamedResourceSet resourceSet = ResourceManager.getResourceSet(vsite);
		if (resourceSet == null) {
			return storages;
		}
		resourceSet.addStorageResources(storages);
		Vector cloned = (Vector) storages.clone();
		for (int i = 0; i < cloned.size(); i++) {
			if (cloned.elementAt(i) instanceof org.unicore.resources.Spool
					|| cloned.elementAt(i) instanceof org.unicore.resources.AlternativeUspace) {
				storages.remove(cloned.elementAt(i));
			}
		}
		cloned.clear();
		return storages;
	}
}