/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.Date;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import com.pallas.unicore.container.GroupContainer;

/**
 * Class for control group panels
 * 
 * @author Ralf Ratering
 * @version $Id: GroupPanel.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class GroupPanel extends JPAPanel {

	/**
	 * container to be modified
	 */
	protected GroupContainer container;
	private boolean editableNameField;
	/**
	 * name for group
	 */
	protected JTextField groupName;

	public GroupPanel(JFrame parentFrame, GroupContainer container) {
		this(parentFrame, container, true);
	}

	public GroupPanel(JFrame parentFrame, GroupContainer container,
			boolean editableNameField) {
		super(parentFrame, container);
		this.container = container;
		this.editableNameField = editableNameField;
		// build gui
		buildComponents();
	}

	/**
	 * Apply values from gui to JobContainer
	 */
	public void applyValues() {
		// remove spaces from text field
		groupName.setText(groupName.getText().trim().replace(' ', '_'));
		container.setName(groupName.getText());
		//container.checkContents();
		updateModifiedTime();
	}

	/**
	 * Build GUI
	 */
	protected void buildComponents() {
		add(buildNamePanel(), BorderLayout.NORTH);
		// split pane between lists and dependency display
		JPanel controlPanel = buildControlPanel();
		if (controlPanel != null) {
			add(controlPanel, BorderLayout.CENTER);
		}
	}

	protected JPanel buildControlPanel() {
		return null;
	}

	protected JPanel buildNamePanel() {
		groupName = new JTextField(15);
		groupName.setToolTipText("Name for group");
		groupName.setEnabled(editableNameField);
		JPanel groupNamePanel = new JPanel();
		groupNamePanel.add(new JLabel("Name"));
		groupNamePanel.add(groupName);
		return groupNamePanel;
	}

	public String getGroupName() {
		return groupName.getText();
	}

	/**
	 * Fill values from JobContainer into gui
	 */
	public void resetValues() {
		String name = container.getName();
		if (name != null) {
			groupName.setText(name);
		}
	}

	protected void updateModifiedTime() {
		if (container.getName() == null
				|| !container.getName().equals(groupName.getText())) {
			container.setModifiedTime(new Date());
			return;
		}
	}

	public void updateValues(boolean vsiteChanged) {
	}
}