/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import javax.swing.JPanel;

/**
 * Interface for objects that provide one or more panels.
 * 
 * @author Ralf Ratering
 * @version $Id: IPanelProvider.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public interface IPanelProvider {

	/**
	 * When the panel is removed from the gui, there may be some finalization
	 * neccessary (e.g. stopping threads). This will be done in this method.
	 */
	public void finalizePanel();

	/**
	 * Get number of panels provided by thhis object
	 * 
	 * @return nr of panels
	 */
	public int getNrOfPanels();

	/**
	 * Get ith panel
	 * 
	 * @param i
	 *            panel index
	 * @return ith panel
	 */
	public JPanel getPanel(int i);

	/**
	 * Get title for ith panel. The title can be used to put the panel into a
	 * tabbed pane for instance
	 * 
	 * @param i
	 *            panel index
	 * @return title string
	 */
	public String getPanelTitle(int i);
}