/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.security.cert.CertificateException;
import java.util.Date;
import java.util.Vector;
import java.util.logging.Level;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.unicore.User;
import org.unicore.Vsite;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.dialogs.ResourceInfoDialog;
import com.pallas.unicore.client.util.ExtensionPlugable;
import com.pallas.unicore.client.util.UnicorePlugable;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Panel to define a job group name, its usite, vsite and an email address
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: JobGroupPanel.java,v 1.2 2004/05/27 15:38:27 ratering Exp $
 */
public class JobGroupPanel extends GroupDependencyPanel {

	/**
	 * Listener class listens to site list selections
	 *  
	 */
	private class SiteListListener implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent evt) {
			// check if this is one of multiple selection events
			if (evt.getValueIsAdjusting()) {
				return;
			}
			final JList source = (JList) evt.getSource();
			if (source.equals(usiteList)) {
				updateUsite((Usite) source.getSelectedValue());
			}
			else if (source.equals(vsiteList)) {
				selectedVsite = (Vsite) source.getSelectedValue();
				JobContainer job = (JobContainer) container;
				if (selectedVsite == null
						|| selectedVsite.equals(job.getVsite())) {
					return;
				}
				job.setVsite(selectedVsite);
				resourcePanel.updateVsite();
				setNewUser();
				updateJPATree(true);
			}
			boolean enabled = vsiteList.getSelectedIndex() >= 0;
			for (int i = 0; i < vsiteToolBar.getComponentCount(); i++) {
				vsiteToolBar.getComponent(i).setEnabled(enabled);
			}
		}
	}

	private class VsiteInfoAction extends javax.swing.AbstractAction {

		public VsiteInfoAction() {
			super("", ResourceManager.getIcon(ResourceManager.CHECK));
		}

		public void actionPerformed(ActionEvent event) {
			NamedResourceSet siteResources = ResourceManager
					.getResourceSet(selectedVsite);
			ResourceInfoDialog dialog = new ResourceInfoDialog(getClient(),
					siteResources, false);
			dialog.show();
		}
	}
	private String defaultProject = "";
	private String defaultUserEmail = "";
	private String defaultXlogin = "";
	// user email text field
	private JTextField emailName;
	protected VsiteInfoAction infoAction;
	private JTextField project;
	private JobResourcePanel resourcePanel;
	private Usite selectedUsite = null;
	private Vsite selectedVsite;
	// split pane between lists and dependency display
	private JSplitPane splitPane;
	private JPanel userInfo;
	private JList usiteList;
	// Lists for usites and vsites
	private DefaultListModel usiteModel;
	private JList vsiteList;
	private DefaultListModel vsiteModel;
	private Vector vsites;
	// toolbar attached to vsite list
	private JToolBar vsiteToolBar;
	private JTextField xlogin;

	public JobGroupPanel(JFrame parentFrame, JobContainer container) {
		super(parentFrame, container);
		updateUserInfo();
	}

	private void addToPanel(JPanel panel, Component l, Component r,
			GridBagLayout g, GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}

	/**
	 * Apply values from gui to JobContainer
	 */
	public void applyValues() {
		logger.fine("Applying job group panel" + container);
		super.applyValues();
		resourcePanel.applyValues();
		JobContainer jobContainer = (JobContainer) container;
		jobContainer.setVsite(selectedVsite);
		jobContainer.setUsite(selectedUsite);
		updateModifiedTime();
		try {
			User defaultUser = ResourceManager.getUser(selectedVsite);
			if (defaultUser == null) {
				return;
			}
			User newUser = jobContainer.getUser();
			if (newUser == null) {
				return;
			}
			if (emailName.getText().equals("")) {
				newUser.setEmailAddress(null);
			}
			else {
				newUser.setEmailAddress(emailName.getText().trim());
			}
			if (xlogin.getText().equals("")) {
				newUser.setXlogin(null);
			}
			else {
				newUser.setXlogin(xlogin.getText());
			}
			if (project.getText().equals("")) {
				newUser.setProject(null);
			}
			else {
				newUser.setProject(project.getText());
			}
			jobContainer.setUser(newUser);
			updateUserInfo();
		}
		catch (Exception e) {
			logger.log(Level.WARNING, "Error setting xlogin/project.", e);
		}
		jobContainer.checkContents();
	}

	/**
	 * Build GUI
	 */
	protected void buildComponents() {
		usiteModel = new DefaultListModel();
		usiteList = new JList(usiteModel);
		usiteList.setToolTipText(res.getString("SELECT_USITE"));
		//uList.setVisibleRowCount(4);
		usiteList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		usiteList.addListSelectionListener(new SiteListListener());
		JScrollPane uSiteScrollPane = new JScrollPane(usiteList);
		uSiteScrollPane.getViewport().setView(usiteList);
		JPanel uSitePanel = new JPanel(new BorderLayout());
		uSitePanel.setBorder(new TitledBorder(res.getString("USITE")));
		uSitePanel.add(uSiteScrollPane, BorderLayout.CENTER);
		vsiteModel = new DefaultListModel();
		vsiteList = new JList(vsiteModel);
		vsiteList.setToolTipText(res.getString("SELECT_VSITE"));
		//vsiteList.setVisibleRowCount(4);
		vsiteList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		vsiteList.addListSelectionListener(new SiteListListener());
		JScrollPane vSiteScrollPane = new JScrollPane(vsiteList);
		vSiteScrollPane.getViewport().setView(vsiteList);
		buildVsiteToolBar();
		JPanel vSitePanel = new JPanel(new BorderLayout());
		vSitePanel.setBorder(new TitledBorder(res.getString("VSITE")));
		vSitePanel.add(vSiteScrollPane, BorderLayout.CENTER);
		vSitePanel.add(vsiteToolBar, BorderLayout.EAST);
		JPanel listPanel = new JPanel(new GridLayout(2, 1));
		listPanel.add(uSitePanel);
		listPanel.add(vSitePanel);
		buildUserInfo();
		JPanel textPanel = new JPanel(new BorderLayout());
		textPanel.add(listPanel, BorderLayout.CENTER);
		// textPanel.add(userInfo, BorderLayout.SOUTH);
		resourcePanel = new JobResourcePanel((JobContainer) container, this);
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add("Dependencies", buildDependencyPanel());
		tabbedPane.add("Resources", resourcePanel);
		tabbedPane.add("Special Settings", userInfo);
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, textPanel,
				tabbedPane);
		splitPane.setDividerLocation(220);
		setTitledBorder(res.getString("JOB_GROUP"));
		setBorder(BorderFactory.createEtchedBorder());
		add(buildNamePanel(), BorderLayout.NORTH);
		add(splitPane, BorderLayout.CENTER);
	}

	private void buildUserInfo() {
		emailName = new JTextField(20);
		emailName.setText(defaultUserEmail);
		emailName.setToolTipText(res.getString("ENTER_EMAIL"));
		xlogin = new JTextField(20);
		xlogin.setText(defaultXlogin);
		xlogin.setToolTipText(res.getString("ENTER_LOGIN"));
		project = new JTextField(20);
		project.setText(defaultProject);
		project.setToolTipText(res.getString("ENTER_PROJECT"));
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(1, 1, 1, 1);
		c.fill = GridBagConstraints.HORIZONTAL;
		JPanel subInfo = new JPanel(gridbag);
		addToPanel(subInfo, new JLabel(res.getString("EMAIL")), emailName,
				gridbag, c);
		addToPanel(subInfo, new JLabel(res.getString("LOGIN")), xlogin,
				gridbag, c);
		addToPanel(subInfo, new JLabel(res.getString("PROJECT")), project,
				gridbag, c);
		userInfo = new JPanel(new BorderLayout());
		userInfo.setBorder(new TitledBorder(res.getString("USER_INFO")));
		JLabel lable = new JLabel();
		lable.setText("Do not edit unless you know what you are doing!!");
		userInfo.add(lable, BorderLayout.NORTH);
		userInfo.add(subInfo, BorderLayout.CENTER);
	}

	private void buildVsiteToolBar() {
		infoAction = new VsiteInfoAction();
		JButton infoButton = new JButton(infoAction);
		infoAction.setEnabled(false);
		infoButton.setToolTipText(res.getString("GET_VSITE_INFO"));
		vsiteToolBar = new JToolBar();
		vsiteToolBar.setLayout(new BoxLayout(vsiteToolBar, BoxLayout.Y_AXIS));
		vsiteToolBar.setBorderPainted(false);
		vsiteToolBar.setFloatable(false);
		vsiteToolBar.add(infoButton, BorderLayout.NORTH);
		// add plugin components
		Vector plugins = Client.getPluginManager().getPluginClasses();
		for (int i = 0; i < plugins.size(); i++) {
			UnicorePlugable plugable = (UnicorePlugable) plugins.elementAt(i);
			if (plugable instanceof ExtensionPlugable) {
				Component component = ((ExtensionPlugable) plugable)
						.getVsiteToolBarComponent();
				if (component != null) {
					vsiteToolBar.add(component);
				}
			}
		}
	}

	public Client getClient() {
		return (Client) getParentFrame();
	}

	/**
	 * Fill values from JobContainer into gui
	 */
	public void resetValues() {
		logger.fine("Resetting job group panel" + container);
		super.resetValues();
		updateUsites();
		resourcePanel.resetValues();
		updateUserInfo();
	}

	protected void setNewUser() {
		JobContainer jobContainer = (JobContainer) container;
		//   toogleUserInfoFields(true);
		emailName.setText("");
		xlogin.setText("");
		project.setText("");
		//   toogleUserInfoFields(false);
		User defaultUser = ResourceManager.getUser(selectedVsite);
		try {
			User newUser = new User(defaultUser.getCertificate(), null);//before ajo4.1
			newUser.setEmailAddress(defaultUser.getEmailAddress());
			newUser.setXlogin(defaultUser.getXlogin());
			newUser.setProject(defaultUser.getProject());
			jobContainer.setUser(newUser);
		}
		catch (CertificateException ce) {
		}
		updateUserInfo();
	}

	protected void toogleUserInfoFields(boolean writeable) {
		emailName.setEditable(writeable);
		xlogin.setEditable(writeable);
		project.setEditable(writeable);
	}

	/**
	 * Update icons in current job in case resource sets do not or do match site
	 * resources now, but do not display errors.
	 */
	public void updateJPATree() {
		getClient().getJPATree().updateCurrentJob(false);
	}

	public void updateJPATree(boolean vsiteChanged) {
		getClient().getJPATree().updateCurrentJob(false, vsiteChanged);
	}

	protected void updateModifiedTime() {
		super.updateModifiedTime();
		JobContainer jobContainer = (JobContainer) container;
		if ((selectedUsite != null && !selectedUsite.equals(jobContainer
				.getUsite()))
				|| (selectedVsite != null && !selectedVsite.equals(jobContainer
						.getVsite()))) {
			jobContainer.setModifiedTime(new Date());
			return;
		}
	}

	protected void updateUserInfo() {
		JobContainer jobContainer = (JobContainer) container;
		User usr = jobContainer.getUser();
		//   toogleUserInfoFields(true);
		if (usr == null) {
			return;
		}
		if (usr.getEmailAddress() == null) {
			emailName.setText(ResourceManager.getUserDefaults().getUserEmail());
		}
		else {
			emailName.setText(usr.getEmailAddress());
		}
		if (usr.getXlogin() == null) {
			xlogin.setText(defaultXlogin);
		}
		else {
			xlogin.setText(usr.getXlogin());
		}
		if (usr.getProject() == null) {
			project.setText(defaultProject);
		}
		else {
			project.setText(usr.getProject());
		}
		//   toogleUserInfoFields(false);
	}

	/**
	 * Function is called if a new Usite was selected
	 * 
	 * @param usite
	 *            newly selected Usite
	 */
	private void updateUsite(Usite usite) {
		if (usite == null) {
			usiteList.clearSelection();
			return;
		}
		selectedUsite = usite;
		// try to previously selected vsite from vsite list
		Vsite oldSelectedVsite = (Vsite) vsiteList.getSelectedValue();
		// if no vsite was selected (in case of loading)
		// get vsite from container
		if (oldSelectedVsite == null) {
			oldSelectedVsite = ((JobContainer) container).getVsite();
		}
		vsiteModel.removeAllElements();
		vsites = ResourceManager.getVsites(selectedUsite);
		selectedVsite = null;
		for (int i = 0; i < vsites.size(); i++) {
			Vsite vsite = (Vsite) vsites.elementAt(i);
			if (ResourceManager.isSpecialVsite(vsite)) {
				continue;
			}
			vsiteModel.addElement(vsite);
			if (vsite.equals(oldSelectedVsite)) {
				selectedVsite = vsite;
			}
		}
		if (selectedVsite != null) {
			vsiteList.setSelectedValue(selectedVsite, true);
		}
	}

	public void updateUsites() {
		// get cached usites, if available
		Vector usites = ResourceManager.getUsites();
		JobContainer jobContainer = (JobContainer) container;
		// try to get selected usite from container
		selectedUsite = jobContainer.getUsite();
		usiteModel.removeAllElements();
		vsiteModel.removeAllElements();
		for (int i = 0; i < usites.size(); i++) {
			usiteModel.addElement(usites.elementAt(i));
		}
		// if no usite was selected get out
		if (selectedUsite == null) {
			return;
		}
		boolean usiteFound = false;
		for (int i = 0; i < usites.size(); i++) {
			Usite usite = (Usite) usites.elementAt(i);
			if (selectedUsite.equals(usite)) {
				// shuffle Usites due to different hashCodes
				selectedUsite = usite;
				usiteList.setSelectedValue(selectedUsite, true);
				updateUsite(selectedUsite);
				usiteFound = true;
				break;
			}
		}
		if (!usiteFound) {
			UserMessages.warning(res.getString("SELECTED_USITE")
					+ selectedUsite + res.getString("IN_JOB")
					+ jobContainer.getName() + res.getString("NOT_AVAILABLE"));
			selectedUsite = null;
		}
		// update resources in resource panel
		resourcePanel.updateVsite();
	}

	public void updateValues(boolean vsiteChanged) {
		logger.fine("Updating job group panel" + container
				+ ". Vsite changed: " + vsiteChanged);
		super.updateValues(vsiteChanged);
		resourcePanel.updateValues(vsiteChanged);
	}
}