/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import org.unicore.ajo.ScriptType;
import org.unicore.resources.CapacityResource;
import org.unicore.resources.Context;
import org.unicore.resources.InformationResource;
import org.unicore.resources.Memory;
import org.unicore.resources.NumericInfoResource;
import org.unicore.resources.PerformanceResource;
import org.unicore.resources.Priority;
import org.unicore.resources.PriorityValue;
import org.unicore.resources.Resource;
import org.unicore.resources.TextInfoResource;
import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.client.controls.ResourceEntry;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.TableSorter;
import com.pallas.unicore.extensions.NamedResourceSet;

/**
 * Panel to display information about a ResourceSet.
 * 
 * @author Ralf Ratering
 * @version $Id: ResourceEditPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class ResourceEditPanel extends JPanel {

	private class CheckBoxListener implements TableModelListener {

		public void tableChanged(TableModelEvent e) {
			if (e.getType() != TableModelEvent.UPDATE) {
				return;
			}
			DefaultTableModel model = (DefaultTableModel) e.getSource();
			if (model.equals(capacityTableModel)) {
				// check if "enabled" was updated
				if (model.getColumnName(e.getColumn()).equals(
						capacityColumnNames[0])) {
					capacityTable.repaint();
				}
			}
			else if (model.equals(storageTableModel)) {
				// check if "enabled" was updated
				if (model.getColumnName(e.getColumn()).equals(
						capacityColumnNames[0])) {
					storageTable.repaint();
				}
			}
			else if (model.equals(performanceTableModel)) {
				// check if "enabled" was updated
				if (model.getColumnName(e.getColumn()).equals(
						performanceColumnNames[0])) {
					performanceTable.repaint();
				}
			}
		}
	}

	class ContextBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			JCheckBox box = (JCheckBox) e.getItemSelectable();
			// get corresponding context resource out of table
			Context context = (Context) contextTable.get(box);
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				if (selectedContexts.contains(context)) {
					selectedContexts.remove(context);
				}
			}
			else if (e.getStateChange() == ItemEvent.SELECTED) {
				if (!selectedContexts.contains(context)) {
					selectedContexts.add(context);
				}
			}
		}
	}

	private class PriorityListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			JCheckBox box = (JCheckBox) e.getSource();
			// get corresponding context resource out of table
			selectedPriority = (Priority) priorityTable.get(box);
		}
	}

	private class ResourceCellEditor extends DefaultCellEditor {

		public ResourceCellEditor() {
			super(new IntTextField(0, 9999999));
		}

		public Object getCellEditorValue() {
			IntTextField intTextField = (IntTextField) editorComponent;
			return new Integer(intTextField.getValue());
		}
	}

	private class ResourceTableCellRenderer extends DefaultTableCellRenderer {

		public Component getTableCellRendererComponent(JTable myTable,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			super.getTableCellRendererComponent(myTable, value, isSelected,
					hasFocus, row, column);
			// disable background optimization from super class
			setOpaque(true);
			TableModel tableModel = myTable.getModel();
			if (myTable.getColumnName(column).equals("Request")) {
				if (((Boolean) tableModel.getValueAt(row, 0)).booleanValue()) {
					if (checkRequest(tableModel, row)) {
						if (isSelected) {
							setBackground(Color.green.darker());
						}
						else {
							setBackground(Color.green);
						}
					}
					else {
						if (isSelected) {
							setBackground(Color.red.darker());
						}
						else {
							setBackground(Color.red);
						}
					}
				}
				else {
					if (isSelected) {
						setBackground(myTable.getSelectionBackground());
					}
					else {
						setBackground(Color.lightGray.brighter());
					}
				}
			}
			else {
				if (isSelected) {
					setForeground(myTable.getSelectionForeground());
					setBackground(myTable.getSelectionBackground());
				}
				else {
					setBackground(myTable.getBackground());
					setForeground(myTable.getForeground());
				}
			}
			return this;
		}
	}

	private class ResourceTableModel extends DefaultTableModel {

		public ResourceTableModel(String[] columnNames, int numRows) {
			super(columnNames, numRows);
		}

		public Class getColumnClass(int c) {
			return getValueAt(0, c).getClass();
		}

		public boolean isCellEditable(int row, int col) {
			// let cols 1 & 2 be editable
			return (col == 0 || col == 2);
		}
	}
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.panels");

	/**
	 * The main program for the ResourceInfoPanel class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		NamedResourceSet siteResources = new NamedResourceSet("bla");
		siteResources.add(new Memory("Memory", 100.0, 50.0, 10.0));
		ResourceInfoPanel resourceInformationPanel = new ResourceInfoPanel(
				siteResources);
		JFrame app = new JFrame();
		app.getContentPane().add(resourceInformationPanel);
		app.pack();
		app.show();
	}
	private String[] capacityColumnNames = {"Enabled", "Name", "Request",
			"Units", "Minimum", "Maximum", "Default", "Description"};
	private AdjustTable capacityTable, storageTable, performanceTable;
	private ResourceTableModel capacityTableModel, storageTableModel,
			performanceTableModel;
	private CheckBoxListener checkBoxListener = new CheckBoxListener();
	private Hashtable contextTable = new Hashtable();
	private String[] performanceColumnNames = {"Enabled", "Name", "Request",
			"Units", "Performance", "Minimum", "Maximum", "Default",
			"Description"};
	private Hashtable priorityTable = new Hashtable();
	private NamedResourceSet request, siteResources;
	private Vector selectedContexts = new Vector();
	private Priority selectedPriority;

	public ResourceEditPanel(NamedResourceSet request,
			NamedResourceSet siteResources) {
		this.request = request;
		this.siteResources = siteResources;
		System.err.println("EDITING " + request + " AND SITE RESOURCES "
				+ siteResources);
		buildComponents();
	}

	private void applyCapacityValues(AdjustTable table) {
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		TableModel model = table.getModel();
		for (int i = 0; i < table.getRowCount(); i++) {
			Boolean enabled = (Boolean) model.getValueAt(i, 0);
			if (enabled.booleanValue()) {
				// add object from resource entry to request
				ResourceEntry entry = (ResourceEntry) model.getValueAt(i, 1);
				CapacityResource capacity = (CapacityResource) entry
						.getResource();
				// get request value from second table col
				Integer requestValue = new Integer(model.getValueAt(i, 2)
						.toString());
				capacity.setRequest(requestValue.doubleValue());
				request.add(capacity);
			}
		}
	}

	private void applyPerformanceValues() {
		if (performanceTable.getCellEditor() != null) {
			performanceTable.getCellEditor().stopCellEditing();
		}
		for (int i = 0; i < performanceTable.getRowCount(); i++) {
			Boolean enabled = (Boolean) performanceTableModel.getValueAt(i, 0);
			if (enabled.booleanValue()) {
				// add object from resource entry to request
				ResourceEntry entry = (ResourceEntry) performanceTableModel
						.getValueAt(i, 1);
				PerformanceResource performance = (PerformanceResource) entry
						.getResource();
				// get request value from second table col
				Integer requestValue = new Integer(performanceTableModel
						.getValueAt(i, 2).toString());
				performance.setRequest(requestValue.doubleValue());
				request.add(performance);
			}
		}
	}

	public void applyValues() {
		request.removeAllElements();
		if (capacityTable != null) {
			applyCapacityValues(capacityTable);
		}
		if (storageTable != null) {
			applyCapacityValues(storageTable);
		}
		if (performanceTable != null) {
			applyPerformanceValues();
		}
		if (selectedPriority != null) {
			request.add(selectedPriority);
		}
		for (int i = 0; i < selectedContexts.size(); i++) {
			request.add((Context) selectedContexts.elementAt(i));
		}
	}

	/**
	 * Build gui
	 */
	private void buildComponents() {
		Vector informations = siteResources
				.addInformationResources(new Vector());
		Vector performances = siteResources
				.addPerformanceResources(new Vector());
		Vector priorities = siteResources.addPriorityResources(new Vector());
		Vector capacities = siteResources.addCapacityResources(new Vector());
		Vector storages = siteResources.addStorageResources(new Vector());
		Vector storageServers = siteResources
				.addStorageServerResources(new Vector());
		Vector contexts = siteResources.addContextResources(new Vector());
		for (int i = 0; i < storages.size(); i++) {
			capacities.remove(storages.elementAt(i));
		}
		// do not use storage servers in common resource requests
		for (int i = 0; i < storageServers.size(); i++) {
			storages.remove(storageServers.elementAt(i));
		}
		//remove from contexts all sript types and all MakeReturnCodeDecision
		// objects
		Vector removeContext = new Vector();
		for (int i = 0; i < contexts.size(); i++) {
			Context context = (Context) contexts.elementAt(i);
			if (context.getName().equals("MakeReturnCodeDecision")) {
				removeContext.addElement(context);
				continue;
			}
			Enumeration e = ScriptType.elements();
			while (e.hasMoreElements()) {
				ScriptType t = (ScriptType) e.nextElement();
				String shell = t.getName();
				if (this.formatShellType(shell).equalsIgnoreCase(
						this.formatShellType(context.toString()))) {
					removeContext.addElement(context);
					break;
				}
			}
		}
		contexts.removeAll(removeContext);
		JPanel subPanel = new JPanel();
		subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
		ResourceTableCellRenderer renderer = new ResourceTableCellRenderer();
		ResourceCellEditor editor = new ResourceCellEditor();
		if (!capacities.isEmpty()) {
			capacityTableModel = new ResourceTableModel(capacityColumnNames, 0);
			capacityTableModel.addTableModelListener(checkBoxListener);
			capacityTable = getCapacityTable(capacities, capacityTableModel);
			capacityTable.setDefaultRenderer(Object.class, renderer);
			capacityTable.setDefaultRenderer(Integer.class, renderer);
			capacityTable.setDefaultEditor(Integer.class, editor);
			capacityTable.adjustWidths();
			capacityTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), (capacityTable.getRowCount() + 1)
							* capacityTable.getRowHeight()));
			JScrollPane capacityPanel = new JScrollPane(capacityTable);
			capacityPanel.setBorder(new TitledBorder("Capacities"));
			subPanel.add(capacityPanel);
		}
		if (!storages.isEmpty()) {
			storageTableModel = new ResourceTableModel(capacityColumnNames, 0);
			storageTableModel.addTableModelListener(checkBoxListener);
			storageTable = getCapacityTable(storages, storageTableModel);
			storageTable.setDefaultRenderer(Object.class, renderer);
			storageTable.setDefaultRenderer(Integer.class, renderer);
			storageTable.setDefaultEditor(Integer.class, editor);
			storageTable.adjustWidths();
			// set scroll pane viewport to table size
			storageTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), (storageTable.getRowCount() + 1)
							* storageTable.getRowHeight()));
			JScrollPane storagePanel = new JScrollPane(storageTable);
			storagePanel.setBorder(new TitledBorder("Storages"));
			subPanel.add(storagePanel);
		}
		if (!performances.isEmpty()) {
			performanceTableModel = new ResourceTableModel(
					performanceColumnNames, 0);
			performanceTableModel.addTableModelListener(checkBoxListener);
			performanceTable = getPerformanceTable(performances,
					performanceTableModel);
			performanceTable.setDefaultRenderer(Object.class, renderer);
			performanceTable.setDefaultRenderer(Integer.class, renderer);
			performanceTable.setDefaultEditor(Integer.class, editor);
			performanceTable.adjustWidths();
			// set scroll pane viewport to table size
			performanceTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), performanceTable.getRowCount()
							* performanceTable.getRowHeight()));
			JScrollPane performancePanel = new JScrollPane(performanceTable);
			performancePanel.setBorder(new TitledBorder("Performance"));
			subPanel.add(performancePanel);
		}
		if (!priorities.isEmpty()) {
			JPanel priorityButtons = getPriorityPanel(priorities);
			JScrollPane priorityPane = new JScrollPane(priorityButtons);
			priorityPane.setBorder(new TitledBorder("Priority"));
			subPanel.add(priorityPane);
		}
		if (!contexts.isEmpty()) {
			JPanel contextButtons = getContextPanel(contexts);
			JScrollPane contextPane = new JScrollPane(contextButtons);
			contextPane.setBorder(new TitledBorder("Context"));
			subPanel.add(contextPane);
		}
		if (!informations.isEmpty()) {
			JTextArea infoArea = getInfoArea(informations);
			JPanel infoPanel = new JPanel(new BorderLayout());
			infoPanel.setBorder(new TitledBorder("Information"));
			infoPanel.add(infoArea, BorderLayout.CENTER);
			subPanel.add(infoPanel);
		}
		setLayout(new BorderLayout());
		add(subPanel, BorderLayout.CENTER);
	}

	private boolean checkRequest(TableModel tableModel, int row) {
		int req = ((Integer) tableModel.getValueAt(row, 2)).intValue();
		ResourceEntry entry = (ResourceEntry) tableModel.getValueAt(row, 1);
		CapacityResource resource = (CapacityResource) entry.getResource();
		int min = (int) resource.getMinRequest();
		int max = (int) resource.getMaxRequest();
		return (req >= min && req <= max);
	}

	private String formatShellType(String shell) {
		for (int i = 0; i < shell.length(); i++) {
			if (Character.isWhitespace(shell.charAt(i))
					|| shell.charAt(i) == '_') {
				shell = shell.substring(0, i)
						+ shell.substring(i + 1, shell.length());
			}
		}
		return shell;
	}

	private AdjustTable getCapacityTable(Vector capacities,
			DefaultTableModel capacityTableModel) {
		// Instead of making the table display the data as it would normally
		// with:
		// AdjustTable tableView = new AdjustTable(dataModel);
		// Add a sorter, by using the following three lines instead of the one
		// above.
		TableSorter sorter = new TableSorter(capacityTableModel);
		AdjustTable table = new AdjustTable();
		table.setModel(sorter);
		sorter.addMouseListenerToHeaderInTable(table);
		try {
			for (int i = 0; i < capacities.size(); i++) {
				// get resource and clone it to prevent multiple references to
				// same resource
				CapacityResource originalResource = (CapacityResource) capacities
						.elementAt(i);
				CapacityResource resource = (CapacityResource) originalResource
						.clone();
				// init with site resource values
				Boolean requestFound = Boolean.FALSE;
				int requestValue = new Double(resource.getDefaultRequest())
						.intValue();
				// find out if resource exists in request
				// cast should be ok, because resource is a CapacityResource
				CapacityResource capacityRequest = (CapacityResource) request
						.findResource(resource);
				if (capacityRequest != null) {
					requestFound = Boolean.TRUE;
					requestValue = new Double(capacityRequest.getRequest())
							.intValue();
				}
				int min = new Double(resource.getMinRequest()).intValue();
				int max = new Double(resource.getMaxRequest()).intValue();
				int def = new Double(resource.getDefaultRequest()).intValue();
				Object[] rowData = {requestFound, new ResourceEntry(resource),
						new Integer(requestValue), resource.getUnits(),
						new Integer(min), new Integer(max), new Integer(def),
						resource.getDescription()};
				capacityTableModel.addRow(rowData);
			}
		}
		catch (CloneNotSupportedException e) {
			logger.log(Level.WARNING, "Cloning capacity resource failed.", e);
		}
		return table;
	}

	/**
	 * Gets the contextPanel attribute of the ResourceEditPanel object
	 * 
	 * @param contexts
	 *            Description of the Parameter
	 * @return The contextPanel value
	 */
	private JPanel getContextPanel(Vector contexts) {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		contextTable.clear();
		ContextBoxListener listener = new ContextBoxListener();
		for (int i = 0; i < contexts.size(); i++) {
			Context context = (Context) contexts.elementAt(i);
			String caption = context.getName();
			if (context.getVersion().length() > 0) {
				caption += ", Version: " + context.getVersion();
			}
			JCheckBox box = new JCheckBox(caption);
			box.setToolTipText(context.getDescription());
			box.addItemListener(listener);
			panel.add(box);
			// remember which box belongs to which resource
			contextTable.put(box, context);
			// check if context was in request
			if (request.findResource(context) != null) {
				box.setSelected(true);
				// this will be done via listener
				//selectedContexts.add(context);
			}
		}
		return panel;
	}

	/**
	 * Build a text area displaying all InformationResources
	 *  
	 */
	private JTextArea getInfoArea(Vector information) {
		JTextArea textArea = new JTextArea();
		textArea.setEditable(false);
		for (int i = 0; i < information.size(); i++) {
			InformationResource resource = (InformationResource) information
					.elementAt(i);
			if (resource instanceof TextInfoResource) {
				TextInfoResource text = (TextInfoResource) resource;
				textArea.append(text.getTag() + ": " + text.getValue() + "\n");
			}
			else if (resource instanceof NumericInfoResource) {
				NumericInfoResource num = (NumericInfoResource) resource;
				textArea.append(num.getDescription() + ": " + num.getValue()
						+ "\n");
			}
		}
		return textArea;
	}

	/**
	 * Build a table displaying all CapacityResources
	 *  
	 */
	private AdjustTable getPerformanceTable(Vector performances,
			DefaultTableModel performanceTableModel) {
		// Instead of making the table display the data as it would normally
		// with:
		// AdjustTable tableView = new AdjustTable(dataModel);
		// Add a sorter, by using the following three lines instead of the one
		// above.
		TableSorter sorter = new TableSorter(performanceTableModel);
		AdjustTable table = new AdjustTable();
		table.setModel(sorter);
		sorter.addMouseListenerToHeaderInTable(table);
		try {
			for (int i = 0; i < performances.size(); i++) {
				// get resource and clone it to prevent multiple references to
				// same resource
				PerformanceResource originalResource = (PerformanceResource) performances
						.elementAt(i);
				PerformanceResource resource = (PerformanceResource) originalResource
						.clone();
				// init with site resource values
				Boolean requestFound = Boolean.FALSE;
				int req = new Double(resource.getRequest()).intValue();
				// find out if resource exists in request
				// cast should be ok, because resource is a CapacityResource
				PerformanceResource performanceRequest = (PerformanceResource) request
						.findResource(resource);
				if (performanceRequest != null) {
					requestFound = Boolean.TRUE;
					req = new Double(performanceRequest.getRequest())
							.intValue();
				}
				int min = new Double(resource.getMinRequest()).intValue();
				int max = new Double(resource.getMaxRequest()).intValue();
				int def = new Double(resource.getDefaultRequest()).intValue();
				int per = new Double(resource.getPerformance()).intValue();
				Object[] rowData = {requestFound,
						new ResourceEntry((Resource) resource.clone()),
						new Integer(req), resource.getUnits(),
						new Integer(per), new Integer(min), new Integer(max),
						new Integer(def), resource.getDescription()};
				performanceTableModel.addRow(rowData);
			}
		}
		catch (CloneNotSupportedException e) {
			logger
					.log(Level.WARNING, "Cloning performance resource failed.",
							e);
		}
		return table;
	}

	private JPanel getPriorityPanel(Vector priorities) {
		PriorityListener priorityListener = new PriorityListener();
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		ButtonGroup group = new ButtonGroup();
		// check if priority was in request
		PriorityValue priorityRequest = request.getPriorityValue();
		for (int i = 0; i < priorities.size(); i++) {
			Priority priority = (Priority) priorities.elementAt(i);
			JCheckBox box = new JCheckBox(priority.getValue().toString());
			box.setToolTipText(priority.getDescription());
			box.addItemListener(priorityListener);
			group.add(box);
			panel.add(box);
			priorityTable.put(box, priority);
			if (priorityRequest != null
					&& priorityRequest.equals(priority.getValue())) {
				box.setSelected(true);
				selectedPriority = priority;
			}
		}
		return panel;
	}
}