/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.autoupdate;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;

import com.pallas.unicore.client.util.ExtensionPlugable;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Thomas Kentemich
 *@version    $Id: AutoUpdatePlugin.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class AutoUpdatePlugin extends ExtensionPlugable {

	/**
	 *  Listener class for settings menu entry
	 */
	private class CustomListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			watcher = new WatchFrame(ResourceManager.getCurrentInstance());
			SwingUtilities.updateComponentTreeUI(watcher);
			watcher.show();
		}
	}

	private WatchFrame watcher;

	/**
	 *  Used to add menus to main frame
	 *
	 *@return    The CustomMenu value
	 */
	public JMenuItem getCustomMenu() {
		JMenu returnMenu = new JMenu("Watch jobs");
		JMenuItem threadMonitor = new JMenuItem("Show list");

		threadMonitor.addActionListener(new CustomListener());
		returnMenu.add(threadMonitor);
		return returnMenu;
	}

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return "Autoupdate JMC entries\n" + "(C)2000-2004 Intel";
	}

	/**
	 *  Called to inititialize the plugin
	 *
	 */
	public void startPlugin() {
	}

	/**
	 *  Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}

}
