/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.autoupdate;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.TreeNode;
import org.unicore.Vsite;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.trees.JMCTree;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.container.JobContainer;

/**
 * @author Thomas Kentemich
 * @version $Id: WatchFrame.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class WatchFrame extends JFrame implements Runnable {

	private class ButtonListener implements ActionListener {

		public void actionPerformed(java.awt.event.ActionEvent e) {
			Object src = e.getSource();
			if (src == addButton) {
				// Add a component if it is a ContainerNode
				// We cannot add vsites or usites because updates on these
				// always
				// delete the nodes.
				synchronized (lock) {
					PanelNode panelNode = jmcTree.getFocussedNode();
					if (!isContained(panelNode)
							&& (panelNode instanceof PanelNode)) {
						Object userObject = panelNode.getUserObject();
						if ((userObject instanceof Vsite)
								|| (userObject instanceof JobContainer)) {
							addRow(panelNode);
						}
					}
				}
			}
			else if (src == removeButton) {
				// remove an entry from the list
				synchronized (lock) {
					while (table.getSelectedRow() > -1) {
						tableModel.removeRow(table.getSelectedRow());
					}
					int selectedRow = table.getRowCount() - 1;
					if (selectedRow >= 0) {
						table.setRowSelectionInterval(selectedRow, selectedRow);
					}
				}
			}
			else if (src == startButton) {
				// start the update thread
				if (stopKicker == true) {
					stopKicker = false;
					kicker = new Thread(instance);
					kicker.start();
					stopButton.setEnabled(true);
					startButton.setEnabled(false);
				}
			}
			else if (src == stopButton) {
				// stop update thread
				stopKicker = true;
				stopButton.setEnabled(false);
				startButton.setEnabled(true);
			}
		}
	}

	private class TreeChangeListener implements PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent evt) {
			String evtName = evt.getPropertyName();
			if (evtName.equals("jmcTreeChanged")) {
				// this updates the status of the watched job
				int nrOfRows = table.getRowCount();
				for (int i = 0; i < nrOfRows; i++) {
					PanelNode node = (PanelNode) tableModel.getValueAt(i, 0);
					if (node.equals(evt.getOldValue())) {
						table.repaint();
					}
				}
			}
			else if (evtName.equals("jmcTreeDelete")) {
				// if the node is deleted in the tree delete it also in the
				// table
				PanelNode node = (PanelNode) evt.getOldValue();
				int index = findRow(node);
				if (index != -1) {
					synchronized (lock) {
						tableModel.removeRow(index);
					}
				}
			}
		}
	}
	private JButton addButton, removeButton, startButton, stopButton;
	private Client client;
	private Runnable instance;
	private JMCTree jmcTree;
	private Thread kicker;
	private Object lock = new Object();
	private long sleepTime = 30000;
	private boolean stopKicker = true;
	private JTable table;
	private DefaultTableModel tableModel;

	public WatchFrame(Client client) {
		super("Watched jobs");
		this.client = client;
		jmcTree = client.getJMCTree();
		// add a PropertyChangeListener to catch events generated by the tree
		jmcTree.addPropertyChangeListener(new TreeChangeListener());
		// catch exit event
		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent evt) {
				stopKicker = true;
				if (kicker != null) {
					kicker.interrupt();
				}
				dispose();
				return;
			}
		});
		buildComponents();
		instance = this;
		pack();
	}

	/**
	 * add a row to the table
	 *  
	 */
	private void addRow(PanelNode node) {
		Vector rowData = new Vector();
		rowData.add(node);
		tableModel.addRow(rowData);
		int selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 * Setup GUI components
	 */
	private void buildComponents() {
		JPanel panel = new JPanel(new BorderLayout());
		ButtonListener buttonListener = new ButtonListener();
		String[] columnNames;
		columnNames = new String[1];
		columnNames[0] = "Watch";
		tableModel = new DefaultTableModel(columnNames, 0);
		table = new JTable(tableModel) {

			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		table.setRowHeight(20);
		Dimension size = new Dimension(200, 100);
		table.setPreferredScrollableViewportSize(size);
		// table column object for column 0
		TableColumnModel tcm = table.getColumnModel();
		TableColumn tableColumn = tcm.getColumn(0);
		tableColumn.setCellRenderer(new WatchTableCellRenderer());
		JScrollPane userScrollPane = new JScrollPane(table);
		JPanel userButtonPanel = new JPanel(new GridLayout(2, 2, 10, 10));
		addButton = new JButton("Add");
		removeButton = new JButton("Remove");
		addButton.addActionListener(buttonListener);
		removeButton.addActionListener(buttonListener);
		userButtonPanel.add(addButton);
		userButtonPanel.add(removeButton);
		startButton = new JButton("Start");
		startButton.addActionListener(buttonListener);
		stopButton = new JButton("Stop");
		stopButton.setEnabled(false);
		stopButton.addActionListener(buttonListener);
		userButtonPanel.add(startButton);
		userButtonPanel.add(stopButton);
		JPanel sliderPanel = new JPanel();
		sliderPanel.setBorder(new TitledBorder("Update frequency (s)"));
		JSlider slider = new JSlider(JSlider.HORIZONTAL, 0, 600,
				(int) (sleepTime / 1000));
		slider.setPaintTicks(true);
		slider.setMajorTickSpacing(100);
		slider.setMinorTickSpacing(20);
		slider.setPaintLabels(true);
		slider.addChangeListener(new ChangeListener() {

			/**
			 * Description of the Method
			 * 
			 * @param evt
			 *            Description of Parameter
			 */
			public void stateChanged(ChangeEvent evt) {
				JSlider source = (JSlider) evt.getSource();
				if (source.getValueIsAdjusting() == false) {
					sleepTime = source.getValue() * 1000;
					if (sleepTime < 10000) {
						// minimum 10 sec
						sleepTime = 10000;
					}
				}
			}
		});
		sliderPanel.add(slider);
		panel.add(userScrollPane, BorderLayout.NORTH);
		panel.add(userButtonPanel, BorderLayout.CENTER);
		panel.add(sliderPanel, BorderLayout.SOUTH);
		this.getContentPane().add(panel);
	}

	private int findRow(PanelNode node) {
		int nrOfRows = table.getRowCount();
		for (int i = 0; i < nrOfRows; i++) {
			PanelNode tableNode = (PanelNode) tableModel.getValueAt(i, 0);
			if (node.equals(tableNode)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Check if a node is already contained in the list.
	 *  
	 */
	private boolean isContained(PanelNode what) {
		boolean contained = false;
		int nrOfRows = table.getRowCount();
		for (int i = 0; i < nrOfRows; i++) {
			PanelNode node = (PanelNode) tableModel.getValueAt(i, 0);
			if (node.equals(what)) {
				contained = true;
				break;
			}
		}
		return contained;
	}

	/**
	 * Update thread. Implementation of the Runnable interface.
	 */
	public void run() {
		// thread performing updates for items in the list
		while (true) {
			synchronized (lock) {
				// loop over rows
				int nrOfRows = table.getRowCount();
				for (int i = 0; i < nrOfRows; i++) {
					PanelNode node = (PanelNode) tableModel.getValueAt(i, 0);
					TreeNode path = node.getRoot();
					if (path.equals(node)) {
						// deleted node
						tableModel.removeRow(i);
					}
					else {
						// the update itself is delegated to the JMC tree and
						// the normal update engine
						jmcTree.refresh(node);
					}
				}
			}
			try {
				Thread.sleep(sleepTime);
			}
			catch (InterruptedException ie) {
			}
			if (stopKicker == true) {
				return;
			}
		}
	}

	/**
	 * Overwrite show() to find a nice position on screen
	 */
	public void show() {
		int xpos = (int) (client.getLocation().getX() + client.getWidth() / 2 - getPreferredSize()
				.getWidth() / 2);
		int ypos = (int) (client.getLocation().getY() + client.getHeight() / 2 - getPreferredSize()
				.getHeight() / 2);
		setLocation(xpos, ypos);
		super.show();
	}
}