/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.command;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 * This class manages the execute command plugin
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: CommandPlugin.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class CommandPlugin extends TaskPlugable {

	private class SettingsListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			showDefaultDialog();
		}
	}
	// plugin in specific user defaults
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.plugins.command.ResourceStrings");
	private CommandDefaults commandDefaults;

	/**
	 * Return a new container for this plugin
	 * 
	 * @param parentContainer
	 * @return new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {
		// build new plugin container
		CommandContainer container = new CommandContainer(parentContainer);
		container.setName(res.getString("INITIAL_NAME") + counter);
		container.setIcon(new ImageIcon(getIconImage()));
		// apply defaults
		container.setCommandDirectory(commandDefaults.getCommandDirectory());
		// count instance
		counter++;
		return container;
	}

	/**
	 * Gets the icon path
	 * 
	 * @return icon path string
	 */
	public String getIconPath() {
		return "com/pallas/unicore/client/plugins/command/command.gif";
	}

	/**
	 * @return "Command" String
	 */
	public String getName() {
		return "Command";
	}

	/**
	 * Build a new panel that manages the settings of a plugin container
	 * 
	 * @param tCont
	 *            plugin container to build panel for
	 * @return JPAPanel that represents container settings in GUI
	 */
	public JPAPanel getPanelInstance(ActionContainer tCont) {
		// build new plugin panel
		CommandJPAPanel panel = new CommandJPAPanel(getClient(),
				(CommandContainer) tCont);
		return panel;
	}

	/**
	 * Get a short info message for the plugin
	 * 
	 * @return String with info message
	 */
	public String getPluginInfo() {
		return res.getString("INFO_STRING");
	}

	public JMenuItem getSettingsItem() {
		// handling setting menu events must be done by the plugin writers
		// as this is an optional feature
		JMenuItem settingsItem = new JMenuItem(res.getString("DEFAULT_ITEM"),
				'C');
		settingsItem.addActionListener(new SettingsListener());
		return settingsItem;
	}

	/**
	 * Show a dialog that configures plugin defaults
	 */
	private void showDefaultDialog() {
		CommandDefaultsDialog commandDefaultsDialog = new CommandDefaultsDialog(
				getClient(), commandDefaults);
		commandDefaultsDialog.show();
	}

	/**
	 * Called to inititialize the plugin
	 */
	public void startPlugin() {
		commandDefaults = new CommandDefaults();
	}

	/**
	 * Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}
}