/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.compile;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Date;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.panels.FileExportPanel;
import com.pallas.unicore.client.panels.FileImportPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Panel that represents Plugin Container in the Client GUI's data area
 * 
 * @author Thomas Kentemich
 * @version $Id: CompileJPAPanel.java,v 1.1 2004/05/25 14:58:46 rmenday Exp $
 */
public class CompileJPAPanel extends JPAPanel {

	private UserDefaults userDefaults = ResourceManager.getUserDefaults();
	private FortranContainer container;
	private FileImportPanel importPanel;
	private FileExportPanel exportPanel;
	private CompileTaskOptionPanel optionsPanel;
	private CompilerOptionPanel compilerOptionPanel;
	private LinkerOptionPanel linkerOptionPanel;
	private SourcesPanel sourcePanel;
	private ObjectsPanel objectPanel;
	private JButton load;
	private Vector sources;
	private JTextField nameTextField;
	private JPanel optionsArea;
	private String defaultDirectory;

	/**
	 * Constructor
	 */
	public CompileJPAPanel(JFrame parentFrame, FortranContainer container) {
		super(parentFrame, container);
		this.container = container;
		// build gui
		buildComponents();
	}

	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}
		nameTextField.setText(container.getName());
		optionsPanel.resetValues();
		compilerOptionPanel.resetValues();
		sourcePanel.resetValues();
		objectPanel.resetValues();
		linkerOptionPanel.resetValues();
		importPanel.resetValues();
		exportPanel.resetValues();
	}

	/**
	 * Method will be called whenever this panel is activated
	 * 
	 * @param vsiteChanged
	 *            true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		// if we are online get current storage resources from server
		if (container.getVsite() != null) {
			if (vsiteChanged) {
				// update storage resources for sub panels
				importPanel.updateValues();
				exportPanel.updateValues();
			}
		}
	}

	public void applyValues() {
		// replace spaces in script name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		container.setModifiedTime(new Date());
		optionsPanel.applyValues();
		importPanel.applyValues();
		exportPanel.applyValues();
		sourcePanel.applyValues();
		objectPanel.applyValues();
		compilerOptionPanel.applyValues();
		linkerOptionPanel.applyValues();
	}

	private void buildComponents() {
		nameTextField = new JTextField(20);
		nameTextField.setToolTipText("Enter identifier for this command");
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.EAST;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		JPanel labelTextPanel = new JPanel(gridbag);
		addToGridBag(new JLabel("Task Name: "), nameTextField, constraints,
				gridbag, labelTextPanel);
		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));
		controlPanel.add(labelTextPanel);
		optionsPanel = new CompileTaskOptionPanel(container);
		compilerOptionPanel = new CompilerOptionPanel(container);
		sourcePanel = new SourcesPanel(container);
		objectPanel = new ObjectsPanel(container, sourcePanel);
		linkerOptionPanel = new LinkerOptionPanel(container);
		importPanel = new FileImportPanel(container);
		exportPanel = new FileExportPanel(container);
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(sourcePanel, "Sources");
		tabbedPane.add(objectPanel, "Objects");
		tabbedPane.add(compilerOptionPanel, "Compiler Options");
		tabbedPane.add(linkerOptionPanel, "Linker Options");
		tabbedPane.add(importPanel, "File Imports");
		tabbedPane.add(exportPanel, "File Exports");
		tabbedPane.add(optionsPanel, "Compile Task Options");
		add(controlPanel, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);
		setBorder(BorderFactory.createEtchedBorder());
		//setBorder(new TitledBorder("Compile and Link"));
		revalidate();
	}

	private void addToGridBag(JLabel label, JComponent component,
			GridBagConstraints constraints, GridBagLayout gridbag, JPanel panel) {
		constraints.gridwidth = GridBagConstraints.RELATIVE;
		gridbag.setConstraints(label, constraints);
		panel.add(label);
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(component, constraints);
		panel.add(component);
	}
}