/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.compile;

import java.util.logging.Level;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.DoubleLength;
import org.unicore.ajo.FortranSourceForm;
import org.unicore.ajo.FortranTask;
import org.unicore.ajo.IntegerLength;
import org.unicore.ajo.LinkTask;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.PreprocessLevel;
import org.unicore.ajo.RealLength;
import org.unicore.sets.OptionSet;
import org.unicore.sets.PortfolioEnumeration;
import org.unicore.sets.PortfolioList;
import org.unicore.sets.PortfolioSet;
import org.unicore.sets.ResourceSet;
import com.pallas.unicore.container.CompileContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Thomas Kentemich
 * @version $Id: FortranContainer.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class FortranContainer extends CompileContainer {

	final static long serialVersionUID = 4983481768961721914L;
	private boolean checkArrayBounds;
	private boolean checkArguments;
	private PreprocessLevel preprocessLevel;
	private IntegerLength integerLength;
	private RealLength realLength;
	private DoubleLength doubleLength;
	private FortranSourceForm sourceForm;
	private OptionSet defines;
	private OptionSet undefines;

	public FortranContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	public void setCheckArrayBounds(boolean check) {
		this.checkArrayBounds = check;
	}

	public void setCheckArguments(boolean check) {
		this.checkArguments = check;
	}

	public void setPreprocessLevel(PreprocessLevel level) {
		this.preprocessLevel = level;
	}

	public void setIntegerLength(IntegerLength len) {
		this.integerLength = len;
	}

	public void setRealLength(RealLength len) {
		this.realLength = len;
	}

	public void setDoubleLength(DoubleLength len) {
		this.doubleLength = len;
	}

	public void setSourceForm(FortranSourceForm form) {
		this.sourceForm = form;
	}

	public void setDefines(OptionSet defines) {
		this.defines = defines;
	}

	public void setUndefines(OptionSet undefines) {
		this.undefines = undefines;
	}

	public boolean getCheckArrayBounds() {
		return checkArrayBounds;
	}

	public boolean getCheckArguments() {
		return checkArguments;
	}

	public PreprocessLevel getPreprocessLevel() {
		return preprocessLevel;
	}

	public IntegerLength getIntegerLength() {
		return integerLength;
	}

	public RealLength getRealLength() {
		return realLength;
	}

	public DoubleLength getDoubleLength() {
		return doubleLength;
	}

	public FortranSourceForm getSourceForm() {
		return sourceForm;
	}

	public OptionSet getDefines() {
		return defines;
	}

	public OptionSet getUndefines() {
		return undefines;
	}

	protected void buildExecuteGroup() {
		if (getSources() == null) {
			return;
		}
		executeGroup = new ActionGroup(getName() + "_EXECUTION");
		// get task resources from job container
		NamedResourceSet taskResourceSet = getResourceSet();
		ResourceSet uspaceResourceSet = new ResourceSet(taskResourceSet
				.getUspace());
		MakePortfolio makePortfolio = new MakePortfolio(getName() + "_Sources");
		makePortfolio.setFiles(getSources());
		String[] includes = getIncludes();
		MakePortfolio includePortfolio = null;
		PortfolioList pList = null;
		if (includes != null && includes.length > 0) {
			includePortfolio = new MakePortfolio(getName() + "_Includes");
			includePortfolio.setFiles(includes);
			pList = new PortfolioList(includePortfolio.getPortfolio());
		}
		FortranTask compileTask = new FortranTask(getName(),
		// name
				null,
				// extra information
				taskResourceSet.getResourceSet(),
				// resource limits
				getEnv(),
				// environment variables
				getCommandLine(),
				// command line string
				null,
				// stdin
				getRedirectStdout(),
				// redirect_stdout
				getRedirectStderr(),
				// redirect stderr
				isVerboseOn(),
				// verbose
				isVersionOn(),
				// version
				new PortfolioSet(makePortfolio.getPortfolio()),
				// sources to compile
				getListingLevel(),
				// listing
				getOptimisationLevel(),
				// optimize
				pList,
				// include files
				getCompileForDebug(),
				// debug level
				getCompileForProfiling(),
				// profiling level
				getCheckArrayBounds(),
				// array bounds check
				getCheckArguments(),
				//argument checking
				getIntegerLength(),
				// int length
				getRealLength(),
				// real length
				getDoubleLength(),
				//double length
				getSourceForm(),
				// source code format
				getPreprocessLevel(),
				// preprocessor
				getDefines(),
				// preprocessor defines
				getUndefines());
		// proprocessor undef
		this.setObjectsPortfolio(compileTask.getObjects());
		executeGroup.add(makePortfolio);
		executeGroup.add(compileTask);
		try {
			if (includes != null && includes.length > 0) {
				executeGroup.add(includePortfolio);
				executeGroup.addDependency(new Dependency(includePortfolio,
						compileTask));
			}
			executeGroup.addDependency(new Dependency(makePortfolio,
					compileTask));
			if (getRedirectStdout()) {
				compileTask.setStdoutFileName(this.getStdout());
				// delete portfolio in case of loops
				DeletePortfolio delStdoutPortfolio = new DeletePortfolio(
						getName() + "_DELETE_STDOUT_"
								+ ResourceManager.getNextObjectIdentifier());
				delStdoutPortfolio
						.setPortfolio(compileTask.getStdout().getId());
				executeGroup.add(delStdoutPortfolio);
				executeGroup.addDependency(new Dependency(compileTask,
						delStdoutPortfolio));
			}
			if (getRedirectStderr()) {
				compileTask.setStderrFileName(this.getStderr());
				// delete portfolio in case of loops
				DeletePortfolio delStderrPortfolio = new DeletePortfolio(
						getName() + "_DELETE_STDERR_"
								+ ResourceManager.getNextObjectIdentifier());
				delStderrPortfolio
						.setPortfolio(compileTask.getStderr().getId());
				executeGroup.add(delStderrPortfolio);
				executeGroup.addDependency(new Dependency(compileTask,
						delStderrPortfolio));
			}
			MakePortfolio stdinPortfolio = null;
			if (getRedirectStdin()) {
				String[] stdinfiles = {this.getStdin()};
				stdinPortfolio = new MakePortfolio(getName() + "_MP_STDIN_"
						+ ResourceManager.getNextObjectIdentifier(),
				// name String name,
						null,
						// extra information
						uspaceResourceSet,
						// storage resource
						stdinfiles);
				// execute
				compileTask.setStdin(stdinPortfolio.getPortfolio());
				executeGroup.add(stdinPortfolio);
				executeGroup.addDependency(new Dependency(stdinPortfolio,
						compileTask));
			}
			if (this.getObjects() != null) {
				MakePortfolio makeObjectPortfolio = new MakePortfolio(getName()
						+ "_Objects");
				PortfolioEnumeration pEnum = getObjects().elements();
				while (pEnum.hasMoreElements()) {
					makeObjectPortfolio.addFile(pEnum.nextElement().getName());
				}
				// perform link step
				LinkTask linkTask = new LinkTask(getName(),
				// name
						null,
						// extra information
						taskResourceSet.getResourceSet(),
						// resource limits
						getEnv(),
						// environment variables
						getCommandLine(),
						// command line string
						null,
						// stdin
						getRedirectStdout(),
						// redirect_stdout
						getRedirectStderr(),
						// redirect stderr
						isVerboseOn(),
						// verbose
						isVersionOn(),
						// version
						//getObjects(),
						new PortfolioList(makeObjectPortfolio.getPortfolio()),
						// objects
						getLibraries(),
						// libs
						getLinkMapLevel(),
						// link map
						getLinkForDebug(),
						// debug level
						getLinkForProfiling()
				// profiling level
				);
				if (getRedirectStdout()) {
					linkTask.setStdoutFileName(this.getStdout());
				}
				if (getRedirectStderr()) {
					linkTask.setStderrFileName(this.getStderr());
				}
				if (getRedirectStdin()) {
					linkTask.setStdin(stdinPortfolio.getPortfolio());
				}
				executeGroup.add(makeObjectPortfolio);
				executeGroup.add(linkTask);
				executeGroup.addDependency(new Dependency(compileTask,
						makeObjectPortfolio));
				executeGroup.addDependency(new Dependency(makeObjectPortfolio,
						linkTask));
			}
		}
		catch (ActionGroup.InvalidDependencyException ex) {
			logger.log(Level.SEVERE, "Could not add dependency.", ex);
		}
	}
}