/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.proxycertificate;

import java.io.File;
import com.pallas.unicore.client.util.Defaults;

/**
 * Define, load and save command plugin default values
 * 
 * @author Thomas Kentemich
 * @version $Id: ProxyCertificateDefaults.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class ProxyCertificateDefaults extends Defaults {

	/**
	 * String constants defining property keys
	 */
	public final static String KEYSIZE_KEY = "plugin.proxycertificate.keysize",
			LIMITED_KEY = "plugin.proxycertificate.limited",
			VALIDTIME_KEY = "plugin.proxycertificate.validtime";

	/**
	 * Sets the keysize
	 * 
	 * @param keysize
	 *            The new keysize value
	 */
	public void setKeysize(int keysize) {
		put(KEYSIZE_KEY, new Integer(keysize).toString());
	}

	/**
	 * Gets the keysize
	 * 
	 * @return The keysize value
	 */
	public int getKeysize() {
		int keySize = intValue(KEYSIZE_KEY);
		if (keySize == 0) {
			keySize = 512;
		}
		return keySize;
	}

	/**
	 * Sets the limited flag
	 * 
	 * @param limited
	 *            The new limited value
	 */
	public void setLimited(boolean limited) {
		put(LIMITED_KEY, new Boolean(limited).toString());
	}

	/**
	 * Gets the limited flag
	 * 
	 * @return The limited value
	 */
	public boolean getLimited() {
		return booleanValue(LIMITED_KEY);
	}

	/**
	 * Sets the valid time
	 * 
	 * @param valid
	 *            The new valid time value
	 */
	public void setValidTime(long valid) {
		put(VALIDTIME_KEY, new Long(valid).toString());
	}

	/**
	 * Gets the validTime
	 * 
	 * @return The valid Time value
	 */
	public long getValidTime() {
		long valid = longValue(VALIDTIME_KEY);
		if (valid == 0) {
			valid = 43200000;
		}
		return valid;
	}

	/**
	 * Load command default values from file
	 * ${HOME}/.unicoreXX/ProxyCertificatesDefaults.txt
	 * 
	 * @return true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		return loadFromFile(unicoreDir + File.separator
				+ "ProxyCertificateDefaults.txt");
	}

	/**
	 * Try to write script default values to file
	 * ${HOME}/.unicoreXX/ProxyCertificates.txt
	 * 
	 * @return true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/ProxyCertificates.txt
		return writeToFile(unicoreDir + File.separator
				+ "ProxyCertificateDefaults.txt");
	}

	/**
	 * Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(KEYSIZE_KEY, new Integer(512).toString());
		put(LIMITED_KEY, new Boolean(false).toString());
		put(VALIDTIME_KEY, new Long(1000L * 60 * 60 * 12).toString());
	}
}