/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.tables;

import java.io.Serializable;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * Class to keep file permissions Used to start Renderer and Editor in tables
 * depending on the value type
 * 
 * @author Klaus-Dieter Oertel
 * @version $Id: FilePermission.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FilePermission implements Serializable, Cloneable {

	/**
	 * Java logger
	 */
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.tables");
	private boolean executePermission = false;
	private boolean readPermission = false;
	private boolean writePermission = false;

	/**
	 * Constructor for the FilePermissionPanel object
	 */
	public FilePermission() {
	}

	public boolean getExecutePermission() {
		return executePermission;
	}

	public boolean getReadPermission() {
		return readPermission;
	}

	public boolean getWritePermission() {
		return writePermission;
	}

	public void setExecutePermission(boolean executePermission) {
		this.executePermission = executePermission;
	}

	/**
	 * Construct from String representation
	 * 
	 * Valid Strings are only substrings of "rwx"
	 */
	public FilePermission setPermission(String perm) {
		int pos = 0;
		if (perm.indexOf("r", pos) == pos) {
			readPermission = true;
			pos++;
		}
		if (perm.indexOf("w", pos) == pos) {
			writePermission = true;
			pos++;
		}
		if (perm.indexOf("x", pos) == pos) {
			executePermission = true;
			pos++;
		}
		if (pos != perm.length()) {
			System.err.println("INVALID FILE PERMISSION: " + perm + "(pos "
					+ pos + ", length " + perm.length());
		}
		System.out.println("Mapped string permission " + perm
				+ " to FilePermission " + this.toString());
		//	new Exception().printStackTrace();
		return this;
	}

	public void setReadPermission(boolean readPermission) {
		this.readPermission = readPermission;
	}

	public void setWritePermission(boolean writePermission) {
		this.writePermission = writePermission;
	}

	public String toString() {
		/*
		 * int perm = 0;
		 * 
		 * if (readPermission) perm += 4; if (writePermission) perm += 2; if
		 * (executePermission) perm += 1;
		 * 
		 * return (new Integer(perm)).toString();
		 */
		String permission = "";
		if (readPermission)
			permission += "r";
		if (writePermission)
			permission += "w";
		if (executePermission)
			permission += "x";
		return permission;
	}

	public Object clone() {
		FilePermission permissionClone = null;
		try {
			permissionClone = (FilePermission) super.clone();
			permissionClone.setReadPermission(this.readPermission);
			permissionClone.setWritePermission(this.writePermission);
			permissionClone.setExecutePermission(this.executePermission);
		}
		catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning file permissions" + this
					+ " failed.", e);
		}
		return permissionClone;
	}
}