/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.tables;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 * @author Ralf Ratering
 * @author Klaus-Dieter Oertel
 * @version $Id: FilePermissionPanel.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FilePermissionPanel extends JPanel implements TableCellRenderer {

	
	private class CheckBoxListener implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			// FilePermission = new FilePermission();
			if (e.getSource().equals(readCheckBox)) {
				permissions.setReadPermission(readCheckBox.isSelected());
			}
			if (e.getSource().equals(writeCheckBox)) {
				permissions.setWritePermission(writeCheckBox.isSelected());
			}
			if (e.getSource().equals(executeCheckBox)) {
				permissions.setExecutePermission(executeCheckBox.isSelected());
			}
		}
	}
	private FilePermission permissions;
	private JCheckBox readCheckBox, writeCheckBox, executeCheckBox;

	public FilePermissionPanel() {
		buildComponents();
	}

	public void addActionListener(ActionListener listener) {
		readCheckBox.addActionListener(listener);
		writeCheckBox.addActionListener(listener);
		executeCheckBox.addActionListener(listener);
	}

	private void buildComponents() {
		readCheckBox = new JCheckBox("r");
		writeCheckBox = new JCheckBox("w");
		executeCheckBox = new JCheckBox("x");

		CheckBoxListener listener = new CheckBoxListener();
		readCheckBox.addActionListener(listener);
		writeCheckBox.addActionListener(listener);
		executeCheckBox.addActionListener(listener);

		setLayout(new GridLayout(1, 5));
		add(readCheckBox);
		add(new JLabel());
		add(writeCheckBox);
		add(new JLabel());
		add(executeCheckBox);
	}

	
	protected FilePermission getFilePermission() {
		return permissions;
	}


	public Component getTableCellRendererComponent(
		JTable table,
		Object permissions,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		if (permissions instanceof FilePermission) {
			setFilePermission((FilePermission) permissions);
		} else {
			// this should never happen ...
			setFilePermission(
				new FilePermission().setPermission(permissions.toString()));
		}
		return this;
	}

	
	protected void setFilePermission(FilePermission permissions) {
		this.permissions = permissions;
		updateCheckBoxes();
	}

	
	protected void updateCheckBoxes() {
		readCheckBox.setSelected(permissions.getReadPermission());
		writeCheckBox.setSelected(permissions.getWritePermission());
		executeCheckBox.setSelected(permissions.getExecutePermission());
	}
}
