/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.tables;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import com.pallas.unicore.extensions.VsiteEntry;
import com.pallas.unicore.utility.VectorSorter;

/**
 * @author Thomas Kentemich
 * @version $Id: VsiteListTable.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class VsiteListTable extends AdjustTable {

	protected static class StatusTableCellRenderer implements TableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Boolean valueString = (Boolean) value;
			JLabel label = new JLabel(value.toString());
			label.setOpaque(true);
			label.setFont(table.getFont());
			if (valueString.booleanValue()) {
				label.setForeground(Color.GREEN);
			}
			else {
				label.setForeground(Color.RED);
			}
			if (isSelected) {
				label.setBackground(table.getSelectionBackground());
			}
			else {
				label.setBackground(table.getBackground());
			}
			return label;
		}
	}

	public class VsiteTableModel extends DefaultTableModel {

		public VsiteTableModel(Object[] names, int rows) {
			super(names, rows);
		}

		public Class getColumnClass(int c) {
			Object columnObject = getValueAt(0, c);
			if (columnObject == null) {
				String dummy = new String();
				return dummy.getClass();
			}
			else {
				return columnObject.getClass();
			}
		}

		public boolean isCellEditable(int row, int col) {
			if (col == 2) {
				return true;
			}
			else {
				return false;
			}
		}
	}

	protected class TableHeaderListener extends MouseAdapter {

		public TableHeaderListener() {
		}

		public void mouseClicked(MouseEvent e) {
			TableColumnModel columnModel = getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			int column = convertColumnIndexToModel(viewColumn);
			sortByColumn(column);
		}
	}
	private final static String[] columnIdentifiers = {"Usite", "Vsite",
			"Select"};
	private boolean ascending;
	private boolean initialized = false;
	private int lastSortColumn = -1;
	private VsiteTableModel model;
	private static TableCellRenderer statusCellRenderer = new StatusTableCellRenderer();

	public VsiteListTable() {
		super();
		buildComponents();
		getTableHeader().addMouseListener(new TableHeaderListener());
		// revalidate();
	}

	private void adjustWidthsAndLookForIterations(Vector rows) {
		adjustWidths();
	}

	private void buildComponents() {
		model = new VsiteTableModel(columnIdentifiers, 0);
		model.setColumnIdentifiers(columnIdentifiers);
		setModel(model);
		//getColumnModel().getColumn(2).setCellRenderer(statusCellRenderer);
		setFont(new Font(getFont().getName(), getFont().getStyle(), 11));
	}

	public void clear() {
		int rowCount = getRowCount();
		for (int i = 0; i < rowCount; i++) {
			model.removeRow(0);
		}
	}

	public void resort() {
		ascending = !ascending;
		sortByColumn(lastSortColumn);
	}

	public void setDataVector(Vector rows) {
		this.clear();
		model.setRowCount(0);
		for (int i = 0; i < rows.size(); i++) {
			model.addRow((Vector) rows.elementAt(i));
		}
		resort();
		if (rows.size() > 0 && !initialized) {
			// adjustWidthsAndLookForIterations(rows);
			initialized = true;
		}
	}

	public void sortByColumn(int column) {
		if (column < 0) {
			return;
		}
		if (lastSortColumn == column) {
			ascending = !ascending;
		}
		else {
			ascending = true;
		}
		Vector dataVector = model.getDataVector();
		VsiteEntry.setCompareIndex(column, 0);
		VectorSorter.sort(dataVector, ascending);
		lastSortColumn = column;
	}
}