/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.themes;

import java.awt.SystemColor;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * This class describes a theme using system colors.
 * 
 * @author Ralf Ratering
 * @version $Id: SystemTheme.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class SystemTheme extends DefaultMetalTheme {

	public static void main(String args[]) {
		SystemTheme c = new SystemTheme();
		System.err.println("PRIMARY1: " + c.getPrimary1());
		System.err.println("PRIMARY2: " + c.getPrimary2());
		System.err.println("PRIMARY3: " + c.getPrimary3());
		System.err.println("SECONDARY1: " + c.getSecondary1());
		System.err.println("SECONDARY2: " + c.getSecondary2());
		System.err.println("SECONDARY3: " + c.getSecondary3());
		MetalLookAndFeel.setCurrentTheme(c);
	}
	// labels & borders
	private final ColorUIResource primary1 = new ColorUIResource(
			SystemColor.controlText);
	// selected menus, scrollbars, progress bars
	private final ColorUIResource primary2 = new ColorUIResource(
			SystemColor.controlShadow);
	// title background, selected text
	private final ColorUIResource primary3 = new ColorUIResource(
			SystemColor.controlHighlight);
	// 3D border effects
	private final ColorUIResource secondary1 = new ColorUIResource(
			SystemColor.controlDkShadow);
	// pushed buttons, disabled button text, inactive borders
	private final ColorUIResource secondary2 = new ColorUIResource(
			SystemColor.controlShadow);
	// background canvas, inactive titles
	private final ColorUIResource secondary3 = new ColorUIResource(
			SystemColor.menu);

	public String getName() {
		return "System";
	}

	protected ColorUIResource getPrimary1() {
		return primary1;
	}

	protected ColorUIResource getPrimary2() {
		return primary2;
	}

	protected ColorUIResource getPrimary3() {
		return primary3;
	}

	protected ColorUIResource getSecondary1() {
		return secondary1;
	}

	protected ColorUIResource getSecondary2() {
		return secondary2;
	}

	protected ColorUIResource getSecondary3() {
		return secondary3;
	}
}