/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.themes;

import java.awt.Color;
import java.awt.Font;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;

/**
 * A theme that can be modified by the user.
 * 
 * @author Ralf Ratering
 * @version $Id: UserTheme.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class UserTheme extends DefaultMetalTheme {

	protected FontUIResource controlFont = super.getControlTextFont(),
			systemFont = super.getSystemTextFont(), userFont = super
					.getUserTextFont(), menuFont = super.getMenuTextFont(),
			titleFont = super.getWindowTitleFont(), subFont = super
					.getSubTextFont();
	private String name = "User Defined";
	private ColorUIResource primary1 = super.getPrimary1(), primary2 = super
			.getPrimary2(), primary3 = super.getPrimary3(), secondary1 = super
			.getSecondary1(), secondary2 = super.getSecondary2(),
			secondary3 = super.getSecondary3(), black = super.getBlack(),
			white = super.getWhite();

	protected ColorUIResource getBlack() {
		return black;
	}

	public Color getBlackColor() {
		return black;
	}

	public FontUIResource getControlTextFont() {
		return controlFont;
	}

	public FontUIResource getMenuTextFont() {
		return menuFont;
	}

	public String getName() {
		return name;
	}

	protected ColorUIResource getPrimary1() {
		return primary1;
	}

	public Color getPrimary1Color() {
		return primary1;
	}

	protected ColorUIResource getPrimary2() {
		return primary2;
	}

	public Color getPrimary2Color() {
		return primary2;
	}

	protected ColorUIResource getPrimary3() {
		return primary3;
	}

	public Color getPrimary3Color() {
		return primary3;
	}

	protected ColorUIResource getSecondary1() {
		return secondary1;
	}

	public Color getSecondary1Color() {
		return secondary1;
	}

	protected ColorUIResource getSecondary2() {
		return secondary2;
	}

	public Color getSecondary2Color() {
		return secondary2;
	}

	protected ColorUIResource getSecondary3() {
		return secondary3;
	}

	public Color getSecondary3Color() {
		return secondary3;
	}

	public FontUIResource getSubTextFont() {
		return subFont;
	}

	public FontUIResource getSystemTextFont() {
		return systemFont;
	}

	public FontUIResource getUserTextFont() {
		return userFont;
	}

	protected ColorUIResource getWhite() {
		return white;
	}

	public Color getWhiteColor() {
		return white;
	}

	public FontUIResource getWindowTitleFont() {
		return titleFont;
	}

	public void setBlackColor(Color b) {
		this.black = new ColorUIResource(b);
	}

	public void setFont(Font font) {
		controlFont = new FontUIResource(font);
		systemFont = new FontUIResource(font);
		userFont = new FontUIResource(font);
		menuFont = new FontUIResource(font);
		titleFont = new FontUIResource(font);
		subFont = new FontUIResource(font);
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setPrimary1Color(Color p1) {
		this.primary1 = new ColorUIResource(p1);
	}

	public void setPrimary2Color(Color p2) {
		this.primary2 = new ColorUIResource(p2);
	}

	public void setPrimary3Color(Color p3) {
		this.primary3 = new ColorUIResource(p3);
	}

	public void setSecondary1Color(Color s1) {
		this.secondary1 = new ColorUIResource(s1);
	}

	public void setSecondary2Color(Color s2) {
		this.secondary2 = new ColorUIResource(s2);
	}

	public void setSecondary3Color(Color s3) {
		this.secondary3 = new ColorUIResource(s3);
	}

	public void setWhiteColor(Color w) {
		this.white = new ColorUIResource(w);
	}
}