/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.trees;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Vector;
import javax.swing.ImageIcon;
import org.unicore.ajo.AbstractAction;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.Outcome;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;

/**
 * Display an AbstractAction in as a tree node.
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: AjoNode.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class AjoNode extends PanelNode {

	private String objectName = "UNKNOWN";

	/**
	 * Constructor
	 * 
	 * @param object
	 *            AbstactAction stored with node
	 */
	public AjoNode(Object object) {
		super(object);
		// get node name from class
		String className = object.getClass().getName();
		String packageName = object.getClass().getPackage().getName();
		objectName = className.substring(packageName.length() + 1);
	}

	/**
	 * Used for node label
	 */
	public String toString() {
		return objectName;
	}

	/**
	 * Get action status from outcome entry
	 *  
	 */
	public void updateActionStatus() {
		AbstractActionStatus status = null;
		Vector outcomes = null;
		outcomes = OutcomeManager
				.getOutcomeEntries((AbstractAction) getUserObject());
		if (outcomes.size() != 0) {
			Outcome outcome = ((OutcomeEntry) outcomes.elementAt(outcomes
					.size() - 1)).getOutcome();
			if (outcome != null) {
				status = outcome.getStatus();
			}
			else {
				status = null;
			}
			setIcon(updateIcon(getColor(status), getIcon()));
		}
		else {
			setIcon(updateIcon(getColor(null), getIcon()));
		}
	}

	/**
	 * Select icon color according to action status
	 * 
	 * @param status
	 *            current status
	 * @return The color value
	 */
	private Color getColor(AbstractActionStatus status) {
		// no status == undefined
		if (status == null) {
			return Color.gray;
		}
		if (status.equals(AbstractActionStatus.PENDING)) {
			return Color.magenta;
		}
		else if (status.equals(AbstractActionStatus.QUEUED)) {
			return Color.blue;
		}
		else if (status.equals(AbstractActionStatus.RUNNING)) {
			return Color.orange;
		}
		else if (status.equals(AbstractActionStatus.CONSIGNED)) {
			return Color.blue.brighter();
		}
		else if (status.equals(AbstractActionStatus.EXECUTING)) {
			return Color.yellow;
		}
		else if (status.equals(AbstractActionStatus.HELD)) {
			return Color.white;
		}
		else if (status.equals(AbstractActionStatus.READY)) {
			return Color.cyan;
		}
		else if (status.equals(AbstractActionStatus.KILLED)) {
			return Color.black;
		}
		else if (status.equals(AbstractActionStatus.SUCCESSFUL)) {
			return Color.green;
		}
		else if (status.equals(AbstractActionStatus.NEVER_RUN)) {
			return Color.gray;
		}
		else if (status.isEquivalent(AbstractActionStatus.NOT_DONE)) {
			return Color.yellow;
		}
		else if (status.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
			return Color.red;
		}
		return Color.gray;
	}

	private ImageIcon updateIcon(Color color, ImageIcon originalIcon) {
		int width = originalIcon.getIconWidth();
		int height = originalIcon.getIconHeight();
		BufferedImage offscreenImage = null;
		offscreenImage = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = offscreenImage.createGraphics();
		g2d.drawImage(originalIcon.getImage(), 0, 0, width, height, null);
		g2d.setColor(color);
		g2d.fillRect(0, height - 4, width - 1, 3);
		g2d.setColor(Color.black);
		g2d.drawRect(0, height - 4, width, 4);
		if (((AbstractAction) getUserObject())
				.propagateSuccessfulIfNotSuccessful()) {
			g2d.setColor(Color.RED);
			int[] xPoints = {width - 8, width - 1, width - 1};
			int[] yPoints = {1, 1, 8};
			g2d.fillPolygon(xPoints, yPoints, 3);
			g2d.setColor(Color.black);
			g2d.drawLine(xPoints[0], yPoints[0], xPoints[2], yPoints[2]);
		}
		ImageIcon statusIcon = new ImageIcon();
		statusIcon.setImage(offscreenImage);
		return statusIcon;
	}
}