/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.trees;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.tree.TreePath;
import org.unicore.Identifier;
import org.unicore.User;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.dialogs.ShowLogDialog;
import com.pallas.unicore.client.graphs.DAGVertex;
import com.pallas.unicore.client.graphs.DependencyEditor;
import com.pallas.unicore.client.graphs.TreeSorter;
import com.pallas.unicore.client.graphs.VertexsTreeComparable;
import com.pallas.unicore.client.panels.DoNPanel;
import com.pallas.unicore.client.panels.DoRepeatPanel;
import com.pallas.unicore.client.panels.ExportJPAPanel;
import com.pallas.unicore.client.panels.GroupPanel;
import com.pallas.unicore.client.panels.HoldJobPanel;
import com.pallas.unicore.client.panels.IfPanel;
import com.pallas.unicore.client.panels.ImportJPAPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.panels.JobGroupPanel;
import com.pallas.unicore.client.panels.OperationJPAPanel;
import com.pallas.unicore.client.panels.TransferJPAPanel;
import com.pallas.unicore.client.util.ClientPluginManager;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.DependencyContainer;
import com.pallas.unicore.container.DoNContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.container.ElseContainer;
import com.pallas.unicore.container.FileExportContainer;
import com.pallas.unicore.container.FileImportContainer;
import com.pallas.unicore.container.FileOperationContainer;
import com.pallas.unicore.container.FileTransferContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.container.ThenContainer;
import com.pallas.unicore.requests.SubmitJobNode;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.TransferableVector;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorSorter;

/**
 * A tree that shows jobs and tasks for the JPA.
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: JPATree.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class JPATree extends ClientTree implements IObserver {

	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.trees.ResourceStrings");
	private Vector addActions, addPluginActions;
	private AddDoNAction addDoNAction = new AddDoNAction();
	private AddDoRepeatAction addDoRepeatAction = new AddDoRepeatAction();
	private AddExportAction addExportAction = new AddExportAction();
	private AddFileOperationAction addFileOperationAction = new AddFileOperationAction();
	private AddHoldJobAction addHoldJobAction = new AddHoldJobAction();
	private AddIfThenElseAction addIfThenElseAction = new AddIfThenElseAction();
	private AddImportAction addImportAction = new AddImportAction();
	private JMenu addMenu = new JMenu("Add");
	private AddSubJobAction addSubJobAction = new AddSubJobAction();
	private AddTransferAction addTransferAction = new AddTransferAction();
	private CheckAction checkAction = new CheckAction();
	private NewJobAction newJobAction = new NewJobAction();
	private OpenAction openAction = new OpenAction();
	private SaveAction saveAction = new SaveAction();
	private SaveAsAction saveAsAction = new SaveAsAction();
	private SortAction sortAction = new SortAction();
	private SubmitAction submitAction = new SubmitAction();

	/**
	 * Constructor inits tree
	 * 
	 * @param client
	 *            reference to client
	 */
	public JPATree(Client client) {
		super(client);
		// root node simply displays "JPA"
		root.setUserObject(res.getString("JPA"));
		root.setUpToDate();
		// popup menu stuff
		buildPopupMenu();
	}

	private void addGroupAt(int index, GroupContainer container,
			PanelNode parentNode, boolean select) {
		JPAPanel panel = null;
		if (container instanceof DoNContainer) {
			panel = new DoNPanel(client, (DoNContainer) container);
		}
		else if (container instanceof DoRepeatContainer) {
			panel = new DoRepeatPanel(client, (DoRepeatContainer) container);
		}
		else if (container instanceof IfContainer) {
			panel = new IfPanel(client, (IfContainer) container);
		}
		else if (container instanceof ThenContainer) {
			panel = (IfPanel) parentNode.getPanel();
		}
		else if (container instanceof ElseContainer) {
			panel = (IfPanel) parentNode.getPanel();
		}
		else {
			//handle plugged GroupContainers
			try {
				Class containerClass = container.getClass();
				Vector plugins = (Vector) Client.getPluginManager()
						.getTaskPlugins();
				for (int i = 0; i < plugins.size(); i++) {
					if (((TaskPlugable) plugins.elementAt(i))
							.hasContainerClass(containerClass)) {
						panel = ((TaskPlugable) plugins.elementAt(i))
								.getPanelInstance(container);
					}
				}
			}
			catch (Exception e) {
				logger.severe(e.getMessage());
			}
			//handle non-plugged JobContainer
			if (panel == null && container instanceof JobContainer) {
				panel = new JobGroupPanel(client, (JobContainer) container);
			}
			if (panel == null) //container type not known
			{
				logger.severe("Wrong container type : " + container.getClass()
						+ " in method.");
				return;
			}
		}
		//	   apply container values
		panel.resetValues();
		//	   build new node
		ContainerNode node = new ContainerNode(container, panel);
		addGroupNodeAt(index, node, parentNode, select);
	}

	private void addGroupNodeAt(int index, PanelNode groupNode,
			PanelNode parentNode, boolean select) {
		insertNodeAt(index, groupNode, parentNode, select);
		GroupContainer container = (GroupContainer) groupNode.getUserObject();
		// loop over children and insert them recursively to job group node
		Vector tasks = container.getTasks();
		for (int i = 0; i < tasks.size(); i++) {
			ActionContainer task = (ActionContainer) tasks.elementAt(i);
			// if task is a sub job recurse
			if (task instanceof GroupContainer) {
				// add sub job as a child of this job
				addGroupAt(groupNode.getChildCount(), (GroupContainer) task,
						groupNode, false);
			}
			// the only TaskContainer that's not a plugin
			else if (task instanceof HoldJobContainer) {
				HoldJobPanel jpaPanel = new HoldJobPanel(client,
						(HoldJobContainer) task);
				jpaPanel.resetValues();
				insertContainerNodeAt(groupNode.getChildCount(), task,
						jpaPanel, groupNode, false);
			}
			// for other tasks get jpa panel from plugin manager and insert node
			else if (task instanceof TaskContainer) {
				ClientPluginManager pManager = Client.getPluginManager();
				JPAPanel jpaPanel = pManager
						.getPanelInstance((TaskContainer) task);
				if (jpaPanel != null) {
					jpaPanel.resetValues();
					insertContainerNodeAt(groupNode.getChildCount(), task,
							jpaPanel, groupNode, false);
				}
				else {
					UserMessages.error(res.getString("COULD_NOT_LOAD")
							+ task.getClass().getName()
							+ res.getString("FROM_JOB"));
				}
			}
		}
		// select main job group
		if (select) {
			setSelectionPath(new TreePath(model.getPathToRoot(groupNode)));
		}
		container.setModifiedTime(new Date());
	}

	/**
	 * Add a JobContainer to the tree
	 * 
	 * @param container
	 *            JobContainer to be added
	 * @param index
	 *            where to insert the job
	 * @param select
	 *            true if new job should be selected
	 */
	public void addJobGroupAt(int index, JobContainer container, boolean select) {
		addGroupAt(index, container, root, select);
	}

	private void addNewGroup(GroupContainer container) {
		ContainerNode parentNode = currentGroupNode;
		GroupContainer parentContainer = (GroupContainer) parentNode
				.getActionContainer();
		// never add groups other than else and then directly to if groups
		if (currentGroupNode.isIfGroup()
				&& !(container instanceof ThenContainer)
				&& !(container instanceof ElseContainer)) {
			parentContainer = parentContainer.getParentContainer();
		}
		// link new group and parent with each other
		container.setParentContainer(parentContainer);
		int index = getInsertionIndex();
		parentContainer.addTaskAt(index, container);
		addGroupAt(index, container, parentNode, true);
	}

	private void addNewPluginTask(TaskPlugable plugable) {
		GroupContainer parentContainer = (GroupContainer) currentGroupNode
				.getActionContainer();
		// never add tasks directly to if groups
		if (currentGroupNode.isIfGroup()) {
			parentContainer = parentContainer.getParentContainer();
		}
		ActionContainer actionContainer = plugable
				.getContainerInstance(parentContainer);
		if (actionContainer == null) {
			return;
		}
		actionContainer.setParentContainer(parentContainer);
		JPAPanel pluginPanel = plugable.getPanelInstance(actionContainer);
		pluginPanel.resetValues();
		int index = getInsertionIndex();
		if (actionContainer instanceof GroupContainer) {
			GroupContainer groupContainer = (GroupContainer) actionContainer;
			if (focussedNode.isRoot()) {
				addPluggedInGroupAt(0, groupContainer, root, true, pluginPanel);
			}
			else {
				// new sub-Job to be inserted
				// add new container to parent and vice versa
				parentContainer.addTaskAt(index, groupContainer);
				addPluggedInGroupAt(index, groupContainer, currentGroupNode,
						true, pluginPanel);
			}
		}
		else {
			// single task insertion
			parentContainer.addTaskAt(index, actionContainer);
			addTaskAt(index, actionContainer, pluginPanel, true);
		}
	}

	/**
	 * Adds a Job Group that is controlled by a plugin
	 */
	private void addPluggedInGroupAt(int index, GroupContainer container,
			PanelNode parentNode, boolean select, JPAPanel panel) {
		//apply container values
		panel.resetValues();
		// build new node
		ContainerNode node = new ContainerNode(container, panel);
		addGroupNodeAt(index, node, parentNode, select);
	}

	/**
	 * Add a new task to the currently selected node, this method will not be
	 * called when loading tasks from previously saved job groups
	 */
	private void addTaskAt(int index, ActionContainer tc, JPAPanel panel,
			boolean select) {
		// update JobGroupPanel to redraw dependency graph
		//((JPAPanel)currentGroupNode.getPanel()).resetValues();
		insertContainerNodeAt(index, tc, panel, currentGroupNode, select);
		tc.setModifiedTime(new Date());
	}

	/**
	 * Apply values to a node
	 * 
	 * @param node
	 */
	public void applyValues(ContainerNode node) {
		JPanel panel = node.getPanel();
		if (panel instanceof JPAPanel) {
			JPAPanel jpaPanel = (JPAPanel) panel;
			jpaPanel.applyValues();
		}
		if (node.isGroup()) {
			for (int i = 0; i < node.getChildCount(); i++) {
				applyValues((ContainerNode) node.getChildAt(i));
			}
		}
	}

	/**
	 * build pop up for jpa tree
	 */
	public void buildPopupMenu() {
		addActions = new Vector();
		addActions.add(addSubJobAction);
		addActions.add(addDoNAction);
		addActions.add(addDoRepeatAction);
		addActions.add(addHoldJobAction);
		addActions.add(addIfThenElseAction);
		addActions.add(addImportAction);
		addActions.add(addExportAction);
		addActions.add(addTransferAction);
		addActions.add(addFileOperationAction);
		addMenu.removeAll();
		addMenu.setIcon(ResourceManager.getIcon(ResourceManager.ADD));
		addMenu.add(new JMenuItem(addSubJobAction));
		addMenu.addSeparator();
		Vector plugins = (Vector) Client.getPluginManager().getTaskPlugins();
		addPluginActions = new Vector();
		for (int i = 0; i < plugins.size(); i++) {
			TaskPlugable plugable = (TaskPlugable) plugins.elementAt(i);
			AddPluginAction action = new AddPluginAction(plugable);
			action.setEnabled(isRootSelected());
			addActions.add(action);
			addPluginActions.add(action);
			addMenu.add(new JMenuItem(action));
		}
		if (plugins.size() > 0) {
			addMenu.addSeparator();
		}
		addMenu.add(new JMenuItem(addDoNAction));
		addMenu.add(new JMenuItem(addDoRepeatAction));
		addMenu.add(new JMenuItem(addHoldJobAction));
		addMenu.add(new JMenuItem(addIfThenElseAction));
		addMenu.addSeparator();
		addMenu.add(new JMenuItem(addImportAction));
		addMenu.add(new JMenuItem(addExportAction));
		addMenu.add(new JMenuItem(addTransferAction));
		addMenu.add(new JMenuItem(addFileOperationAction));
		popup.removeAll();
		popup.setInvoker(this);
		popup.add(newJobAction);
		popup.add(saveAction);
		popup.add(addMenu);
		popup.add(cutAction);
		popup.add(copyAction);
		popup.add(pasteAction);
		popup.add(removeAction);
		addMenu.addSeparator();
		popup.add(new JMenuItem(checkAction));
		popup.add(new JMenuItem(submitAction));
	}

	private boolean checkContainers(ContainerNode node, boolean displayErrors) {
		ActionContainer container = node.getActionContainer();
		String errorMessage = "";
		// if we have selected a job container get errors for complete job
		if (container instanceof JobContainer) {
			JobContainer job = (JobContainer) container;
			errorMessage = job.checkContents().toString();
		}
		else {
			errorMessage = container.checkContents().toString();
		}
		if (errorMessage == "") {
			return true;
		}
		if (displayErrors) {
			ShowLogDialog logDialog = new ShowLogDialog(client, errorMessage,
					"UNICORE: Errors for " + container.getName());
			logDialog.setSize(800, 300);
			logDialog.setEditable(false);
			logDialog.setLocationRelativeTo(client);
			logDialog.setVisible(true);
		}
		return false;
	}

	private ContainerNode cloneNode(ContainerNode node) {
		JPAPanel newPanel = null;
		ActionContainer containerClone = null;
		// copying a single task
		if (node.getActionContainer() instanceof TaskContainer) {
			// get container from node and clone it
			TaskContainer container = (TaskContainer) node.getActionContainer();
			TaskContainer taskClone = (TaskContainer) container.clone();
			// build a new JPAPanel for the container clone
			// we build new panels instead of cloning them, because JPanels are
			// not
			// cloneable
			newPanel = Client.getPluginManager().getPanelInstance(taskClone);
			containerClone = taskClone;
		}
		// copying a group
		else if (node.getActionContainer() instanceof GroupContainer) {
			GroupContainer group = (GroupContainer) node.getActionContainer();
			GroupContainer groupClone = (GroupContainer) group.clone();
			if (groupClone instanceof JobContainer) {
				newPanel = new JobGroupPanel(client, (JobContainer) groupClone);
			}
			else if (groupClone instanceof DoNContainer) {
				newPanel = new DoNPanel(client, (DoNContainer) groupClone);
			}
			else if (groupClone instanceof DoRepeatContainer) {
				newPanel = new DoRepeatPanel(client,
						(DoRepeatContainer) groupClone);
			}
			else if (groupClone instanceof IfContainer) {
				newPanel = new IfPanel(client, (IfContainer) groupClone);
			}
			containerClone = groupClone;
		}
		ContainerNode nodeCopy = new ContainerNode(containerClone, newPanel);
		newPanel.resetValues();
		return nodeCopy;
	}

	/**
	 * Copy a node with renaming
	 */
	protected void copy() {
		copy(true);
	}

	private void copy(boolean rename) {
		TransferableVector copiedNodes = new TransferableVector(PanelNode
				.getFlavor());
		TreePath[] paths = getSelectionPaths();
		Vector selectedNodes = new Vector();
		for (int i = 0; i < paths.length; i++) {
			Object pathComponent = paths[i].getLastPathComponent();
			if (!(pathComponent instanceof ContainerNode)) {
				continue;
			}
			ContainerNode node = (ContainerNode) pathComponent;
			// if then or else was selected, copy complete if task
			if (!node.isThenElseGroup()) {
				selectedNodes.add(node);
			}
		}
		// remove descendants
		Vector selectedParents = new Vector();
		for (int i = 0; i < selectedNodes.size(); i++) {
			ContainerNode node1 = (ContainerNode) selectedNodes.elementAt(i);
			boolean isParent = true;
			for (int j = 0; j < selectedNodes.size(); j++) {
				ContainerNode node2 = (ContainerNode) selectedNodes
						.elementAt(j);
				if (!node1.equals(node2) && node2.isNodeDescendant(node1)) {
					isParent = false;
					break;
				}
			}
			if (isParent) {
				selectedParents.add(node1);
			}
		}
		for (int i = 0; i < selectedParents.size(); i++) {
			ContainerNode node = (ContainerNode) selectedParents.elementAt(i);
			applyValues(node);
			ContainerNode nodeCopy = cloneNode(node);
			if (rename) {
				renameContainer(nodeCopy.getActionContainer());
			}
			try {
				copiedNodes.add(nodeCopy);
			}
			catch (UnsupportedFlavorException e) {
				logger.log(Level.WARNING, "Could copy node " + node, e);
			}
		}
		clipboard.setContents(copiedNodes, this);
		enableDefaultActions();
	}

	/**
	 * Copy a node
	 * 
	 * @param node
	 *            node to be copied
	 * @param rename
	 *            true if copy will be renamed
	 */
	public void copyNode(ContainerNode node, boolean rename) {
		TransferableVector copiedNodes = new TransferableVector(PanelNode
				.getFlavor());
		applyValues(node);
		ContainerNode nodeCopy = cloneNode((ContainerNode) node);
		if (rename) {
			renameContainer(nodeCopy.getActionContainer());
		}
		try {
			copiedNodes.add(nodeCopy);
		}
		catch (UnsupportedFlavorException e) {
			logger.log(Level.WARNING, "Could copy node " + node, e);
		}
		clipboard.setContents(copiedNodes, this);
		enableDefaultActions();
	}

	/**
	 * Cut a node
	 */
	protected void cut() {
		// copy without renaming
		copy(false);
		remove();
	}

	/**
	 * update menus
	 */
	public void enableDefaultActions() {
		if (focussedNode == null) {
			focussedNode = root;
		}
		if (focussedNode.isRoot()) {
			enableRootActions();
			return;
		}
		newJobAction.setEnabled(true);
		openAction.setEnabled(true);
		saveAction.setEnabled(true);
		saveAsAction.setEnabled(true);
		addMenu.setEnabled(true);
		for (int i = 0; i < addActions.size(); i++) {
			((AbstractAction) addActions.elementAt(i)).setEnabled(true);
		}
		boolean enableCutCopyRemove = (getSelectionCount() > 1)
				|| !focussedNode.isThenElseGroup();
		cutAction.setEnabled(enableCutCopyRemove);
		copyAction.setEnabled(enableCutCopyRemove);
		pasteAction.setEnabled(clipboard.getContents(this) != null);
		removeAction.setEnabled(enableCutCopyRemove);
		checkAction.setEnabled(true);
		submitAction.setEnabled(!ResourceManager.isOffline());
		sortAction.setEnabled(focussedNode.isJobGroup());
	}

	private void enableRootActions() {
		newJobAction.setEnabled(true);
		openAction.setEnabled(true);
		saveAction.setEnabled(false);
		saveAsAction.setEnabled(false);
		addMenu.setEnabled(false);
		for (int i = 0; i < addActions.size(); i++) {
			((AbstractAction) addActions.elementAt(i)).setEnabled(false);
		}
		cutAction.setEnabled(false);
		copyAction.setEnabled(false);
		// only paste jobs at root
		boolean pasteEnabled = false;
		TransferableVector vector = (TransferableVector) clipboard
				.getContents(this);
		for (int i = 0; vector != null && i < vector.size(); i++) {
			ContainerNode node = (ContainerNode) vector.elementAt(i);
			if (node.isJobGroup()) {
				pasteEnabled = true;
			}
			else {
				pasteEnabled = false;
				break;
			}
		}
		pasteAction.setEnabled(pasteEnabled);
		removeAction.setEnabled(false);
		checkAction.setEnabled(false);
		submitAction.setEnabled(false);
		sortAction.setEnabled(false);
	}

	public AbstractAction getAddDoNAction() {
		return addDoNAction;
	}

	public AbstractAction getAddDoRepeatAction() {
		return addDoRepeatAction;
	}

	public AbstractAction getAddExportAction() {
		return addExportAction;
	}

	public AbstractAction getAddFileOperationAction() {
		return addFileOperationAction;
	}

	public AbstractAction getAddHoldJobAction() {
		return addHoldJobAction;
	}

	public AbstractAction getAddIfThenElseAction() {
		return addIfThenElseAction;
	}

	public AbstractAction getAddImportAction() {
		return addImportAction;
	}

	public Vector getAddPluginActions() {
		return addPluginActions;
	}

	public AbstractAction getAddSubJobAction() {
		return addSubJobAction;
	}

	public AbstractAction getAddTransferAction() {
		return addTransferAction;
	}

	public AbstractAction getCheckAction() {
		return checkAction;
	}

	public AbstractAction getNewJobAction() {
		return newJobAction;
	}

	public AbstractAction getOpenAction() {
		return openAction;
	}

	public AbstractAction getSaveAction() {
		return saveAction;
	}

	public AbstractAction getSaveAsAction() {
		return saveAsAction;
	}

	public AbstractAction getSortAction() {
		return sortAction;
	}

	public AbstractAction getSubmitAction() {
		return submitAction;
	}

	private int getInsertionIndex() {
		if (focussedNode.isIfGroup()) {
			return currentGroupNode.getParent().getIndex(focussedNode) + 1;
		}
		else if (focussedNode.isGroup()) {
			return 0;
		}
		return currentGroupNode.getIndex(focussedNode) + 1;
	}

	/**
	 * Get all top level jobs
	 * 
	 * @return all top level jobs
	 */
	public Enumeration getJobs() {
		return root.children();
	}

	/**
	 * Test, if there are jobs that have been modified since they were saved.
	 * 
	 * @return true, if there are unsaved jobs
	 */
	public Vector getUnsavedJobs() {
		Vector unsavedJobs = new Vector();
		if (!root.children().hasMoreElements()) {
			return unsavedJobs;
		}
		// first make sure all values have been applied
		updateCurrentJob(false);
		for (int i = 0; i < root.getChildCount(); i++) {
			ContainerNode jobNode = (ContainerNode) root.getChildAt(i);
			JobContainer jc = (JobContainer) jobNode.getActionContainer();
			// if the job has not been saved at all or has been modified,
			// add it to the list
			if (jc.getSavedTime() == null
					|| !jc.getSavedTime().equals(jc.getModifiedTime())) {
				unsavedJobs.add(jc);
			}
		}
		return unsavedJobs;
	}

	private ContainerNode insertContainerNodeAt(int index,
			ActionContainer container, JPAPanel panel, PanelNode parentNode,
			boolean select) {
		// build new node
		ContainerNode node = new ContainerNode(container, panel);
		insertNodeAt(index, node, parentNode, select);
		return node;
	}

	private synchronized void insertNodeAt(int index, PanelNode node,
			PanelNode parentNode, boolean select) {
		Object object = node.getUserObject();
		if (parentNode.isIfGroup()
				&& !((object instanceof ThenContainer) || (object instanceof ElseContainer))) {
			parentNode = (ContainerNode) parentNode.getParent();
		}
		// add node to tree
		model.insertNodeInto(node, parentNode, index);
		// get path to node
		TreePath path = new TreePath(model.getPathToRoot(node));
		// show path to jobgroup node
		//makeVisible(path);
		expandPath(path);
		// select job group node
		if (select) {
			setSelectionPath(path);
		}
	}

	/**
	 * Test if currently selected node is a top level job group
	 * 
	 * @return true if parent of currently selected node is the root node
	 */
	public boolean isTopLevelJobGroupSelected() {
		return focussedNode.getParent().equals(root);
	}

	/**
	 * Implementation of the IObserver interface
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
		// if we have been called from a SubmitJob task
		// reset the job icon
		if (theObserved instanceof SubmitJobNode) {
			ContainerNode jobNode = (ContainerNode) changeCode;
			jobNode.setState(ActionContainer.STATE_UP_TO_DATE);
			((JobContainer) jobNode.getActionContainer()).selectJPAIcon();
			updateNode(jobNode);
		}
	}

	/**
	 * paste a node
	 */
	protected void paste() {
		paste(focussedNode);
	}

	/**
	 * Paste a node from clipboard into tree
	 * 
	 * @param destinationNode
	 *            where to paste the node
	 */
	public void paste(PanelNode destinationNode) {
		Transferable clipboardContent = clipboard.getContents(this);
		try {
			// get node from clipboard
			TransferableVector clipboardNodes = (TransferableVector) clipboardContent
					.getTransferData(PanelNode.getFlavor());
			for (int i = 0; i < clipboardNodes.size(); i++) {
				// clone node
				ContainerNode nodeCopy = cloneNode((ContainerNode) clipboardNodes
						.elementAt(i));
				ActionContainer container = nodeCopy.getActionContainer();
				// if root is selected paste as new job
				if (destinationNode.isRoot()) {
					// only add jobs to root
					if (container instanceof JobContainer) {
						// remove parent in case job was a sub job before
						container.setParentContainer(null);
						addGroupNodeAt(0, nodeCopy, root, true);
					}
					continue;
				}
				ContainerNode parentNode = (ContainerNode) destinationNode;
				while (parentNode.isTask() || parentNode.isIfGroup()) {
					parentNode = (ContainerNode) parentNode.getParent();
				}
				GroupContainer parentContainer = (GroupContainer) parentNode
						.getActionContainer();
				// add new container to parent and vice versa
				int index = getInsertionIndex();
				parentContainer.addTaskAt(index, container);
				container.setParentContainer(parentContainer);
				if (container instanceof IfContainer) {
					// check, if test container still exists in this group
					IfContainer ifContainer = (IfContainer) container;
					ActionContainer testContainer = ifContainer
							.getReturnCodeTest().getTestContainer();
					if (!parentContainer.getTasks().contains(testContainer)) {
						ifContainer.getReturnCodeTest().setTestContainer(null);
						// update the panel
						((JPAPanel) nodeCopy.getPanel()).updateValues();
					}
				}
				if (container instanceof GroupContainer) {
					addGroupNodeAt(index, nodeCopy, parentNode, true);
				}
				else if (container instanceof TaskContainer) {
					insertNodeAt(index, nodeCopy, parentNode, true);
					container.setModifiedTime(new Date());
				}
				else {
					logger.warning("Unknown container in clipboard: "
							+ container.getClass());
				}
			}
		}
		catch (Exception e) {
			logger.log(Level.WARNING,
					"Could not paste into job preparation tree.", e);
			UserMessages.error("Could not paste into job preparation tree.", e
					.getMessage());
		}
	}

	/**
	 * Remove a node
	 */
	protected void remove() {
		// return if we selected the root node
		if (focussedNode.isRoot() && getSelectionCount() == 1) {
			return;
		}
		if (JOptionPane.showConfirmDialog(client,
				"Remove selected task/job node(s) from job preparation tree?",
				"UNICORE: Confirm Remove", JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION) {
			return;
		}
		// remove selected nodes from tree
		TreePath[] paths = getSelectionPaths();
		Vector parents = new Vector();
		for (int i = 0; i < paths.length; i++) {
			PanelNode node = (PanelNode) paths[i].getLastPathComponent();
			if (node.equals(root) || node.isThenElseGroup()) {
				continue;
			}
			parents.add(node.getParent());
			remove(node);
		}
		for (int i = 0; i < parents.size(); i++) {
			if (parents.elementAt(i) instanceof ContainerNode) {
				applyValues((ContainerNode) parents.elementAt(i));
			}
		}
		// select root node as default
		selectRootNode();
	}

	/**
	 * Remove a node
	 * 
	 * @param node
	 *            node to be removed
	 */
	public void remove(PanelNode node) {
		PanelNode parentNode = (PanelNode) node.getParent();
		if (!parentNode.isRoot()) {
			removeDependencies((ContainerNode) node);
			// remove ignore failure for if tasks
			if (node.getUserObject() instanceof IfContainer) {
				IfContainer ifTask = (IfContainer) node.getUserObject();
				if (ifTask.getReturnCodeTest().isEnabled()) {
					ActionContainer testTask = ifTask.getReturnCodeTest()
							.getTestContainer();
					if (testTask != null) {
						testTask.setIgnoreFailure(false);
					}
				}
			}
		}
		// remove selected node from jpa tree
		model.removeNodeFromParent(node);
		node = null;
	}

	private void removeDependencies(ContainerNode node) {
		ActionContainer container = node.getActionContainer();
		// parent is a group
		ContainerNode groupNode = (ContainerNode) node.getParent();
		GroupContainer groupContainer = (GroupContainer) groupNode
				.getActionContainer();
		groupContainer.setModifiedTime(new Date());
		// remove container from job
		groupContainer.removeTask(container);
		// remove all dependencies belonging to the selected subtree
		// by comparing nodes to children of selected node
		// get dependencies from parent job group
		DependencyContainer deps[] = groupContainer.getDependencies();
		if (deps == null) {
			return;
		}
		// loop over edges
		Vector newDeps = new Vector();
		for (int i = 0; i < deps.length; i++) {
			boolean predFound = false;
			boolean succFound = false;
			DependencyContainer dc = deps[i];
			Identifier predId = dc.getPred().getIdentifier();
			Identifier succId = dc.getSucc().getIdentifier();
			// loop over job tasks and test, if current edge matches tasks
			Vector tasks = groupContainer.getTasks();
			for (int j = 0; j < tasks.size(); j++) {
				ActionContainer task = (ActionContainer) tasks.elementAt(j);
				Identifier taskId = task.getIdentifier();
				if (taskId.equals(predId)) {
					predFound = true;
				}
				if (taskId.equals(succId)) {
					succFound = true;
				}
			}
			if (predFound && succFound) {
				newDeps.add(dc);
			}
		}
		// put newDeps into JobContainer, by building
		// an array out of the newDeps vector and passing that one
		// to the job group
		DependencyContainer[] newDepsArray = new DependencyContainer[newDeps
				.size()];
		for (int i = 0; i < newDeps.size(); i++) {
			newDepsArray[i] = (DependencyContainer) newDeps.elementAt(i);
		}
		groupContainer.setDependencies(newDepsArray);
		((JPAPanel) groupNode.getPanel()).updateValues();
	}

	private void renameContainer(ActionContainer container) {
		if (!(container instanceof ThenContainer)
				&& !(container instanceof ElseContainer)) {
			container.setName(container.getName() + "_Copy");
		}
		if (container instanceof GroupContainer) {
			GroupContainer group = (GroupContainer) container;
			Vector tasks = group.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				renameContainer((ActionContainer) tasks.elementAt(i));
			}
		}
	}

	private void sort(GroupContainer container, ContainerNode jobNode) {
		if (jobNode == null || jobNode.isIfGroup()) {
			return;
		}
		if (container.getTasks().isEmpty()) {
			return;
		}
		applyValues(jobNode);
		//get the unsorted elements of the belonging JPATree
		DependencyEditor depEditor = new DependencyEditor(container);
		Hashtable table = new Hashtable();
		Vector unsort = new Vector();
		for (int i = 0; i < depEditor.getGraph().getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) depEditor.getGraph().getVertex(i);
			ContainerNode node = (ContainerNode) model.getChild(jobNode, i);
			table.put(vertex, node);
			unsort.addElement(vertex);
			if (node.isGroup()) {
				//recursive
				GroupContainer nodeContainer = (GroupContainer) node
						.getActionContainer();
				this.sort(nodeContainer, node);
			}
		}
		for (int i = 0; i < unsort.size(); i++) {
			model.removeNodeFromParent((PanelNode) table.get((DAGVertex) unsort
					.elementAt(i)));
		}
		//sorting of the elements of the belonging JPATree
		VertexsTreeComparable vertexs = new VertexsTreeComparable();
		// do not sort then and else!
		unsort = VectorSorter.sort(unsort, true);
		TreeSorter treeSorter = new TreeSorter(vertexs);
		Vector sort = treeSorter.sort(unsort);
		DAGVertex key;
		ContainerNode value;
		Vector tasks = new Vector();
		for (int i = 0; i < sort.size(); i++) {
			key = (DAGVertex) sort.elementAt(i);
			value = (ContainerNode) table.get(key);
			tasks.addElement(value.getActionContainer());
			insertNodeAt(i, value, jobNode, false);
		}
		container.setTasks(tasks);
	}

	/**
	 * Submit a job
	 */
	private void submit() {
		if (currentTopLevelJobNode == null) {
			return;
		}
		if (!updateCurrentJob(true)) {
			return;
		}
		currentTopLevelJobNode.setState(ActionContainer.STATE_SUBMITTING);
		updateNode(currentTopLevelJobNode);
		SubmitJobNode submitJob = new SubmitJobNode(currentTopLevelJobNode);
		submitJob.addObserver(this);
		submitJob.start();
	}

	/**
	 * Update a node and its parents
	 * 
	 * @param displayErrors
	 *            true, if a detailed error window should popup
	 * @return true, if job is correct
	 */
	public boolean updateCurrentJob(boolean displayErrors) {
		// apply panels in job
		if (currentTopLevelJobNode == null) {
			return true;
		}
		applyValues(currentTopLevelJobNode);
		// check containers in job
		boolean result = checkContainers(currentTopLevelJobNode, displayErrors);
		updateNode(currentTopLevelJobNode);
		return result;
	}

	/**
	 * Update a node and its parents
	 * 
	 * @param displayErrors
	 *            true, if a detailed error window should popup
	 * @param vsiteChanged
	 *            Description of the Parameter
	 * @return true, if job is correct
	 */
	public boolean updateCurrentJob(boolean displayErrors, boolean vsiteChanged) {
		// apply panels in job
		if (currentTopLevelJobNode == null) {
			return true;
		}
		updateValues(currentTopLevelJobNode, vsiteChanged);
		return true;
	}

	/**
	 * Overwrites method from super class. This method is the callback for all
	 * tree selections in the super class.
	 */
	protected void updateSelection() {
		// update selected objects by calling method of super class
		super.updateSelection();
		if (focussedNode.isRoot()) {
			// switch main panel to logo
			client.clearDataArea();
			currentGroupNode = null;
			client.setStatusMessage("Job Preparation");
			return;
		}
		// if we get here we have a current top level job
		String message = getCurrentTopLevelJob() + " not saved yet.";
		if (getCurrentTopLevelJob().getFilename() != null) {
			message = getCurrentTopLevelJob().getFilename();
		}
		client.setStatusMessage(message);
		// set identity string in StatusPanel
		User user = getCurrentTopLevelJob().getUser();
		try {
			String identity = ResourceManager.getKeystoreManager()
					.getKeyAliasFromCertificate(user.getCertificate());
			client.getStatusPanel().setIdentity(identity);
		}
		catch (Exception e) {
			client.getStatusPanel().setIdentity("UNKNOWN");
		}
		// update panel from tree and add it to main data area
		((JPAPanel) focussedPanel).updateValues();
		client.addToDataArea(focussedPanel);
		updateCurrentJob(false);
	}

	/**
	 * Propagate change of look and feel
	 */
	public void updateUI() {
		super.updateUI();
		setCellRenderer(new ClientCellRenderer());
		if (popup != null) {
			SwingUtilities.updateComponentTreeUI(popup);
		}
	}

	/**
	 * Update Usite lists in job group panels
	 */
	public void updateUsites() {
		updateUsites(root);
	}

	/**
	 * Recursive update of usites in job group panels
	 */
	private void updateUsites(PanelNode parent) {
		for (int i = 0; i < parent.getChildCount(); i++) {
			PanelNode node = (PanelNode) parent.getChildAt(i);
			if (node.getPanel() instanceof JobGroupPanel) {
				JobGroupPanel panel = (JobGroupPanel) node.getPanel();
				panel.updateUsites();
				updateUsites(node);
			}
		}
		if (focussedPanel != null) {
			((JPAPanel) focussedPanel).updateValues();
		}
		updateCurrentJob(false);
	}

	private void updateValues(ContainerNode node, boolean vsiteChanged) {
		JPanel panel = node.getPanel();
		if (panel instanceof JPAPanel) {
			JPAPanel jpaPanel = (JPAPanel) panel;
			jpaPanel.updateValues(vsiteChanged);
		}
		if (node.isGroup()) {
			for (int i = 0; i < node.getChildCount(); i++) {
				updateValues((ContainerNode) node.getChildAt(i), vsiteChanged);
			}
		}
	}

	class AddDoNAction extends AbstractAction {

		private int counter = 0;

		public AddDoNAction() {
			super("Add Do N", ResourceManager.getIcon(ResourceManager.DON));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// count instance
			counter++;
			DoNContainer container = new DoNContainer();
			container.setName("New_DoN" + counter);
			addNewGroup(container);
		}
	}

	class AddDoRepeatAction extends AbstractAction {

		private int counter = 0;

		public AddDoRepeatAction() {
			super("Add Do Repeat", ResourceManager
					.getIcon(ResourceManager.DOREPEAT));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// count instance
			counter++;
			DoRepeatContainer container = new DoRepeatContainer();
			container.setName("New_DoRepeat" + counter);
			addNewGroup(container);
		}
	}

	class AddExportAction extends AbstractAction {

		private int counter = 0;

		public AddExportAction() {
			super("Add Export", ResourceManager
					.getIcon(ResourceManager.FILEOPERATION));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			updateCurrentJob(false);
			// count instance
			counter++;
			// build container
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			FileExportContainer container = new FileExportContainer(
					parentContainer);
			container.setName(res.getString("INITIAL_EXPORT") + counter);
			// build panel
			ExportJPAPanel panel = new ExportJPAPanel(client, container);
			panel.resetValues();
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, container);
			// insert sub job at the appropriate place
			addTaskAt(index, container, panel, true);
		}
	}

	class AddFileOperationAction extends AbstractAction {

		private int counter = 0;

		public AddFileOperationAction() {
			super("Add File Operation", ResourceManager
					.getIcon(ResourceManager.FILEOPERATION));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			updateCurrentJob(false);
			// count instance
			counter++;
			// build container
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			FileOperationContainer container = new FileOperationContainer(
					parentContainer);
			container.setName(res.getString("INITIAL_FILEOP") + counter);
			// build panel
			OperationJPAPanel panel = new OperationJPAPanel(client, container);
			panel.resetValues();
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, container);
			// insert sub job at the appropriate place
			addTaskAt(index, container, panel, true);
		}
	}

	class AddHoldJobAction extends AbstractAction {

		private int counter = 0;

		public AddHoldJobAction() {
			super("Add Hold Job", ResourceManager
					.getIcon(ResourceManager.HOLDJOB));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// count instance
			counter++;
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			HoldJobContainer container = new HoldJobContainer(parentContainer);
			container.setName("New_HoldJob" + counter);
			// build panel
			HoldJobPanel panel = new HoldJobPanel(client, container);
			panel.resetValues();
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, container);
			// insert sub job at the appropriate place
			addTaskAt(index, container, panel, true);
		}
	}

	class AddIfThenElseAction extends AbstractAction {

		private int counter = 0;

		public AddIfThenElseAction() {
			super("Add If Then Else", ResourceManager
					.getIcon(ResourceManager.IF));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// count instance
			counter++;
			IfContainer ifContainer = new IfContainer();
			ifContainer.setName("New_IfThenElse" + counter);
			ThenContainer thenContainer = new ThenContainer();
			thenContainer.setName("Then");
			thenContainer.setParentContainer(ifContainer);
			ifContainer.addTaskAt(0, thenContainer);
			ElseContainer elseContainer = new ElseContainer();
			elseContainer.setName("Else");
			elseContainer.setParentContainer(ifContainer);
			ifContainer.addTaskAt(1, elseContainer);
			addNewGroup(ifContainer);
		}
	}

	class AddImportAction extends AbstractAction {

		private int counter = 0;

		public AddImportAction() {
			super("Add Import", ResourceManager
					.getIcon(ResourceManager.FILEOPERATION));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			updateCurrentJob(false);
			// count instance
			counter++;
			// build container
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			FileImportContainer container = new FileImportContainer(
					parentContainer);
			container.setName(res.getString("INITIAL_IMPORT") + counter);
			// build panel
			ImportJPAPanel panel = new ImportJPAPanel(client, container);
			panel.resetValues();
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, container);
			// insert sub job at the appropriate place
			addTaskAt(index, container, panel, true);
		}
	}

	class AddPluginAction extends AbstractAction {

		private TaskPlugable plugable;

		public AddPluginAction(TaskPlugable plugable) {
			super("Add " + plugable.getName(), new ImageIcon(plugable
					.getIconImage()));
			this.plugable = plugable;
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			addNewPluginTask(plugable);
		}
	}

	class AddSubJobAction extends AbstractAction {

		private int counter = 0;

		public AddSubJobAction() {
			super("Add Sub Job", ResourceManager
					.getIcon(ResourceManager.JOBGROUP));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// check if we have really selected a job group before...
			updateCurrentJob(false);
			// count instance
			counter++;
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			JobContainer parentJobContainer = (JobContainer) currentTopLevelJobNode
					.getActionContainer();
			// build new job container
			JobContainer jc = new JobContainer();
			jc.setName(res.getString("INITIAL_SUB_JOB") + counter);
			// Attention: parent and sub job group reference to the same user!
			jc.setUser(parentJobContainer.getUser());
			// link subjob and jo with each other
			jc.setParentContainer(parentContainer);
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, jc);
			// insert sub job at the appropriate place
			addGroupAt(index, jc, currentGroupNode, true);
		}
	}

	class AddTransferAction extends AbstractAction {

		private int counter = 0;

		public AddTransferAction() {
			super("Add Transfer", ResourceManager
					.getIcon(ResourceManager.FILEOPERATION));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			updateCurrentJob(false);
			// count instance
			counter++;
			// build container
			GroupContainer parentContainer = (GroupContainer) currentGroupNode
					.getActionContainer();
			FileTransferContainer container = new FileTransferContainer(
					parentContainer);
			container.setName(res.getString("INITIAL_TRANSFER") + counter);
			// build panel
			TransferJPAPanel panel = new TransferJPAPanel(client, container);
			panel.resetValues();
			int index = getInsertionIndex();
			parentContainer.addTaskAt(index, container);
			// insert sub job at the appropriate place
			addTaskAt(index, container, panel, true);
		}
	}

	class CheckAction extends AbstractAction {

		public CheckAction() {
			super("Check", ResourceManager.getIcon(ResourceManager.CHECK));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			// first apply complete job
			updateCurrentJob(false);
			// now check selected node
			boolean correct = checkContainers((ContainerNode) focussedNode,
					true);
			// display info if node was correct
			if (correct) {
				UserMessages.showInfo(focussedNode + " is correct.");
			}
		}
	}

	class NewJobAction extends AbstractAction {

		private int counter = 0;

		public NewJobAction() {
			super("New Job", ResourceManager.getIcon(ResourceManager.NEW));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			if (ResourceManager.getKeystoreManager() == null) {
				UserMessages.warning(res.getString("NO_CERT_AVAILABLE"));
				return;
			}
			// count instance
			counter++;
			UserDefaults userDefaults = ResourceManager.getUserDefaults();
			// build new job container
			JobContainer jc = new JobContainer();
			jc.setName(res.getString("INITIAL_JOB") + counter);
			// add new container to tree root
			addGroupAt(0, jc, root, true);
		}
	}

	class OpenAction extends AbstractAction {

		public OpenAction() {
			super("Open Job", ResourceManager.getIcon(ResourceManager.OPEN));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			client.openJob();
		}
	}

	class SaveAction extends AbstractAction {

		public SaveAction() {
			super("Save Job", ResourceManager.getIcon(ResourceManager.SAVE));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			client.saveJob();
		}
	}

	class SaveAsAction extends AbstractAction {

		public SaveAsAction() {
			super("Save Job As...", ResourceManager
					.getIcon(ResourceManager.SAVEAS));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			client.saveJobAs();
		}
	}

	class SortAction extends AbstractAction {

		public SortAction() {
			super("Sort", ResourceManager.getIcon(ResourceManager.SORT));
			this.setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			ContainerNode jobNode = currentGroupNode;
			GroupContainer container = (GroupContainer) jobNode
					.getActionContainer();
			container.setName(((GroupPanel) jobNode.getPanel()).getGroupName());
			sort(container, jobNode);
			focussedNode = jobNode;
			setSelectionPath(new TreePath(focussedNode.getPath()));
		}
	}

	class SubmitAction extends AbstractAction {

		public SubmitAction() {
			super("Submit", ResourceManager.getIcon(ResourceManager.SUBMIT));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			submit();
		}
	}
}