/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.security.Principal;
/**
 *  A principal represents a distinguished name of an individual which includes
 *  different fields
 *
 *@author     Kirsten Foerster
 *@version    $Id: DistinguishedNameFields.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */

public class DistinguishedNameFields {

	private String commonName = "";
	private String country = "";
	private String email = "";
	private String location = "";
	private String org = "";
	private String orgUnit = "";
	private String principal;
	private String state = "";

	
	public DistinguishedNameFields(Principal principal) {
		this.principal = principal.toString();
		this.setFields();
	}

	/**
	 * Gets the commonName attribute of the DistinguishedNameFields object
	 * 
	 * @param flag -
	 *            true, you get the common name value with mnemonic; otherwise
	 *            without this mnemonic
	 * @return The commonName value
	 */
	public String getCommonName(boolean flag) {
		if (!flag && !this.commonName.equals("")) {
			return this.commonName.substring(3, this.commonName.length());
		}
		return this.commonName;
	}

	/**
	 *  Gets the country attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the state value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The country value
	 */
	public String getCountry(boolean flag) {
		if (!flag && !this.country.equals("")) {
			return this.country.substring(2, this.country.length());
		}
		return this.country;
	}

	/**
	 *  Gets the email attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the email value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The email value
	 */
	public String getEmail(boolean flag) {
		if (!flag && !this.email.equals("")) {
			return this.email.substring(6, this.email.length());
		}
		return this.email;
	}

	/**
	 *  Gets the location attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the location value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The location value
	 */
	public String getLocation(boolean flag) {
		if (!flag && !this.location.equals("")) {
			return this.location.substring(2, this.location.length());
		}
		return this.location;
	}

	/**
	 *  Gets the org attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the org value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The org value
	 */
	public String getOrg(boolean flag) {
		if (!flag && !this.org.equals("")) {
			return this.org.substring(2, this.org.length());
		}
		return this.org;
	}

	/**
	 *  Gets the orgUnit attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the orgUnit value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The orgUnit value
	 */
	public String getOrgUnit(boolean flag) {
		if (!flag && !this.orgUnit.equals("")) {
			return this.orgUnit.substring(3, this.orgUnit.length());
		}
		return this.orgUnit;
	}

	/**
	 *  Returns only one information of the principal.
	 *
	 *@param  textFrom  - a string represents the part of the principal you want
	 *      give back
	 *@return           - returns one information of the distinguished name
	 */
	private String getPrincipalField(String textFrom) {
		String text = "";
		String modPrincipal = this.principal.toLowerCase();
		String modTextFrom = textFrom.toLowerCase();
		int indexFrom = modPrincipal.indexOf(modTextFrom);
		int indexTo;

		if (indexFrom != -1) {
			indexTo = modPrincipal.indexOf(",", indexFrom);
			if (indexTo == -1) {
				text
					+= this.principal.substring(
						indexFrom,
						this.principal.length());
			} else {
				text += this.principal.substring(indexFrom, indexTo);
			}
		}

		return text;

	}

	/**
	 *  Gets the state attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the state value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The state value
	 */
	public String getState(boolean flag) {
		if (!flag && !this.state.equals("")) {
			return this.state.substring(3, this.state.length());
		}
		return this.state;
	}

	/**
	 *  Set the fields information of a distinguished name.
	 */
	private void setFields() {
		String text = principal.toString();
		this.commonName = this.commonName + this.getPrincipalField("CN");
		this.orgUnit = this.orgUnit + this.getPrincipalField("OU=");
		this.org = this.org + this.getPrincipalField("O=");
		this.location = this.location + this.getPrincipalField("L=");
		this.state = this.state + this.getPrincipalField("ST=");
		this.country = this.country + this.getPrincipalField("C=");
		this.email = this.email + this.getPrincipalField("EMAIL");
		if (email.indexOf("=") >= 0) {
			this.email =
				"EMAIL"
					+ this.email.substring(
						this.email.indexOf("="),
						this.email.length());
		}
	}
}
