/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.awt.Component;
import javax.swing.JMenuItem;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;

/**
 * Abstract base class for UNICORE client extension plugins
 * 
 * @author Ralf Ratering
 * @version $Id: ExtensionPlugable.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public abstract class ExtensionPlugable extends UnicorePlugable {

	/**
	 * Used to add a plugin controlled item or sub menu to the client's
	 * extension menu. The client does not attach a listener to this item so it
	 * is up to the plugin developer to respond to this item. The method returns
	 * null if no menu item should be added to the clients extension menu.
	 * 
	 * @return a menu item that will be added to the client's extension menu
	 */
	public JMenuItem getCustomMenu() {
		return null;
	}

	/**
	 * Get a Component that will be added to the toolbar that is attached to the
	 * JPA tree.
	 * 
	 * @return a Component, ideally a button containing an icon
	 */
	public Component getJPAToolBarComponent() {
		return null;
	}

	/**
	 * Get a Component that will be added to the toolbar that is attached to the
	 * vsite list in a job group panel.
	 * 
	 * @return a Component, ideally a button containing an icon
	 */
	public Component getVsiteToolBarComponent() {
		return null;
	}

	/**
	 * This method checks if the plugin can enable special features for a vsite.
	 * It may for example create a Globus proxy certifcate for a Globus enabled
	 * Vsite.
	 * 
	 * @param vsite
	 *            The vsite of the job
	 * @param job
	 *            The abstract job object
	 * @return Objects returned.
	 * @exception Exception
	 */
	public Object setupSpecialVsiteFeatures(Vsite vsite, AbstractJob job)
			throws Exception {
		return null;
	}
}