/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;
import org.unicore.Vsite;
import com.pallas.unicore.extensions.Usite;

/**
 * @author Thomas Kentemich
 * @version $Id: SiteDefaults.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class SiteDefaults extends Defaults {

	private final static String defaultsFile = unicoreDir + File.separator
			+ "SiteDefaults.txt";
	private final static String USITE = "unicore.usite.";
	private final static String VSITE = "unicore.vsite.";
	private final static String SITELIST = "unicore.sitelist.";

	public SiteDefaults() {
	}

	protected void genSystemDefaults() {
	}

	public void setUsite(Usite usite, boolean enabled) {
		put(USITE + usite.getName(), new Boolean(enabled).toString());
	}

	public boolean getUsiteValue(Usite usite) {
		//		System.out.println("Usite: "+usite.getName()+" value:
		// "+booleanValue(USITE+usite.getName()));
		return booleanValue(USITE + usite.getName());
	}

	public boolean isUsiteContained(Usite usite) {
		String result = (String) get(USITE + usite.getName());
		return (result != null);
	}

	public void setVsite(Vsite vsite, boolean enabled) {
		put(VSITE + vsite.getName(), new Boolean(enabled).toString());
	}

	public boolean isVsiteContained(Vsite vsite) {
		String result = (String) get(VSITE + vsite.getName());
		return (result != null);
	}

	public boolean getVsiteValue(Vsite vsite) {
		return booleanValue(VSITE + vsite.getName());
	}

	public void setUrl(String site, boolean enabled) {
		put(SITELIST + site, new Boolean(enabled).toString());
	}

	public boolean isSiteContained(String site) {
		String result = (String) get(SITELIST + site);
		return (result != null);
	}

	public boolean getSiteValue(String site) {
		return booleanValue(SITELIST + site);
	}

	public Vector getAllEntries() {
		// this returns all usite names
		Enumeration all = keys();
		Vector out = new Vector();
		while (all.hasMoreElements()) {
			String next = (String) all.nextElement();
			if (next.indexOf(SITELIST) != -1) {
				out.add(next.substring(SITELIST.length()));
			}
			else if (next.indexOf(USITE) != -1) {
				out.add(next.substring(USITE.length()));
			}
			else if (next.indexOf(VSITE) != -1) {
				out.add(next.substring(VSITE.length()));
			}
		}
		return out;
	}

	public void removeEntry(String key) {
		Object returned = this.remove(USITE + key);
		// writeToFile();
	}

	public void removeDeletedUsites(Vector allUsites) {
		Enumeration all = keys();
		Vector out = new Vector();
		String nextUsite;
		boolean found = false;
		while (all.hasMoreElements()) {
			String next = (String) all.nextElement();
			if (next.indexOf(USITE) != -1) {
				found = false;
				for (int i = 0; i < allUsites.size(); i++) {
					nextUsite = USITE
							+ ((Usite) allUsites.elementAt(i)).getName();
					if (next.equals(nextUsite)) {
						// usite key is still in the usite list
						found = true;
						break;
					}
				}
				if (!found) {
					remove(next);
				}
			}
		}
	}

	public void removeDeletedVsites(Vector allVsites) {
		Enumeration all = keys();
		Vector out = new Vector();
		String nextVsite;
		boolean found = false;
		while (all.hasMoreElements()) {
			String next = (String) all.nextElement();
			if (next.indexOf(VSITE) != -1) {
				found = false;
				for (int i = 0; i < allVsites.size(); i++) {
					nextVsite = VSITE
							+ ((Vsite) allVsites.elementAt(i)).getName();
					if (next.equals(nextVsite)) {
						// vsite key is still in the vsite list
						found = true;
						break;
					}
				}
				if (!found) {
					remove(next);
				}
			}
		}
	}

	public void removeDeletedUrls(Vector allUrls) {
		Enumeration all = keys();
		Vector out = new Vector();
		String nextUrl;
		boolean found = false;
		while (all.hasMoreElements()) {
			String next = (String) all.nextElement();
			if (next.indexOf(SITELIST) != -1) {
				found = false;
				for (int i = 0; i < allUrls.size(); i++) {
					nextUrl = SITELIST + (String) allUrls.elementAt(i);
					if (next.equals(nextUrl)) {
						// usite key is still in the usite list
						found = true;
						break;
					}
				}
				if (!found) {
					remove(next);
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.pallas.unicore.client.util.Defaults#loadFromFile()
	 */
	public boolean loadFromFile() {
		// look for ${HOME}/.unicoreXX/UserDefaults.txt
		return loadFromFile(defaultsFile);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.pallas.unicore.client.util.Defaults#writeToFile()
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/UserDefaults.txt
		return writeToFile(defaultsFile);
	}
}