/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.xml;

import java.nio.charset.Charset;
import java.util.logging.Handler;
import java.util.logging.XMLFormatter;

/**
 * Format a LogRecord into a standard XML format similar to the XMLFormatter.
 * 
 * @author Kirsten Foerster
 * @version $Id: NewXMLFormatter.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class NewXMLFormatter extends XMLFormatter {

	/**
	 * Return the header string for a set of XML formatted records.
	 * 
	 * @param h
	 *            The target handler.
	 * @return header string
	 */
	public String getHead(Handler h) {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\"");
		String encoding = h.getEncoding();
		if (encoding == null) {
			// Figure out the default encoding.
			encoding = sun.io.Converters.getDefaultEncodingName();
		}
		// Try to map the encoding name to a canonical name.
		try {
			Charset cs = Charset.forName(encoding);
			encoding = cs.name();
		}
		catch (Exception ex) {
			// We hit problems finding a canonical name.
			// Just use the raw encoding name.
		}
		sb.append(" encoding=\"");
		sb.append(encoding);
		sb.append("\"");
		sb.append(" standalone=\"no\"?>\n");
		sb.append("<log>\n");
		return sb.toString();
	}
}