/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.xml;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.io.Writer;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import sun.misc.BASE64Encoder;
import com.pallas.unicore.utility.UserMessages;

/**
 * @author Thomas Kentemich
 * @version $Id: XMLObjectWriter.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class XMLObjectWriter extends XMLObjectIO {

	class KeyObject {

		Object o;

		public KeyObject(Object o) {
			this.o = o;
		}

		// used in Hashtable to distinguish keys
		public boolean equals(Object x) {
			if (x instanceof KeyObject) {
				return ((KeyObject) x).o == this.o;
			}
			UserMessages
					.error("ERROR in XMLObjectWriter.KeyObject.equals(): argument "
							+ x + " is NOT a KeyObject");
			return false;
		}

		// used in Hashtable to calculate an index into the Hashtable
		public int hashCode() {
			// Caution: System.identityHashCode(o) may be identical for
			// different objects,
			//          i.e. it is not guaranteed to be calculated from the memory
			// address
			// Consequently, System.identityHashCode(o) cannot be used as a key
			// into the Hashtable
			return System.identityHashCode(o);
		}
	}

	class ValueObject {

		Object obj;
		int objID;

		public ValueObject(Object obj, int objID) {
			this.obj = obj;
			this.objID = objID;
		}
	}
	private static BASE64Encoder encoder = new BASE64Encoder();
	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.xml");
	// Counter for objects, used for IDs in XML file
	private static int objectID;
	//	static Class[] OOS_ARGS = {ObjectOutputStream.class};
	private static Hashtable objTable;
	//	public static Object[] writeObjectArglist;
	final static String versionUnicoreXML = "4.0.0";
	private DocumentBuilder db;
	private DocumentBuilderFactory dbf;
	DebugMessages debug = new DebugMessages("|", " |");
	private Document doc;
	private OutputStream out = null;
	private Writer writer = null;

	private XMLObjectWriter() throws TransformerConfigurationException {
		// Create Hashtable to store information about XML writtem objects
		objTable = new Hashtable();
		dbf = DocumentBuilderFactory.newInstance();
		//dbf.setValidating(true);
		//dbf.setNamespaceAware(true);
		try {
			db = dbf.newDocumentBuilder();
		}
		catch (ParserConfigurationException e) {
			logger.log(Level.SEVERE, "", e);
		}
		doc = db.newDocument();
	}

	public XMLObjectWriter(File fout) throws FileNotFoundException,
			IOException, TransformerConfigurationException {
		//this(new FileWriter(fout));
		this(new BufferedOutputStream(new DataOutputStream(
				new FileOutputStream(fout))));
	}

	public XMLObjectWriter(OutputStream out)
			throws TransformerConfigurationException {
		this();
		if (out instanceof DataOutputStream) {
			this.out = new BufferedOutputStream(out);
		}
		else {
			this.out = new BufferedOutputStream(new DataOutputStream(out));
		}
	}

	public XMLObjectWriter(Writer writer)
			throws TransformerConfigurationException {
		this();
		this.writer = new BufferedWriter(writer);
	}

	private Element createDOMArray(String fieldName, Object value)
			throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		String fName;
		if (fieldName != null) {
			fName = fieldName;
		}
		else {
			fName = validXMLTag(value.getClass());
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
		}
		if (!newHashed(fieldElem, value)) {
			return fieldElem;
		}
		if (fieldName != null) {
			fieldElem.setAttribute(ATTR_VALUETYPE, value.getClass().getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		int dim = -1;
		try {
			//	  dim = checkDim(fieldType);
			dim = checkDim(value.getClass());
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		fieldElem.setAttribute(ATTR_DIM, dim + "");
		int length = Array.getLength(value);
		fieldElem.setAttribute(ATTR_LENGTH, length + "");
		Element arrayElem;
		for (int i = 0; i < length; i++) {
			Object value_i = Array.get(value, i);
			arrayElem = createDOMElement(fName, value_i);
			arrayElem.setAttribute(ATTR_INDEX, i + "");
			fieldElem.appendChild(arrayElem);
		}
		return fieldElem;
	}

	private Element createDOMDate(String fieldName, Object value)
			throws IOException {
		boolean isSuperClass = false;
		return createDOMDate(fieldName, value, value.getClass(), isSuperClass);
	}

	private Element createDOMDate(String fieldName, Object value,
			Class valueClass, boolean isSuperClass) throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		String fName;
		if (fieldName == null || isSuperClass) {
			fName = validXMLTag(valueClass);
		}
		else {
			fName = fieldName;
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
		}
		if (!isSuperClass) {
			if (!newHashed(fieldElem, value)) {
				return fieldElem;
			}
		}
		if (fieldName != null) {
			fieldElem.setAttribute(ATTR_VALUETYPE, Date.class.getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		Date valueDate = (Date) value;
		Long fastTime = new Long(valueDate.getTime());
		Element timeElem = createDOMElement("fastTime", fastTime);
		fieldElem.appendChild(timeElem);
		return fieldElem;
	}

	private Element createDOMElement(String fieldName, Object value)
			throws IOException {
		Class valueClass = null;
		if (value instanceof Class) {
			valueClass = (Class) value;
		}
		else {
			valueClass = value.getClass();
		}
		return createDOMElement(fieldName, value, valueClass);
	}

	private Element createDOMElement(String fieldName, Object obj,
			Class objClass) throws IOException {
		try {
			debug.println("createDOMElement(" + obj + " : " + objClass + ")");
		}
		catch (NullPointerException npe) {
			debug.println("createDOMElement(" + npe.getMessage() + " : "
					+ objClass + ")");
		}
		debug.pushIndent();
		int mod;
		Element elem = null;
		try {
			if (obj == null) {
				elem = doc.createElement(fieldName + NULL_INDICATOR);
				debug.popIndent();
				return elem;
			}
			String tag;
			if (fieldName != null) {
				tag = fieldName;
			}
			else {
				tag = validXMLTag(objClass);
			}
			elem = createDOMPrimitive(tag, obj);
			if (elem != null) {
				debug.popIndent();
				return elem;
			}
			elem = createDOMSpecial(tag, obj, objClass);
			if (elem != null) {
				debug.popIndent();
				return elem;
			}
			debug.println("Generating XML tag <" + tag + ">");
			try {
				elem = doc.createElement(tag);
			}
			catch (DOMException ex) {
				logger.log(Level.SEVERE, "", ex);
				tag = ex.toString().replace(' ', '_');
				debug
						.println("\nDOMException: Found illegal tag, final now XML tag <"
								+ tag + ">");
				elem = doc.createElement(tag);
			}
			if (!newHashed(elem, obj)) {
				debug.popIndent();
				return elem;
			}
			if (fieldName != null) {
				elem.setAttribute(ATTR_VALUETYPE, objClass.getName());
			}
			scanFields(elem, obj, objClass);
		}
		catch (Exception e) {
			// throw new IOException("error: "+ e);
			logger.log(Level.SEVERE, "ERROR in createDOMElement: ", e);
		}
		// Set XML version of "Clinet" classes
		elem.setAttribute(ATTR_ELEMVERSION, versionUnicoreXML);
		debug.popIndent();
		return elem;
	}

	private Element createDOMFile(String fieldName, Object value)
			throws IOException {
		boolean isSuperClass = false;
		return createDOMFile(fieldName, value, value.getClass(), isSuperClass);
	}

	private Element createDOMFile(String fieldName, Object value,
			Class valueClass, boolean isSuperClass) throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		String fName;
		if (fieldName == null || isSuperClass) {
			fName = validXMLTag(valueClass);
		}
		else {
			fName = fieldName;
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
		}
		if (!isSuperClass) {
			if (!newHashed(fieldElem, value)) {
				return fieldElem;
			}
		}
		if (fieldName != null) {
			fieldElem.setAttribute(ATTR_VALUETYPE, File.class.getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		File valueFile = (File) value;
		String path = valueFile.getPath();
		Element pathElem = createDOMElement("path", path);
		fieldElem.appendChild(pathElem);
		return fieldElem;
	}

	//------------------------------------------------------------------
	/**
	 * Create a DOM element node for a Hashtable.
	 * 
	 * @param fieldName
	 *            the XML tag for the vector.
	 * @param value
	 *            the object to be serialized in XML format.
	 * @return Description of the Returned Value
	 * @exception IOException
	 *                Description of Exception
	 */
	private Element createDOMHashtable(String fieldName, Object value)
			throws IOException {
		boolean isSuperClass = false;
		return createDOMHashtable(fieldName, value, value.getClass(),
				isSuperClass);
	}

	//------------------------------------------------------------------
	/**
	 * Create a DOM element node for a Hashtable.
	 * 
	 * @param fieldName
	 *            the XML tag for the vector.
	 * @param value
	 *            the object to be serialized in XML format.
	 * @param valueClass
	 *            Description of the Parameter
	 * @param isSuperClass
	 *            Description of the Parameter
	 * @return Description of the Returned Value
	 * @exception IOException
	 *                Description of Exception
	 */
	private Element createDOMHashtable(String fieldName, Object value,
			Class valueClass, boolean isSuperClass) throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		String fName;
		if (fieldName == null || isSuperClass) {
			fName = validXMLTag(valueClass);
		}
		else {
			fName = fieldName;
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
		}
		if (!isSuperClass) {
			if (!newHashed(fieldElem, value)) {
				return fieldElem;
			}
		}
		if (fieldName != null) {
			fieldElem.setAttribute(ATTR_VALUETYPE, valueClass.getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		Hashtable table = (Hashtable) value;
		int size = table.size();
		fieldElem.setAttribute(ATTR_SIZE, size + "");
		Enumeration enumKey = table.keys();
		Element hashElem;
		Element keyElem;
		Element valueElem;
		int i = 0;
		while (enumKey.hasMoreElements()) {
			hashElem = doc.createElement(fName);
			hashElem.setAttribute(ATTR_INDEX, (i) + "");
			Object key_i = enumKey.nextElement();
			keyElem = createDOMElement(fName, key_i);
			keyElem.setAttribute(ATTR_HASHKEY, (i) + "");
			hashElem.appendChild(keyElem);
			Object value_i = table.get(key_i);
			valueElem = createDOMElement(fName, value_i);
			valueElem.setAttribute(ATTR_HASHVALUE, (i++) + "");
			hashElem.appendChild(valueElem);
			fieldElem.appendChild(hashElem);
		}
		return fieldElem;
	}

	private Element createDOMPrimitive(String fieldName, Object value)
			throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		Node text = createDOMText(value);
		if (text == null) {
			return null;
		}
		String fName;
		if (fieldName != null) {
			fName = fieldName;
		}
		else {
			fName = validXMLTag(value.getClass());
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
			logger.severe("\nDOMException: wrong XML Tag>" + fName + "<");
			fName = "DOMEXCEPTION";
			debug.println("\nDOMException: Final XML tag " + fName);
			fieldElem = doc.createElement(fName);
		}
		if (!newHashed(fieldElem, value)) {
			return fieldElem;
		}
		if (fieldName != null) {
			Class clazz = value.getClass();
			fieldElem.setAttribute(ATTR_VALUETYPE, value.getClass().getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		fieldElem.appendChild(text);
		return fieldElem;
	}

	private Element createDOMSpecial(String fieldName, Object value)
			throws IOException {
		Class valueClass = null;
		if (value instanceof Class) {
			valueClass = (Class) value;
		}
		else {
			valueClass = value.getClass();
		}
		return createDOMSpecial(fieldName, value, valueClass);
	}

	private Element createDOMSpecial(String tag, Object obj, Class objClass)
			throws IOException {
		Element elem = null;
		if (objClass.isArray()) {
			elem = createDOMArray(tag, obj);
		}
		else if (objClass == Class.class) {
			elem = doc.createElement("Class.class-NOT-YET-IMPLEMENTED");
			throw new RuntimeException("Class.class-NOT-YET-IMPLEMENTED");
		}
		else if (objClass == Vector.class) {
			elem = createDOMVector(tag, obj);
		}
		else if (objClass == Hashtable.class) {
			elem = createDOMHashtable(tag, obj);
		}
		else if (obj instanceof File) {
			elem = createDOMFile(tag, obj);
		}
		else if (obj instanceof Date) {
			elem = createDOMDate(tag, obj);
		}
		return elem;
	}

	private Node createDOMText(Object fieldValue) {
		Node text = null;
		if (fieldValue == null) {
		}
		else if ((fieldValue instanceof Boolean)
				|| (fieldValue instanceof Byte)
				|| (fieldValue instanceof Short)
				|| (fieldValue instanceof Integer)
				|| (fieldValue instanceof Long)
				|| (fieldValue instanceof Float)
				|| (fieldValue instanceof Double)) {
			text = doc.createTextNode(fieldValue.toString());
		}
		else if ((fieldValue instanceof Character)
				|| (fieldValue instanceof String)) {
			//          text = doc.createTextNode(fieldValue.toString());
			if (USE_CDATA_SECTION) {
				text = doc.createCDATASection(fieldValue.toString());
			}
			else {
				text = doc.createTextNode(validXML(fieldValue.toString()));
			}
		}
		else if (fieldValue instanceof Byte[]) {
			Byte[] val = (Byte[]) fieldValue;
			//text = doc.createTextNode(val.toString());
			byte[] valbyte = new byte[val.length];
			for (int i = 0; i < val.length; i++) {
				valbyte[i] = val[i].byteValue();
			}
			String valString = encoder.encode(valbyte);
			//text = createDOMText(valString);
			text = doc.createCDATASection(valString);
		}
		else if (fieldValue instanceof byte[]) {
			byte[] val = (byte[]) fieldValue;
			long timing = System.currentTimeMillis();
			String valString = encoder.encode(val);
			timing = System.currentTimeMillis() - timing;
			if (TIMING > 2) {
				logger.info("TIMING[3]: BASE64 encoding in " + timing
						+ " milliseconds");
			}
			text = doc.createCDATASection(valString);
		}
		/*
		 * / Alternative approach: write "primitive" arrays as comma separated
		 * list else if ( (fieldValue instanceof Boolean[]) || (fieldValue
		 * instanceof Character[]) || (fieldValue instanceof Byte[]) ||
		 * (fieldValue instanceof Short[]) || (fieldValue instanceof Integer[]) ||
		 * (fieldValue instanceof Long[]) || (fieldValue instanceof Float[]) ||
		 * (fieldValue instanceof Double[]) ) / String[] handled as array {
		 * /text = doc.createTextNode(fieldValue.toString()); /String s =
		 * ((Long[])fieldValue).toString(); /text =
		 * doc.createTextNode(((Long[])fieldValue).toString()); Object[] val =
		 * (Object[])fieldValue; int len = val.length; StringBuffer sb = new
		 * StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * boolean[]) { boolean[] val = (boolean[])fieldValue; StringBuffer sb =
		 * new StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * char[]) { char[] val = (char[])fieldValue; text =
		 * doc.createTextNode(val.toString()); } else if (fieldValue instanceof
		 * byte[]) { byte[] val = (byte[])fieldValue; text =
		 * doc.createTextNode(val.toString()); } else if (fieldValue instanceof
		 * short[]) { short[] val = (short[])fieldValue; StringBuffer sb = new
		 * StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * int[]) { int[] val = (int[])fieldValue; StringBuffer sb = new
		 * StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * long[]) { long[] val = (long[])fieldValue; StringBuffer sb = new
		 * StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * float[]) { float[] val = (float[])fieldValue; StringBuffer sb = new
		 * StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); } else if (fieldValue instanceof
		 * double[]) { double[] val = (double[])fieldValue; StringBuffer sb =
		 * new StringBuffer(10000); int i; for (i=0; i <val.length-1;i++)
		 * sb.append(val[i]+","); sb.append(val[i]); text =
		 * doc.createTextNode(sb.toString()); }
		 */
		return text;
	}

	private Element createDOMVector(String fieldName, Object value)
			throws IOException {
		boolean isSuperClass = false;
		return createDOMVector(fieldName, value, value.getClass(), isSuperClass);
	}

	private Element createDOMVector(String fieldName, Object value,
			Class valueClass, boolean isSuperClass) throws IOException {
		Element fieldElem = null;
		if (value == null) {
			fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			return fieldElem;
		}
		String fName;
		if (fieldName == null || isSuperClass) {
			fName = validXMLTag(valueClass);
		}
		else {
			fName = fieldName;
		}
		try {
			fieldElem = doc.createElement(fName);
		}
		catch (DOMException e) {
			logger.log(Level.SEVERE, "", e);
		}
		if (!isSuperClass) {
			if (!newHashed(fieldElem, value)) {
				return fieldElem;
			}
		}
		if (fieldName != null) {
			fieldElem.setAttribute(ATTR_VALUETYPE, valueClass.getName());
		}
		//        fieldElem.setAttribute("valueType", getTypeName(valueType));
		//        fieldElem.setAttribute("fieldType", getTypeName(fieldType));
		Vector valueVector = (Vector) value;
		int size = valueVector.size();
		fieldElem.setAttribute(ATTR_SIZE, size + "");
		Element vectorElem;
		for (int i = 0; i < size; i++) {
			Object value_i = valueVector.get(i);
			vectorElem = createDOMElement(fName, value_i);
			vectorElem.setAttribute(ATTR_INDEX, i + "");
			fieldElem.appendChild(vectorElem);
		}
		return fieldElem;
	}

	private boolean newHashed(Element elem, Object obj) {
		KeyObject key = new KeyObject(obj);
		ValueObject prev = (ValueObject) objTable.get(key);
		if (prev != null) {
			if (obj == prev.obj) {
				debug.println(" IDREF=" + prev.objID + ", object " + obj
						+ " already written");
				elem.setAttribute(ATTR_IDREF, prev.objID + "");
			}
			else {
				String error = "ERROR in XMLObjectWriter: identical keys for distinct objects!!!"
						+ "\nPrev: "
						+ prev.obj.getClass()
						+ "@"
						+ System.identityHashCode(prev.obj)
						+ "\nNew:  "
						+ obj.getClass() + "@" + System.identityHashCode(obj);
				logger.severe(error);
				UserMessages.error(error);
				elem.setAttribute("WRONG_ID", prev.objID + ":"
						+ prev.hashCode());
			}
			return false;
		}
		else {
			objectID++;
			objTable.put(key, new ValueObject(obj, objectID));
			debug.println(" ID=" + objectID);
			elem.setAttribute(ATTR_ID, objectID + "");
			return true;
		}
	}

	private void scanFields(Element elem, Object obj, Object classobj)
			throws Exception {
		Class objClass;
		if (classobj instanceof Class) {
			objClass = (Class) classobj;
		}
		else {
			objClass = classobj.getClass();
		}
		// Return if top of class chain reached
		if (objClass == null || objClass == Object.class) {
			return;
		}
		debug.println("CLASS: " + objClass.getName());
		XMLObjectClass xmlClass = XMLObjectClass.getClassXML(objClass);
		// Objects may create DOM elements by themselves
		/*
		 * Method writeObjectMethod = getDeclaredMethod(objClass,
		 * "writeObjectXML", OOS_ARGS, Modifier.PRIVATE, Modifier.STATIC); if
		 * (writeObjectMethod != null) { writeObjectArglist = new
		 * ObjectOutputStream[] {new ObjectOutputStream(out)};
		 * writeObjectMethod.invoke(obj, writeObjectArglist); }
		 */
		Method createDOMElementMethod = xmlClass.getCreateDOMElement();
		if (createDOMElementMethod != null) {
			Object[] createDOMElementArgs = new Element[]{elem};
			createDOMElementMethod.invoke(obj, createDOMElementArgs);
			return;
		}
		/*
		 * if (xmlClass.overwritesSerialization()) { / createDOMSpecial(elem,
		 * obj, objClass); Element thisElem = createDOMElement(null, this);
		 * elem.appendChild(thisElem); return; }
		 */
		Element fieldElem = null;
		boolean isSuperClass = true;
		if (obj instanceof Vector) {
			fieldElem = createDOMVector(null, obj, Vector.class, isSuperClass);
			elem.appendChild(fieldElem);
		}
		else if (obj instanceof Hashtable) {
			fieldElem = createDOMHashtable(null, obj, Hashtable.class,
					isSuperClass);
			elem.appendChild(fieldElem);
		}
		XMLObjectField xmlFields[] = xmlClass.getFieldsXML();
		// Scan over all fields of the current class
		for (int i = 0; i < xmlFields.length; i++) {
			Field field = xmlFields[i].getField();
			Object value = field.get(obj);
			Class fieldType = field.getType();
			String fieldName = xmlFields[i].getFieldNameXML();
			if (value == null) {
				debug.println("field[" + i + "]: " + field);
				fieldElem = doc.createElement(fieldName + NULL_INDICATOR);
			}
			else {
				Class valueType = value.getClass();
				debug.println("field[" + i + "]: " + field);
				try {
					debug.println("value: " + valueType + "(" + value + ")");
				}
				catch (NullPointerException npe) {
					debug.println("value: " + valueType + "("
							+ npe.getMessage() + ")");
				}
				try {
					fieldElem = createDOMElement(fieldName, value);
				}
				catch (Exception e) {
					logger.log(Level.SEVERE, "ERROR in scanFields: ", e);
					logger.severe("CLASS: " + objClass.getName());
					logger.severe("FIELD: " + fieldName);
				}
			}
			if (FIELDTYPE) {
				fieldElem.setAttribute(ATTR_FIELDTYPE, fieldType.getName());
			}
			elem.appendChild(fieldElem);
		}
	}

	public synchronized void writeObjectXML(Object obj) throws IOException {
		long timing = System.currentTimeMillis();
		long tstart = timing;
		// Reset hash table for objects and object ID counter
		objTable.clear();
		objectID = 0;
		debug.println("\n\nSTARTING DOM OUTPUT of " + obj + "\nSTARTED");
		if (obj == null) {
			debug.println("tried writing NULL object");
			// throw new IOException("tried writing NULL object");
			return;
		}
		Element root = doc.createElement(TAG_UNICORE);
		Element elemObj = createDOMElement(null, obj);
		root.appendChild(elemObj);
		root.setAttribute(ATTR_VERSION, versionUnicoreXML);
		root.setAttribute(ATTR_CHECKSUM, getCheckSum(elemObj) + "");
		long tstop = System.currentTimeMillis();
		if (TIMING > 1) {
			logger
					.info("TIMING[2]: Creating DOM element (tree) from object in "
							+ (tstop - tstart) + " milliseconds");
		}
		tstart = tstop;
		String objClassName = this.validXMLTag(obj.getClass());
		Element elemObjBin = doc.createElement(TAG_BINARY + objClassName);
		if (obj instanceof Serializable) {
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bout);
			oos.writeObject(obj);
			oos.close();
			tstop = System.currentTimeMillis();
			if (TIMING > 1) {
				logger
						.info("TIMING[2]: Java Object Serialization to ByteArrayOutputStream in "
								+ (tstop - tstart) + " milliseconds");
			}
			tstart = tstop;
			Node textObjBin = createDOMText(bout.toByteArray());
			elemObjBin.appendChild(textObjBin);
			tstop = System.currentTimeMillis();
			if (TIMING > 1) {
				logger
						.info("TIMING[2]: Generating DOM element (encoded) from ByteArrayOutputStream in "
								+ (tstop - tstart) + " milliseconds");
			}
			tstart = tstop;
		}
		root.appendChild(elemObjBin);
		if (doc.hasChildNodes()) {
			doc.replaceChild(root, doc.getFirstChild());
		}
		else {
			doc.appendChild(root);
		}
		try {
			Transformer transformer = tFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result;
			if (out != null) {
				result = new StreamResult(out);
			}
			else if (writer != null) {
				result = new StreamResult(writer);
			}
			else {
				throw new Exception(
						"ERROR in XMLObjectWriter: neither OutputStream nor Writer specified");
			}
			transformer.setOutputProperty(
					javax.xml.transform.OutputKeys.INDENT, "yes");
			transformer.setOutputProperty(
					"{http://xml.apache.org/xslt}indent-amount", "2");
			transformer.transform(source, result);
			if (out != null) {
				out.close();
			}
			else if (writer != null) {
				writer.close();
			}
			else {
				throw new Exception(
						"ERROR in XMLObjectWriter: neither OutputStream nor Writer specified");
			}
			tstop = System.currentTimeMillis();
			if (TIMING > 1) {
				logger
						.info("TIMING[2]: Transforming (writing) DOM tree to file in "
								+ (tstop - tstart) + " milliseconds");
			}
			tstart = tstop;
			if (DEBUG >= 3) {
				StreamResult resdebug = new StreamResult(new FileWriter(
						"tttwrite.xml"));
				Transformer transdebug = tFactory.newTransformer();
				transdebug.transform(source, resdebug);
			}
		}
		catch (TransformerConfigurationException tce) {
			// Error generated by the parser
			logger.severe("\n** Transformer Factory error");
			logger.severe("   " + tce.getMessage());
			// Use the contained exception, if any
			Throwable x = tce;
			if (tce.getException() != null) {
				x = tce.getException();
			}
			logger.log(Level.SEVERE, "", x);
		}
		catch (TransformerException te) {
			// Error generated by the parser
			logger.severe("\n** Transformation error");
			logger.severe("   " + te.getMessage());
			// Use the contained exception, if any
			Throwable x = te;
			if (te.getException() != null) {
				x = te.getException();
			}
			logger.log(Level.SEVERE, "", x);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		debug.println("FINISHED new DOM OUTPUT of " + obj);
		timing = System.currentTimeMillis() - timing;
		if (TIMING > 0) {
			logger.info("TIMING[1]: XML format written in " + timing
					+ " milliseconds");
		}
	}
}