/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.util.ResourceBundle;
import org.unicore.ajo.ActionGroup;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.ImportFactory;

/**
 * Container for all import tasks that import files from different locations
 * into one uspace.
 * 
 * @author Ralf Ratering
 * @version $Id: FileImportContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class FileImportContainer extends FileContainer {

	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = 6149750275075422934L;

	/**
	 * Constructor builds file container as child of job container
	 * 
	 * @param parentContainer
	 *            parent job group
	 */
	public FileImportContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 * Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		// import files
		ImportFactory importFactory = new ImportFactory(getFileImports(),
				actionGroup, this);
		importFactory.build();
		// copy identifier from action group to container to to have a unique id
		// for the njs.
		syncIdentifiers();
	}

	/**
	 * Add container specific errors to those of super classes
	 * 
	 * @return ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();
		if (fileImports == null || fileImports.length == 0) {
			errorSet.add(new UError(getIdentifier(), res
					.getString("NO_IMPORTS")));
		}
		setErrors(errorSet);
		return errorSet;
	}
}