/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.util.ResourceBundle;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Time;
import org.unicore.ajo.WaitUntil;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.extensions.HoldJobTest;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @author Sven Haubold
 * @version $Id: HoldJobContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class HoldJobContainer extends TaskContainer {

	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -9063565457856815042L;
	private HoldJobTest test = new HoldJobTest();
	private WaitUntil waitUntil;

	public HoldJobContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	public void buildActionGroup() {
		// build the action group
		actionGroup = new ActionGroup(getName() + "_HOLD_JOB_"
				+ ResourceManager.getNextObjectIdentifier());
		// synchronize identifiers to find corresponding outcomes for
		// container in jmc
		setIdentifier(actionGroup.getId());
		long time = test.getDate().getTime();
		if (!test.isEnabled()) {
			time = 0;
		}
		else if (test.getType().equals(HoldJobTest.FOREVER)) {
			time = Long.MAX_VALUE;
		}
		waitUntil = new WaitUntil(getName() + "_WAIT_UNTIL_"
				+ ResourceManager.getNextObjectIdentifier());
		// set ignore failure flag
		// This makes it possible to resume job execution by sending a
		// Control.ABORT Action
		// to the WaitUntil task
		waitUntil.setPropagateSuccessfulIfNotSuccessful(true);
		waitUntil.setTime(new Time(time));
		actionGroup.add(waitUntil);
	}

	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		test.checkContents(err, this);
		setErrors(err);
		return err;
	}

	public HoldJobTest getHoldJobTest() {
		return test;
	}

	/**
	 * WaitUntil task will be aborted from jmc tree, if the user wants to resume
	 * job execution.
	 * 
	 *  
	 */
	public WaitUntil getWaitUntil() {
		return waitUntil;
	}

	public void setHolJobTest(HoldJobTest test) {
		this.test = test;
	}
}