/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ListDirectory;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ListDirectory_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.XFileEnumeration;
import org.unicore.sets.XFileSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Get the listing for a remote directory
 * 
 * @author Ralf Ratering
 * @version $Id: GetListing.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetListing extends ObservableRequestThread {

	private RemoteFile dir;
	private XFileSet xFileSet;

	/**
	 * Constructor
	 * 
	 * @param dir
	 *            Get listing for this directory
	 */
	public GetListing(RemoteFile dir) {
		this.dir = dir;
	}

	/**
	 * Get directory
	 * 
	 * @return directory
	 */
	public RemoteFile getDirectory() {
		return dir;
	}

	/**
	 * Get listing as array of RemoteFile objects
	 * 
	 * @return listing
	 */
	public RemoteFile[] getRemoteFileArray() {
		Vector listing = new Vector();
		XFileEnumeration fileEnum = xFileSet.elements();
		while (fileEnum.hasMoreElements()) {
			XFile xfile = (XFile) fileEnum.nextElement();
			RemoteFile file = new RemoteFile(xfile, dir, dir.getFileSystem());
			listing.add(file);
		}
		RemoteFile[] fileArray = new RemoteFile[listing.size()];
		for (int i = 0; i < listing.size(); i++) {
			fileArray[i] = (RemoteFile) listing.elementAt(i);
		}
		return fileArray;
	}

	/**
	 * Start the thread
	 */
	public void run() {
		Vector listing = new Vector();
		String dirname = dir.getSubspacePath();
		Storage storage = dir.getStorage();
		Vsite vsite = dir.getVsite();
		ListDirectory listDir = new ListDirectory("ListDirectory");
		listDir.setResources(new ResourceSet(storage));
		listDir.setTarget(dirname);
		User user = ResourceManager.getUser(vsite);
		AbstractJob job = new AbstractJob(ResourceManager.getServicePrefix()
				+ "_GetListingsJob_"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				user);
		job.add(listDir);
		logger
				.fine("Getting listing for " + dirname + " at storage "
						+ storage);
		Reply reply = nonPolling(job, vsite, user);
		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning("Virtual site not available: " + vsite
					+ "\nReason: " + reply.getLastEntry().getComment());
			return;
		}
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes)).readObject();
				AbstractJob_Outcome ajOut = ConsignForm
						.convertFrom((AbstractJob_Outcome) outcome);
				Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
				if (ajOutcome instanceof ListDirectory_Outcome) {
					ListDirectory_Outcome listDirOutcome = (ListDirectory_Outcome) ajOutcome;
					xFileSet = listDirOutcome.getListing();
				}
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get directory listing ", e);
			}
		}
		if (xFileSet == null) {
			logger
					.severe("List Directory request did not return a listing for "
							+ dirname + " at storage " + storage);
			xFileSet = new XFileSet();
		}
		// notify all my observers on task end
		notifyObservers(this, xFileSet);
	}
}