/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.util.logging.Logger;
import com.pallas.unicore.threadpool.GenericObservable;
import com.pallas.unicore.threadpool.IObservable;
import com.pallas.unicore.threadpool.IObserver;

/**
 * A thread that can be observed by IObservers
 * 
 * @author Ralf Ratering
 * @version $Id: ObservableThread.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public abstract class ObservableThread extends Thread implements IObservable {

	/**
	 * standard logger
	 */
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.requests");
	/**
	 * encapsulated observable object
	 */
	protected GenericObservable theGenericObservable = new GenericObservable();

	/**
	 * add an observer to this thread
	 * 
	 * @param anObserver
	 *            IObservable that watches this thread
	 */
	public void addObserver(IObserver anObserver) {
		theGenericObservable.addObserver(anObserver);
	}

	/**
	 * Remove all observers
	 */
	public void deleteAllObservers() {
		theGenericObservable.deleteAllObservers();
	}

	/**
	 * Remove a specific observer
	 * 
	 * @param anObserver
	 *            IObservable to be removed
	 */
	public void deleteObserver(IObserver anObserver) {
		this.theGenericObservable.deleteObserver(anObserver);
	}

	/**
	 * Notify our observers about an event that occured
	 * 
	 * @param theObserved
	 *            Observer
	 * @param changeCode
	 *            description of the event that occured
	 */
	public void notifyObservers(Object theObserved, Object changeCode) {
		theGenericObservable.notifyObservers(theObserved, changeCode);
	}
}