/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.resourcemanager;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.StreamCorruptedException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.net.UnknownHostException;
import java.security.Policy;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Vector;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.resources.Resource;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.ListVsites;
import org.unicore.upl.ListVsitesReply;
import org.unicore.upl.Reply;
import org.unicore.upl.UnicoreResponse;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import com.fujitsu.arcon.upl.ListPorts;
import com.fujitsu.arcon.upl.ListPortsReply;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.SystemDefaults;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.dialogs.DefinePasswordDialog;
import com.pallas.unicore.client.dialogs.ServiceURLDialog;
import com.pallas.unicore.client.dialogs.UserDefaultsDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.util.SiteDefaults;
import com.pallas.unicore.client.xml.NewXMLFormatter;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.connection.ConnectionManager;
import com.pallas.unicore.connection.URLReader;
import com.pallas.unicore.connection.UnicoreSSLSocketFactory;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.AjoContainer;
import com.pallas.unicore.container.DoNContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.container.ElseContainer;
import com.pallas.unicore.container.FileContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.ThenContainer;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.security.CertificateUtility;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.security.UnicoreSecurityManager;
import com.pallas.unicore.utility.LoggingLevels;
import com.pallas.unicore.utility.UrlEncoder;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.Version;

/**
 * ResourceManager class keeps track of all global objects as static members.
 * These objects are made accessible to other classes via static methods, so
 * that all classes pass objects to each other via the ResourceManager class.
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: ResourceManager.java,v 1.2 2004/06/16 15:24:49 rmenday Exp $
 */
public class ResourceManager {

	/**
	 * String as description for resources that have not been sent from a vsite
	 */
	public final static String USER_RESOURCE_DESCRIPTION = "User Resource";
	/**
	 * logs are written into a new log directory
	 */
	public final static int NEW_LOG_DIRECTORY = 0;
	/**
	 * logs are written into a file with XML-format
	 */
	public final static int LOG_INTO_XMLFILE = 1;
	/**
	 * logs are written into a file with simple format
	 */
	public final static int LOG_INTO_PLAINFILE = 2;
	/**
	 * logs are written into files both with XML-format and with simple format
	 */
	public final static int LOG_INTO_FILES = 3;
	/**
	 * logs are written to the console
	 */
	public final static int LOG_TO_CONSOLE = 4;
	public final static String ABSTRACTTASK = "abstractTask.gif";
	public final static String ACTIONGROUP = "actionGroup.gif";
	public final static String ABSTRACTJOB = "abstractJob.gif";
	public final static String ADD = "add.gif";
	public final static String ARROW = "arrow.gif";
	public final static String CHECK = "check.gif";
	public final static String CLEAR = "clear.gif";
	public final static String CONNECTION = "estab_conn.gif";
	public final static String COPY = "copy.gif";
	public final static String CUT = "cut.gif";
	public final static String DON = "doN.gif";
	public final static String DOREPEAT = "doRepeat.gif";
	public final static String DOWN = "down.gif";
	public final static String EDIT = "edit.gif";
	public final static String ELSE = "else.gif";
	public final static String FETCHOUTPUT = "fetchOutput.gif";
	public final static String FILEOPERATION = "fileOperation.gif";
	public final static String HOLDJOB = "holdJob.gif";
	public final static String IF = "if.gif";
	public final static String JOBGROUP = "jobGroup.gif";
	public final static String KEY = "key.gif";
	public final static String NEW = "new.gif";
	public final static String NOCONN = "no_conn.gif";
	public final static String NOKEY = "nokey.gif";
	public final static String OPEN = "open.gif";
	public final static String PASTE = "paste.gif";
	public final static String REDO = "redo.gif";
	public final static String REFRESH = "refresh.gif";
	public final static String REMOVE = "remove.gif";
	public final static String SAVE = "save.gif";
	public final static String SAVEAS = "saveAs.gif";
	public final static String SELECTALL = "selectAll.gif";
	public final static String SORT = "sort.gif";
	public final static String STOP = "stop.gif";
	public final static String SUBMIT = "submit.gif";
	public final static String THEN = "then.gif";
	public final static String UNAVAILABLEUSITE = "unavailableUsite.gif";
	public final static String UNAVAILABLEVSITE = "unavailableVsite.gif";
	public final static String UNDO = "undo.gif";
	public final static String UNICORE = "unicore.gif";
	public final static String UNICOREICON = "UNICOREIcon16x16.gif";
	public final static String UNKNOWNTASK = "unknownTask.gif";
	public final static String UP = "up.gif";
	public final static String USITE = "usite.gif";
	public final static String VSITE = "vsite.gif";
	public final static String WAIT = "wait.gif";
	public final static String X509VERSION1 = "x509Version1.gif";
	public final static String X509CRITEXTENSION = "x509CritExtension.gif";
	public final static String X509NONCRITEXTENSION = "x509NonCritExtension.gif";
	private final static String servicePrefix = "@JMC@";
	private final static String[] imageFilenames = {ABSTRACTTASK, ACTIONGROUP,
			ABSTRACTJOB, ADD, ARROW, CHECK, CLEAR, CONNECTION, COPY, CUT, DON,
			DOREPEAT, DOWN, EDIT, ELSE, FETCHOUTPUT, FILEOPERATION, HOLDJOB,
			IF, JOBGROUP, KEY, NEW, NOCONN, NOKEY, OPEN, PASTE, REDO, REFRESH,
			REMOVE, SAVE, SAVEAS, SELECTALL, SORT, STOP, SUBMIT, THEN,
			UNAVAILABLEUSITE, UNAVAILABLEVSITE, UNDO, UNICORE, UNICOREICON,
			UNKNOWNTASK, UP, USITE, VSITE, WAIT, X509VERSION1,
			X509CRITEXTENSION, X509NONCRITEXTENSION};
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.resourcemanager.ResourceStrings");
	private static int logInto = LOG_TO_CONSOLE;
	// used for logging
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.resourcemanager");
	private static boolean offline = false;
	private static Object lock = new Object();
	private static Handler fileXmlHandler = null;
	private static Handler filePlainHandler = null;
	private static Handler consoleHandler = null;
	private static File fileXml = null;
	private static File filePlain = null;
	// Use of path "img"+file.separator not appropriate on Windows
	// when storing img directory in a jar file.
	private static String imagePath = "com/pallas/unicore/img/";
	private static boolean initialized = false;
	// the resource manager delivers an identifier
	private static long objectIdentifier = 0;
	private static ResourceCache resourceCache = new ResourceCache();
	private static LoggingLevels logLevels = new LoggingLevels();
	private static Hashtable userTable = new Hashtable();
	private static SimpleDateFormat dateFormatter = new SimpleDateFormat(
			"MM/dd/yyyy"), timeFormatter = new SimpleDateFormat("HH:mm:ss"),
			completeFormatter = new SimpleDateFormat("HH:mm:ss MM/dd/yyyy"),
			utcFormatter = new SimpleDateFormat("HH:mm:ss MM/dd/yyyy");
	private static String resourceCacheFilename;
	// reference to current client instance
	private static Client currentInstance;
	private static UnicoreSSLSocketFactory sslFactory;
	private static SecureRandom secureRandom;
	// security objects
	private static KeyStoreManager keystoreManager;
	private static UnicoreSecurityManager securityManager;
	// private static char password[];
	private static String defaultIdentity;
	// get urls with this one...
	private static URLReader urlReader;
	private static SystemDefaults systemDefaults;
	private static UserDefaults userDefaults;
	private static SiteDefaults siteDefaults;
	private static User user;
	private static String project;
	private static String[] usiteNames;
	private static String jarDir;
	private static Image[] image;
	private static Hashtable iconTable;
	private static Hashtable imageTable;
	private static Hashtable filechooserTable;
	// contains the URLs of the usite definitions
	private static Vector usiteURLs = new Vector();
	// connection stuff
	private static ConnectionManager connectionManager;

	/**
	 * Private Constructor for the ResourceManager object
	 */
	private ResourceManager() {
	}

	/**
	 * Sets the keystoreManager
	 * 
	 * @param manager
	 *            The new keystoreManager value
	 */
	public final static void setKeystoreManager(KeyStoreManager manager) {
		keystoreManager = manager;
	}

	/**
	 * Sets client to offline mode
	 * 
	 * @param b
	 *            true if offline
	 */
	public static void setOfflineMode(boolean b) {
		offline = b;
	}

	/**
	 * Sets current client instance
	 * 
	 * @param instance
	 *            current client instance
	 */
	public static void setCurrentInstance(Client instance) {
		currentInstance = instance;
	}

	public static void setSSLFactory(UnicoreSSLSocketFactory sslfact) {
		sslFactory = sslfact;
	}

	public static void setSecureRandom(SecureRandom random) {
		secureRandom = random;
	}

	public static void setUser(User user, Vsite vsite) {
		userTable.put(vsite, user);
	}

	public static void setUser(User user, Usite usite) {
		userTable.put(usite, user);
	}

	/**
	 * Set the relevat security policies for startup-dir, system die and plugin
	 * dir
	 */
	private static void setAdditionalPolicy() {
		// the directory containing system jar's
		String defaultLibDir = ResourceManager.getJarDirectory();
		defaultLibDir = defaultLibDir + "/*";
		if (!defaultLibDir.startsWith("http")
				&& !defaultLibDir.startsWith("HTTP")
				&& !defaultLibDir.startsWith("file")) {
			// assume file URL
			defaultLibDir = "file:" + defaultLibDir;
		}
		// get medium priority permissions (see unicore.policy)
		ResourceManager
				.expandPolicy("unicore.system.plugin.dir", defaultLibDir);
		// medium trust for plugins from the user plugin dir
		ResourceManager.expandPolicy("unicore.user.plugin.dir", "file:"
				+ userDefaults.getPluginDirectory() + "/*");
		ResourceManager.expandPolicy("unicore.dir", Client.getUnicoreDir());
	}

	public final static boolean getOfflineMode() {
		// offlineMode is if offline
		return offline;
	}

	public final static KeyStoreManager getKeystoreManager() {
		return keystoreManager;
	}

	public static ImageIcon getIcon(String name) {
		if (iconTable == null) {
			ResourceManager.loadImages();
		}
		ImageIcon icon = (ImageIcon) iconTable.get(name);
		if (icon == null) {
			icon = (ImageIcon) iconTable.get(UNKNOWNTASK);
		}
		return icon;
	}

	public static ImageIcon getIcon(ActionContainer container) {
		if (container instanceof AjoContainer) {
			return getIcon(ABSTRACTJOB);
		}
		else if (container instanceof JobContainer) {
			return getIcon(JOBGROUP);
		}
		else if (container instanceof DoNContainer) {
			return getIcon(DON);
		}
		else if (container instanceof DoRepeatContainer) {
			return getIcon(DOREPEAT);
		}
		else if (container instanceof HoldJobContainer) {
			return getIcon(HOLDJOB);
		}
		else if (container instanceof IfContainer) {
			return getIcon(IF);
		}
		else if (container instanceof ThenContainer) {
			return getIcon(THEN);
		}
		else if (container instanceof ElseContainer) {
			return getIcon(ELSE);
		}
		else if (container instanceof FileContainer) {
			return getIcon(FILEOPERATION);
		}
		else if (container instanceof ActionContainer) {
			// if we got here container belongs to a plugin
			return Client.getPluginManager().getIcon(container);
		}
		return getIcon(UNKNOWNTASK);
	}

	public static synchronized String getNextObjectIdentifier() {
		return System.currentTimeMillis() + "_" + objectIdentifier++;
	}

	public static boolean isOffline() {
		return offline;
	}

	public static String getJarDirectory() {
		if (jarDir != null) {
			return jarDir;
		}
		String classFile = "com/pallas/unicore/client/Client.class";
		// dirty, but works....
		URL classDir = currentInstance.getClass().getClassLoader().getResource(
				classFile);
		// if we have been started from a jar, look in jar directory
		// for plugin jars
		if (classDir.toString().startsWith("jar:")) {
			// classDir :=
			// jar:file:/home/rrate/tmp/client31.jar!/com/pallas/unicore/client/resourcemanager/ResourceManager.class
			String tmp = classDir.getFile();
			// cutoff "file:" prefix
			tmp = tmp.substring(tmp.indexOf("file:") + 5);
			// cutoff everything behind "!"
			tmp = tmp.substring(0, tmp.indexOf("!"));
			// cutoff jar name
			jarDir = tmp.substring(0, tmp.lastIndexOf("/"));
			jarDir = UrlEncoder.decodeURL(jarDir);
		}
		// not started from jar -> try to use Jars dir
		else {
			// Assumption classDir :=
			// /home/kdoertel/Unicore/unicore-4.0/classes/com/pallas/unicore/client/Client.class
			String tmp = classDir.getFile();
			String classesDir = "classes/" + classFile;
			// HACK: cutoff everything behind classesDir and add "lib" directory
			if (tmp.endsWith(classesDir)) {
				jarDir = tmp.substring(0, tmp.lastIndexOf(classesDir)) + "lib";
				jarDir = UrlEncoder.decodeURL(jarDir);
			}
			else {
				// use current working dir
				jarDir = System.getProperty("user.dir");
			}
		}
		return jarDir;
	}

	public static Usite getUsite(Vsite vsite) {
		return resourceCache.getUsite(vsite);
	}

	/**
	 * Get all know usites
	 * 
	 * @return The usites value
	 */
	public static Vector getAllUsites() {
		return resourceCache.getUsites();
	}

	/**
	 * Get all know Vsites for a usite
	 * 
	 * @return The usites value
	 */
	public static Vector getAllVsites(Usite usite) {
		return resourceCache.getVsites(usite);
	}

	public static Vector getUsites() {
		Vector enabledUsites = new Vector();
		Vector allUsites = resourceCache.getUsites();
		boolean enabled = false;
		for (int i = 0; i < allUsites.size(); i++) {
			Usite next = (Usite) allUsites.elementAt(i);
			if (siteDefaults.isUsiteContained((next))) {
				enabled = siteDefaults.getUsiteValue(next);
				if (enabled) {
					// if the usites is contained in the defaults and enabled
					// add to list
					enabledUsites.add(next);
				}
			}
			else {
				// usite is not contained in the defaults add to list and
				// defaults
				enabledUsites.add(next);
				siteDefaults.setUsite(next, true);
			}
		}
		return enabledUsites;
	}

	public static Vector getVsites(Usite usite) {
		Vector enabledVsites = new Vector();
		Vector allVsites = resourceCache.getVsites(usite);
		boolean enabled = false;
		for (int i = 0; i < allVsites.size(); i++) {
			Vsite next = (Vsite) allVsites.elementAt(i);
			if (siteDefaults.isVsiteContained((next))) {
				enabled = siteDefaults.getVsiteValue(next);
				if (enabled) {
					// if the usites is contained in the defaults and enabled
					// add to list
					enabledVsites.add(next);
				}
			}
			else {
				// usite is not contained in the defaults add to list and
				// defaults
				enabledVsites.add(next);
				siteDefaults.setVsite(next, true);
			}
		}
		return enabledVsites;
	}

	/**
	 * Returns a list of all configured URL's to read usites from
	 * 
	 * @return Vector containing URL strings
	 */
	public static Vector getUsiteURLs() {
		return usiteURLs;
	}

	/**
	 * Returns a combined String from vsite name and reference/address This is
	 * only a hack to fix the brocken vsite.equals method
	 * 
	 */
	public static Object getUniqueVsiteIdentifier(Vsite vsite) {
		return vsite.getName() + vsite.getReference().getAddress();
		// return vsite;
	}

	public static NamedResourceSet getResourceSet(Vsite vsite) {
		return resourceCache.getResourceSet(vsite);
	}

	/**
	 * Returns the Vsite from the cache matching the characteristics of the
	 * provided vsite, which may be a different object.
	 * 
	 * @param vsite
	 *            Vsite in question
	 * @return Matching vsite or null
	 */
	public static Vsite getMatchingVsite(Vsite vsite) {
		return resourceCache.getMatchingVsite(vsite);
	}

	public static boolean isUsiteAvailable(Usite usite) {
		return resourceCache.isUsiteAvailable(usite);
	}

	public static boolean isVsiteAvailable(Vsite vsite) {
		return resourceCache.isVsiteAvailable(vsite);
	}

	public static boolean isSpecialVsite(Vsite vsite) {
		return resourceCache.isSpecialVsite(vsite);
	}

	public static String getUsiteErrorMessage(Usite usite) {
		return resourceCache.getUsiteErrorMessage(usite);
	}

	public static String getVsiteErrorMessage(Vsite vsite) {
		return resourceCache.getVsiteErrorMessage(vsite);
	}

	public static Client getCurrentInstance() {
		return currentInstance;
	}

	public static SystemDefaults getSystemDefaults() {
		return systemDefaults;
	}

	public static UnicoreSSLSocketFactory getSSLFactory() {
		return sslFactory;
	}

	/**
	 * Returns a secure random value
	 */
	public static SecureRandom getSecureRandom() {
		return secureRandom;
	}

	public static UnicoreSecurityManager getSecurityManager() {
		return securityManager;
	}

	/**
	 * Get reference to UserDefaults
	 */
	public static UserDefaults getUserDefaults() {
		return userDefaults;
	}

	/**
	 * Get Service Action Prefix
	 */
	public static String getServicePrefix() {
		return servicePrefix;
	}

	/**
	 * Get current Account
	 */
	public static String getProject() {
		return project;
	}

	/**
	 * Get current User
	 */
	public static User getUser() {
		return user;
	}

	/**
	 * Get current User for a Vsite
	 */
	public static User getUser(Vsite vsite) {
		if (vsite == null) {
			return user;
		}
		User returnUser = (User) userTable.get(vsite);
		if (returnUser == null) {
			// try to initialize userTable from UserDefaults
			// which might have not yet happened due to empty ResourceCache
			initUserVsite(vsite);
			if ((returnUser = (User) userTable.get(vsite)) == null) {
				// use identity of corresponding Usite (cf. SetIdentityDialog)
				Usite usite = getUsite(vsite);
				returnUser = getUser(usite);
			}
		}
		return returnUser;
	}

	public static User getUser(Usite usite) {
		if (usite == null) {
			return user;
		}
		User returnUser = (User) userTable.get(usite);
		if (returnUser == null) {
			// try to initialize userTable from UserDefaults
			// which might have not yet happened due to empty ResourceCache
			initUserUsite(usite);
			if ((returnUser = (User) userTable.get(usite)) == null) {
				// return default user if nothing special available
				returnUser = user;
			}
		}
		return returnUser;
	}

	public static ResourceCache getResourceCache() {
		return resourceCache;
	}

	public static ConnectionManager getConnectionManager() {
		return connectionManager;
	}

	public static Image getImage(String name) {
		return (Image) imageTable.get(name);
	}

	/**
	 * Gets a cached SelectorDialog. If the requested dialog does not exist a
	 * new one will be built.
	 * 
	 * @param vsite
	 *            if vsite is null a local SelectorDialog will be returned
	 * @param selectedFile
	 *            initially selected File
	 * @return a SelectorDialog
	 */
	public static SelectorDialog getSelectorDialog(File selectedFile,
			Vsite vsite) {
		Object key = vsite;
		if (vsite == null) {
			key = FileStorage.NSPACE_STRING;
		}
		SelectorDialog dialog = (SelectorDialog) filechooserTable.get(key);
		if (dialog == null) {
			dialog = new SelectorDialog(currentInstance, selectedFile, vsite);
			filechooserTable.put(key, dialog);
		}
		else {
			try {
				// reset chooser settings
				if (!dialog.getLocalEnabled()) {
					dialog.setLocalEnabled(true);
				}
				dialog.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				if (selectedFile != null) {
					dialog.setSelectedFile(selectedFile);
				}
				// remove filters from previous calls
				dialog.removeFileFilters();
				dialog.getSelectorTable().resetValues();
			}
			catch (FileNotFoundException e) {
				logger.log(Level.SEVERE, "File not found: " + selectedFile, e);
			}
		}
		return dialog;
	}

	/**
	 * Get a DateFormatter with format "MM/dd/yyyy". Use this method to get a
	 * format consistent with the rest of the client GUI.
	 */
	public static SimpleDateFormat getDateFormatter() {
		return dateFormatter;
	}

	/**
	 * Get a DateFormatter with format "HH:mm:ss". Use this method to get a
	 * format consistent with the rest of the client GUI.
	 */
	public static SimpleDateFormat getTimeFormatter() {
		return timeFormatter;
	}

	/**
	 * Get a DateFormatter with format "HH:mm:ss MM/dd/yyyy". Use this method to
	 * get a format consistent with the rest of the client GUI.
	 */
	public static SimpleDateFormat getCompleteFormatter() {
		return completeFormatter;
	}

	/**
	 * Get a DateFormatter translating local time to UTC. Use this method to get
	 * a format consistent with the rest of the client GUI.
	 */
	public static SimpleDateFormat getUTCFormatter() {
		return utcFormatter;
	}

	/**
	 * Check if the keystore file already exists. The filename is taken from the
	 * user defaults.
	 * 
	 * @return true if keystore file exists
	 */
	public static boolean isKeystoreAvailable() {
		String keystoreName = userDefaults.getKeyStoreName();
		File file = new File(keystoreName);
		// if unicore keystore exists, return true
		if (file.exists()) {
			return true;
		}
		return false;
	}

	public static String userInfo(User user) {
		String userinfo = null;
		try {
			userinfo = "\nuser alias: "
					+ keystoreManager.getAliasFromCertifcate(user
							.getCertificate()) + "\n      cert: "
					+ user.getCertificate().getSubjectDN() + "\n    xlogin: "
					+ user.getXlogin() + "\n   project: " + user.getProject();
		}
		catch (Exception e) {
			userinfo = "\nuser info not available";
			logger.log(Level.WARNING, userinfo, e);
		}
		return userinfo;
	}

	public static boolean vsiteExists(Vsite vsite) {
		return resourceCache.vsiteExists(vsite);
	}

	public static boolean usiteExists(Usite usite) {
		return resourceCache.usiteExists(usite);
	}

	/**
	 * Initialization routine tries to load UserDefaults from file
	 */
	public static void init(Client client) {
		currentInstance = client;
		// make sure constructor is called only once
		if (initialized) {
			return;
		}
		utcFormatter.setTimeZone(TimeZone.getTimeZone("Etc/UTC"));
		connectionManager = new ConnectionManager();
		// the directory (or jar) where we have been started from
		String classPath = System.getProperty("java.class.path");
		StringTokenizer tokenizer = new StringTokenizer(classPath,
				File.pathSeparator);
		java.net.URL url = null;
		try {
			url = new File(tokenizer.nextToken()).toURL();
			String bas = url.toString();
			ResourceManager.expandPolicy("unicore.startup.dir", url.toString());
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Could not install security policy for "
					+ url, e);
		}
		if (System.getProperty("com.pallas.unicore.security.manager") != null) {
			System.setSecurityManager(new SecurityManager());
		}
		SecurityManager sm = System.getSecurityManager();
		if (sm == null || !(sm instanceof SecurityManager)) {
			initialized = false;
			//        System.exit(-1);
		}
		initialized = true;
		// build system defaults
		if (systemDefaults == null) {
			systemDefaults = new SystemDefaults();
			String filename = getJarDirectory() + File.separator
					+ "SystemDefaults.txt";
			logger.info("Loading system defaults from file " + filename);
			if (!systemDefaults.loadFromFile(filename)) {
				UserMessages
						.warning("Missing system defaults!\nUNICORE Client may not work properly!");
			}
		}
		// build new user defaults, these will be loaded from
		// file or newly generated
		if (userDefaults == null) {
			userDefaults = new UserDefaults();
		}
		offline = userDefaults.getStartOffline();
		setAdditionalPolicy();
		if (siteDefaults == null) {
			siteDefaults = new SiteDefaults();
		}
		// take values from UserDefaults
		redirectLog(userDefaults.getRedirect(), userDefaults.getLogFormat(),
				logLevels.get(userDefaults.getLogLevel()));
		printVersionInfo();
		// set the required security provider
		// set the SunJSSE for accessing PKCS12 keystores
		//Provider sunjsse = new com.sun.net.ssl.internal.ssl.Provider();
		//Security.addProvider(sunjsse);
		//log.logWrite("Adding new security provider: " + sunjsse.getInfo());
		Security
				.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
		logger.info("Adding new security provider: ");
		urlReader = new URLReader(currentInstance);
		//       sslFactory = new UnicoreSSLSocketFactory();
		//   sslFactory.setSecureRandom(secureRandom);
		// set system property that allows to read https via the
		// java.net.URL class
		System.setProperty("java.protocol.handler.pkgs",
				"com.sun.net.ssl.internal.www.protocol");
		// this works for JavaWebstart and Windows !!
		URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {

			public URLStreamHandler createURLStreamHandler(final String protocol) {
				if (protocol != null && protocol.compareTo("https") == 0) {
					return new com.sun.net.ssl.internal.www.protocol.https.Handler();
				}
				return null;
			}
		});
		// build new user defaults, these will be loaded from
		// file or newly generated
		//  if(userDefaults == null) {
		//    userDefaults = new UserDefaults();
		//}
		// take values from UserDefaults
		//redirectStdout(userDefaults.getRedirect(),
		// userDefaults.getLogFormat(),
		// logLevels.get(userDefaults.getLogLevel()));
		// build file chooser table as background process
		Thread buildChooser = new Thread() {

			public void run() {
				// build file chooser table
				// containing a local chooser with initially selected job
				// directory
				buildFileChooserTable(userDefaults.getJobDirectory());
			}
		};
		buildChooser.start();
		//        buildFileChooserTable(userDefaults.getJobDirectory());
		defaultIdentity = userDefaults.getDefaultIdentity();
		String unicoreDir = userDefaults.getUnicoreDir();
		resourceCacheFilename = unicoreDir + File.separator
				+ "resourceCache.bin";
		// read cached resources
		readCacheFromDisk();
		logger.info("Starting in offline mode: " + offline);
	}

	/**
	 * Build a new Keystore and generate the user's test certificate
	 */
	public static void buildTestCertKeystore(String keystoreName)
			throws Exception {
		keystoreManager =
		// KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
		KeyStoreManager.getInstance(null, "JKS", keystoreName);
		// define new password
		DefinePasswordDialog definePasswordDialog = new DefinePasswordDialog(
				currentInstance, "UNICORE: Choose a Password");
		definePasswordDialog
				.setStatusMessage("Define a password for your new keystore.");
		definePasswordDialog.show();
		char[] passwd = definePasswordDialog.getPassword();
		if (passwd == null) {
			throw new Exception("Building initial keystore aborted by user.");
		}
		keystoreManager.buildEmptyKeystore(keystoreName, passwd, passwd);
		passwd = null;
		ServiceURLDialog urlDialog = null;
		urlDialog = new ServiceURLDialog(currentInstance, "URL:",
				"Enter URL to CA web service", true);
		urlDialog
				.setText("http://testgrid.unicorepro.com:5900/unicore/services/CAService?WSDL");
		urlDialog.show();
		if (urlDialog.getText() == null) {
			return;
		}
		Certificate[] caCerts = ServiceManager.getCACertificate(urlDialog
				.getText());
		X509Certificate userCert = (X509Certificate) ServiceManager
				.getSignedRequest(urlDialog.getText());
		if (userCert == null) {
			logger.log(Level.INFO, "Empty keystore has been built.");
			return;
		}
		for (int i = 0; i < caCerts.length; i++) {
			keystoreManager.addTrustedCertificate(caCerts[i], true);
		}
		String alias = CertificateUtility.getCommonName(userCert);
		keystoreManager.replaceKeyCertificate(alias, userCert, true);
		keystoreManager.reWriteKeyStore();
		userDefaults.setDefaultIdentity(alias);
		updateDefaultIdentity();
	}

	  /**
	   * added by rmenday
	   * called by buildEmptyKeystore to add certs in the 'certs' dir to keystore ...
	   * 
	   * @throws Exception
	   */
	  private static void importTrustedCerts()
	  {
	    Certificate cert;
	    File[] files = new File(new File(ResourceManager.getJarDirectory()).getParentFile(), "cert").listFiles();
	    for (int i = 0; i < files.length; i++)
	    {
	      if (files[i].isFile() && files[i].getName().endsWith(".pem"))
	      {
	        try
	        {
	          cert=CertificateUtility.importTrustedCertifcate(files[i]);
	          keystoreManager.addTrustedCertificate(cert, true);
	        }
	        catch (Exception e)
	        {
	          // some kind of problem reading cert file - ignore it for now.
	          e.printStackTrace();
	        }
	      }
	    }
	  }
		
	
	public static void initKeystoreManager(String keystoreName) {
		if (keystoreManager != null) {
			return;
		}
		keystoreManager =
		// KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
		KeyStoreManager.getInstance(null, "JKS", keystoreName);
	}

	/**
	 * Build an empty new keystore
	 */
	public static void buildEmptyKeystore(String keystoreName) throws Exception {
		keystoreManager =
			// KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
	KeyStoreManager.getInstance(null, "JKS", keystoreName);
		// define new password
		DefinePasswordDialog definePasswordDialog =
			new DefinePasswordDialog(currentInstance, "UNICORE: Choose a Password");
		definePasswordDialog.setStatusMessage("Define a password for your new keystore.");
		definePasswordDialog.show();
		char[] passwd = definePasswordDialog.getPassword();
		if (passwd == null) {
			throw new Exception("Building initial keystore aborted by user.");
		}
		keystoreManager.buildEmptyKeystore(keystoreName, passwd, passwd);
		// added by rmenday
		// add certs in the "cert" directory ...
		importTrustedCerts();
		
		passwd = null;
		File keystoreFile = new File(keystoreName);
		try {
			keystoreManager.reWriteKeyStore(keystoreFile);
		}
		catch (Exception exp) {
			logger.log(Level.WARNING, "Keystore is not complete. ", exp);
		}
	}

	public static void updateDefaultIdentity() throws Exception {
		updateDefaultIdentity(null);
	}

	/**
	 * Load keystore from file
	 */
	public static boolean loadKeystore(String keystoreName, char[] passwd)
			throws Exception {
		try {
			keystoreManager =
			// KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
			KeyStoreManager.getInstance(null, "JKS", keystoreName);
			keystoreManager.loadKeyStore(passwd, passwd);
		}
		catch (Exception firstException) {
			firstException.printStackTrace();
			if (firstException.getMessage().startsWith("Keystore")) {
				// autsch!! wrong password
				keystoreManager.resetKeystore();
				return false;
			}
			try {
				keystoreManager = KeyStoreManager.getInstance("SunJCE",
						"JCEKS", keystoreName);
				keystoreManager.loadKeyStore(passwd, passwd);
			}
			catch (Exception secondException) {
				secondException.printStackTrace();
				if (secondException.getMessage().startsWith("Keystore")) {
					// autsch!! wrong password
					keystoreManager.resetKeystore();
					return false;
				}
				try {
					// try PKCS12 format
					keystoreManager = KeyStoreManager.getInstance(null,
							"PKCS12", keystoreName);
					keystoreManager.loadKeyStore(passwd, passwd);
				}
				catch (Exception e) {
					UserMessages.warning("Could not open UNICORE keystore: "
							+ keystoreName, e.getLocalizedMessage());
					logger.log(Level.SEVERE, "Could not load keystore", e);
					if (e.getMessage().startsWith("Keystore")) {
						// autsch!! wrong password
						keystoreManager.resetKeystore();
						return false;
					}
					// if we got here something else but password went wrong...
					throw e;
				}
			}
		}
		try {
			if (userDefaults.getDefaultIdentity() == null
					|| userDefaults.getDefaultIdentity().equals("")) {
				Vector keyEntries = keystoreManager.getCachedKeyEntries();
				if (keyEntries != null && keyEntries.size() == 1) {
					getUserDefaults().setDefaultIdentity(
							(String) keyEntries.elementAt(0));
				}
			}
			updateDefaultIdentity(passwd);
		}
		catch (Exception e) {
			UserMessages.warning(res.getString("NO_DEFAULT_CERT"), e
					.getLocalizedMessage());
			// clear User defaults entry
			userDefaults.getDefaultIdentity();
			logger.log(Level.SEVERE, "Could not set user identity", e);
		}
		// keystore loading ok
		return true;
	}

	/**
	 * Copy a resource set and return the cloned set
	 * 
	 * @param resourceSet
	 *            input resource set
	 * @return deep copy of resource set
	 */
	public static ResourceSet cloneResourceSet(ResourceSet resourceSet) {
		ResourceSet resourceClones = new ResourceSet();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			try {
				resourceClones.add((Resource) resource.clone());
			}
			catch (CloneNotSupportedException e) {
				logger.log(Level.SEVERE, "Could not clone resource set "
						+ resourceSet, e);
			}
		}
		return resourceClones;
	}

	/**
	 * Load current USites into ResourceManager
	 * 
	 * @param page
	 *            input html page
	 * @return Vector containing Usite objects
	 * @throws MalformedURLException
	 *             if URL for USites is syntactically incorrect
	 * @throws IOException
	 *             if USites could not be loaded
	 */
	public static Vector parseHTML(String page) throws MalformedURLException,
			IOException {
		Vector usiteVector = new Vector();
		// first, search for tag "<dl id="Usites">
		int start = page.toUpperCase().indexOf("<DL ID=\"USITES\"");
		if (start == -1) {
			throw new IOException(
					"Bad usites file: Didn't find start tag '<dl id=\"Usites\">'.");
		}
		String currentString = page.substring(start);
		// loop over usite entries
		while (currentString.toUpperCase().indexOf("<A HREF=") != -1) {
			// look for host address URL
			start = currentString.indexOf("://");
			if (start == -1) {
				throw new IOException(
						"Bad usites file. Didn't find host address.");
			}
			currentString = currentString.substring(start + 3);
			// look for end of URL
			start = currentString.indexOf("\"");
			if (start == -1) {
				throw new IOException(
						"Bad usites file. Didn't find end of host address.");
			}
			try {
				// HACK: Usite requires a URL in constructor, so we re-add
				// the protocol here. This should be removed some day
				URL host = new URL("http://"
						+ currentString.substring(0, start));
				// find usite name to be displayed
				start = currentString.indexOf(">");
				currentString = currentString.substring(start);
				start = currentString.indexOf("<");
				String name = currentString.substring(1, start);
				usiteVector.add(new Usite(host, name));
				logger.info("Added Usite " + host);
			}
			catch (MalformedURLException e) {
				UserMessages.error("Error in Usite URL.", e.getMessage());
			}
		}
		return usiteVector;
	}

	/**
	 * Get current VSites from USite
	 * 
	 * @param usite
	 *            destination usite
	 */
	public static void loadVsites(Usite usite) {
		if (offline) {
			return;
		}
		if (usite.getType().equals(Usite.Type.CLASSIC)) {
			loadClassicalVsites(usite);
		}
	}

	/**
	 * Redirect console output to file
	 */
	public static void redirectLog(boolean redirect, String format, Level level) {
		File logDir = new File(userDefaults.getLogDirectory());
		boolean logDirExists = true;
		if (!logDir.exists() && !logDir.isDirectory()) {
			if (JOptionPane.showConfirmDialog(currentInstance,
					"Logging Directory " + logDir
							+ " does not exist. Create it?",
					"UNICORE: Missing Directory", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				logDir.mkdirs();
			}
			else {
				logger.warning("Logging Directory " + logDir
						+ " does not exist. Logging to console...");
				userDefaults.setLogFormat(UserDefaultsDialog.PLAIN_FORMAT);
				userDefaults.setRedirect(false);
				redirect = false;
				logDirExists = false;
			}
		}
		if (logDirExists) {
			fileXml = new File(userDefaults.getLogDirectory() + File.separator
					+ "clientlog.xml");
			filePlain = new File(userDefaults.getLogDirectory()
					+ File.separator + "clientlog.txt");
		}
		initHandler();
		//sets the level of the loggs
		Logger.getLogger("").setLevel(level);
		if (consoleHandler != null) {
			consoleHandler.setLevel(level);
		}
		if (redirect) {
			redirectLogsIntoFile(format);
		}
		else {
			redirectLogsToConsole(level);
		}
	}

	public static Object readObjectFromFile(String filename) {
		byte[] in = null;
		long tstart = System.currentTimeMillis();
		long timing = tstart;
		try {
			File file = new File(filename);
			FileInputStream f = new FileInputStream(file);
			ObjectInputStream ois = new ObjectInputStream(
					new BufferedInputStream(f));
			Object object = ois.readObject();
			ois.close();
			f.close();
			return object;
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Could not read object from file "
					+ filename, e);
		}
		return null;
	}

	public static boolean writeObjectToFile(Object object, String filename) {
		try {
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bout);
			oos.writeObject(object);
			oos.flush();
			oos.close();
			bout.flush();
			byte[] out = bout.toByteArray();
			bout.close();
			File file = new File(filename);
			FileOutputStream f = new FileOutputStream(file);
			BufferedOutputStream dout = new BufferedOutputStream(
					new DataOutputStream(f));
			dout.write(out, 0, out.length);
			dout.flush();
			dout.close();
			f.flush();
			f.close();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Could not write object " + object, e);
			return false;
		}
		return true;
	}

	public static void writeCacheToDisk() {
		logger.info("Writing cached information to disk.");
		Thread writer = new Thread() {

			public void run() {
				logger.info("Writing resource cache to file "
						+ resourceCacheFilename);
				/*
				 * try { FileOutputStream file = new
				 * FileOutputStream(resourceCacheFilename, false);
				 * XMLObjectWriter writer = new XMLObjectWriter(file);
				 * writer.writeObjectXML(resourceCache); file.flush();
				 * file.close(); } catch(Exception e) { log.logWrite("Could not
				 * write resource cache: " + e.getMessage()); }
				 */
				writeObjectToFile(resourceCache, resourceCacheFilename);
			}
		};
		writer.start();
	}

	/**
	 * load images only once
	 */
	public static void loadImages() {
		Class clazz = ResourceManager.class;
		ClassLoader loader = clazz.getClassLoader();
		imageTable = new Hashtable();
		iconTable = new Hashtable();
		for (int i = 0; i < imageFilenames.length; i++) {
			try {
				URL url = loader.getResource(imagePath + imageFilenames[i]);
				logger.fine("Loading image " + url);
				if (url == null) {
					logger.severe("Could not load " + imagePath
							+ imageFilenames[i]);
					continue;
				}
				Image image = Toolkit.getDefaultToolkit().createImage(url);
				if (image != null) {
					// put image into table
					imageTable.put(imageFilenames[i], image);
					// put icon into table
					iconTable.put(imageFilenames[i], new ImageIcon(image));
				}
				// put image into table
				imageTable.put(imageFilenames[i], image);
				// put icon into table
				iconTable.put(imageFilenames[i], new ImageIcon(image));
			}
			catch (Error e) {
				logger.log(Level.SEVERE, "Could not load " + imagePath
						+ imageFilenames[i], e);
			}
		}
	}

	/**
	 * Expand a property in the policy file
	 * 
	 * @param property
	 *            Which property
	 * @param newCodebase
	 *            The code base for the property
	 */
	public static void expandPolicy(String property, String newCodebase) {
		System.setProperty(property, newCodebase);
		logger.info("Expand: " + property + " to " + newCodebase);
		Policy pol = Policy.getPolicy();
		pol.refresh();
	}

	public static void initUsers() {
		if (getUsites().isEmpty()) {
			logger
					.severe("Usites haven't been loaded so far or no usites are available");
			return;
		}
		for (int i = 0; i < getUsites().size(); i++) {
			Usite usite = (Usite) getUsites().elementAt(i);
			initUserUsite(usite);
			Vector vsites = getVsites(usite);
			for (int j = 0; j < vsites.size(); j++) {
				initUserVsite((Vsite) vsites.elementAt(j));
			}
		}
	}

	/**
	 * Get Usite adresses from HTML or XML page. This method is called by
	 * getUsites()
	 */
	public static synchronized void loadUsites() {
		// return cached usites in offline mode
		if (offline) {
			return;
		}
		// try to get usite url from user defaults
		String urlStrings = userDefaults.getUsiteURL();
		if (urlStrings == null) {
			logger.severe("Error in UNICORE Sites URL from user defaults: "
					+ userDefaults.getUsiteURL());
			return;
		}
		// build up a vector containing *all* defined URL's
		usiteURLs = new Vector();
		// urlStrings is actually a list of URLs separated by "\t" or ","
		StringTokenizer st = new StringTokenizer(urlStrings, "\t,");
		// loop over usite urls
		while (st.hasMoreElements()) {
			String urlString = st.nextToken();
			// read usite page
			String usitePage;
			URL url = null;
			try {
				url = new URL(urlString);
			}
			// URL
			catch (java.net.MalformedURLException e) {
				//                logger.log(Level.SEVERE, "Error in Usite URL "+url+".", e);
				UserMessages.error("Error in UNICORE site URL " + url + ".", e
						.getMessage());
				continue;
			}
			usiteURLs.add(urlString);
		}
		// this should eventually go into a seperate method
		Vector enabledUsites = new Vector();
		boolean enabled = false;
		for (int i = 0; i < usiteURLs.size(); i++) {
			String next = (String) usiteURLs.elementAt(i);
			if (siteDefaults.isSiteContained((next))) {
				enabled = siteDefaults.getSiteValue(next);
				if (enabled) {
					// if the usites is contained in the defaults and enabled
					// add to list
					enabledUsites.add(next);
				}
			}
			else {
				// usite is not contained in the defaults add to list and
				// defaults
				enabledUsites.add(next);
				siteDefaults.setUrl(next, true);
			}
		}
		// loop over usite urls
		Vector newUsites = new Vector();
		for (int i = 0; i < enabledUsites.size(); i++) {
			String urlString = (String) enabledUsites.elementAt(i);
			// read usite page
			String usitePage;
			URL url = null;
			try {
				url = new URL(urlString);
				logger.info("Loading Usite information from: " + url);
				usitePage = urlReader.readURL(url);
			}
			// URL
			catch (java.net.MalformedURLException e) {
				//                logger.log(Level.SEVERE, "Error in Usite URL "+url+".", e);
				UserMessages.error("Error in UNICOREpro site URL " + url + ".",
						e.getMessage());
				continue;
			}
			// proxy or server unknown
			catch (UnknownHostException e) {
				UserMessages.warning("Could not read UNICORE sites from URL "
						+ url + ".", e.getMessage());
				continue;
			}
			// could not connect
			catch (ConnectException e) {
				UserMessages
						.warning(
								"Could not read UNICORE sites.",
								"Connection to: "
										+ url
										+ " refused."
										+ "\nPlease, check your proxy configuration in menu Settings -> User Defaults -> General."
										+ "\n(" + e.getMessage() + ")");
				continue;
			}
			// requestURL
			catch (IOException e) {
				//                logger.log(Level.SEVERE, "Could not request Usite URL
				// "+url+".", e);
				UserMessages.error("Could not request UNICORE site URL " + url
						+ ".", e.getMessage());
				continue;
			}
			// reading failed, continue with next urlString
			if (usitePage == null) {
				continue;
			}
			try {
				Vector v = parseXML(usitePage);
				newUsites.addAll(v);
			}
			catch (SAXException saxe) {
				// try html format
				try {
					Vector v = parseHTML(usitePage);
					newUsites.addAll(v);
				}
				catch (IOException ioe) {
					UserMessages
							.warning("Could not load UNICORE Site Severs from URL:\n<"
									+ urlString
									+ ">\n"
									+ "The file seems to be syntactically incorrect or unreadable.\n"
									+ "Please enter a valid URL in your user defaults.");
					UserMessages.print("Reason: " + ioe.getLocalizedMessage());
				}
			}
			catch (Exception e) {
				UserMessages
						.warning("Could not load UNICORE Site Severs from URL:\n<"
								+ urlString
								+ ">\n"
								+ "The file seems to be syntactically incorrect or unreadable.\n"
								+ "Please enter a valid URL in your user defaults.");
				UserMessages.print("Reason: " + e.getLocalizedMessage());
			}
		}
		resourceCache.updateUsites(newUsites);
		resourceCache.clearUnavailableUsites();
		resourceCache.clearUnavailableVsites();
		resourceCache.clearSpecialVsites();
	}

	public static void buildFileChooserTable(String selectedFile) {
		filechooserTable = new Hashtable();
		SelectorDialog dialog = new SelectorDialog(currentInstance, new File(
				selectedFile));
		filechooserTable.put(FileStorage.NSPACE_STRING, dialog);
	}

	public static void resetFileHandlers() {
		Handler[] handlers = Logger.getLogger("").getHandlers();
		for (int i = 0; i < handlers.length; i++) {
			if (handlers[i] instanceof FileHandler) {
				Handler handler = handlers[i];
				Logger.getLogger("").removeHandler(handlers[i]);
			}
		}
		if (logInto == LOG_INTO_FILES || logInto == LOG_INTO_XMLFILE) {
			appendTailToXmlFile(fileXml.toString());
		}
		filePlainHandler = null;
		fileXmlHandler = null;
		fileXml = null;
		filePlain = null;
		logInto = NEW_LOG_DIRECTORY;
	}

	/**
	 * Load user for Usite from Userdefaults
	 */
	private static void initUserUsite(Usite usite) {
		User userUsite = userDefaults.getUser(usite);
		if (userUsite != null) {
			userTable.put(usite, userUsite);
		}
	}

	/**
	 * Load user for Usite from Userdefaults
	 *  
	 */
	private static void initUserVsite(Vsite vsite) {
		User userVsite = userDefaults.getUser(vsite);
		if (userVsite != null) {
			userTable.put(vsite, userVsite);
		}
	}

	private static void loadFullVsites(Usite usite) {
		URL url = usite.getAddress();
		logger.info("Loading Vsite description from: " + usite);
		Reply vsitesReply = null;
		try {
			// Use ListVsite to get full Vsites
			ListVsites listVsites = new ListVsites();
			User user = getUser(usite);
			logger.info("Updating list of Vsites for Usite "
					+ usite.getName()
					+ " user "
					+ keystoreManager.getAliasFromCertifcate(user
							.getCertificate()) + " (ListVsites)");
			Connection connection = connectionManager.lookupConnection(usite,
					user);
			vsitesReply = connection.sendMessage(listVsites, null);
			int returnCode = vsitesReply.getLastEntry().getReturnCode();
			UnicoreResponse resp = vsitesReply.getLastEntry();
			Vector newVsites = new Vector();
			// Use ListVsite to get full Vsites
			if (!(vsitesReply instanceof ListVsitesReply)) {
				resp = vsitesReply.getLastEntry();
				resourceCache.addUnavailableUsite(usite, res
						.getString("NO_GATEWAY")
						+ url.getProtocol()
						+ "://"
						+ url.getHost()
						+ ":"
						+ url.getPort()
						+ res.getString("REASON")
						+ resp.getComment());
			}
			else {
				// Use ListVsite to get full Vsites
				Vsite[] vsiteArray = ((ListVsitesReply) vsitesReply).getList();
				if (returnCode == 0 && vsiteArray != null) {
					newVsites = resourceCache.updateVsites(usite, vsiteArray);
				}
				else {
					resp = vsitesReply.getLastEntry();
					resourceCache.addUnavailableUsite(usite, res
							.getString("NO_GATEWAY")
							+ url.getProtocol()
							+ "://"
							+ url.getHost()
							+ ":"
							+ url.getPort()
							+ res.getString("REASON")
							+ resp.getComment());
				}
			}
			// put vsite vector into table
			synchronized (lock) {
				resourceCache.addVsites(usite, newVsites);
			}
		}
		catch (StreamCorruptedException e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("NO_GATEWAY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		}
		catch (ClassNotFoundException e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("BOGUS_REPLY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		}
		catch (ConnectException e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage() + res.getString("WRONG_ADDRESS"));
		}
		catch (UnknownHostException e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		}
		catch (IOException e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage()
					+ res.getString("AUTHENTICATION")
					+ userInfo(user));
		}
		catch (Exception e) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		}
		if (vsitesReply == null) {
			resourceCache.addUnavailableUsite(usite, res
					.getString("CANNOT_LOAD_VSITES")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort());
		}
	}

	private static void loadClassicalVsites(Usite usite) {
		URL url = usite.getAddress();
		logger.info("Loading Vsite description from: " + usite);
		Reply vsitesReply = null;
		try {
			// Use ListPorts to get pure Broker Vsites
			ListPorts listVsites = new ListPorts();
			User user = getUser(usite);
			logger.info("Updating list of Vsites for Usite "
					+ usite.getName()
					+ " user "
					+ keystoreManager.getAliasFromCertifcate(user
							.getCertificate()) + "(ListPorts)");
			Connection connection = connectionManager.lookupConnection(usite,
					user);
			vsitesReply = connection.sendMessage(listVsites, null);
			UnicoreResponse resp = vsitesReply.getLastEntry();
			int returnCode = resp.getReturnCode();
			Vector newVsites = new Vector();
			// Use ListPorts to get pure Broker Vsites
			if (!(vsitesReply instanceof ListPortsReply)) {
				logger.warning(res.getString("NO_GATEWAY") + url.getProtocol()
						+ "://" + url.getHost() + ":" + url.getPort()
						+ res.getString("REASON") + resp.getComment());
			}
			else {
				// Use ListPorts to get pure Broker Vsites
				Vsite[] vsiteArray = ((ListPortsReply) vsitesReply).getList();
				if (returnCode == 0 && vsiteArray != null) {
					newVsites = resourceCache.updateVsites(usite, vsiteArray);
					// put vsite vector into table
					synchronized (lock) {
						resourceCache.addVsites(usite, newVsites);
					}
					// successfully received Vsites list
					return;
				}
				else {
					logger.warning(res.getString("NO_GATEWAY")
							+ url.getProtocol() + "://" + url.getHost() + ":"
							+ url.getPort() + res.getString("REASON")
							+ resp.getComment());
				}
			}
			// put EMPTY vsite vector into table
			synchronized (lock) {
				resourceCache.addVsites(usite, newVsites);
			}
			// go on with loadFullVsites
			loadFullVsites(usite);
		}
		catch (StreamCorruptedException e) {
			logger.warning(res.getString("NO_GATEWAY") + url.getProtocol()
					+ "://" + url.getHost() + ":" + url.getPort()
					+ res.getString("REASON") + e.getLocalizedMessage());
			loadFullVsites(usite);
		}
		catch (ClassNotFoundException e) {
			logger.warning(res.getString("BOGUS_REPLY") + url.getProtocol()
					+ "://" + url.getHost() + ":" + url.getPort()
					+ res.getString("REASON") + e.getLocalizedMessage());
			loadFullVsites(usite);
		}
		catch (ConnectException e) {
			logger.warning(res.getString("CONNECTION_REFUSED")
					+ url.getProtocol() + "://" + url.getHost() + ":"
					+ url.getPort() + res.getString("REASON")
					+ e.getLocalizedMessage() + res.getString("WRONG_ADDRESS"));
			loadFullVsites(usite);
		}
		catch (UnknownHostException e) {
			logger.warning(res.getString("UNKNOWN_HOST") + url.getProtocol()
					+ "://" + url.getHost() + ":" + url.getPort()
					+ res.getString("REASON") + e.getLocalizedMessage());
			loadFullVsites(usite);
		}
		catch (IOException e) {
			logger.warning(res.getString("CONNECTION_REFUSED")
					+ url.getProtocol() + "://" + url.getHost() + ":"
					+ url.getPort() + res.getString("REASON")
					+ e.getLocalizedMessage() + res.getString("AUTHENTICATION")
					+ userInfo(user));
			loadFullVsites(usite);
		}
		catch (Exception e) {
			logger.warning(res.getString("UNKNOWN_HOST") + url.getProtocol()
					+ "://" + url.getHost() + ":" + url.getPort()
					+ res.getString("REASON") + e.getLocalizedMessage());
			loadFullVsites(usite);
		}
		finally {
			if (vsitesReply == null) {
				logger.warning(res.getString("CANNOT_LOAD_VSITES")
						+ url.getProtocol() + "://" + url.getHost() + ":"
						+ url.getPort());
			}
		}
	}

	/**
	 * When user identity has changed from SecPropsEditor update it here
	 */
	private static void updateDefaultIdentity(char[] passwd) throws Exception {
		String defaultAlias = userDefaults.getDefaultIdentity();
		if (defaultAlias == null || defaultAlias.equals("")) {
			return;
		}
		initUsers();
		// update ssl keystore
		keystoreManager.createSSLKeystore(true, passwd, passwd);
		// we must initialize the SSL factory
		//       initializeSSLfactory(passwd);
		user = new User();
		X509Certificate[] chain = keystoreManager
				.getCertificateByAlias(defaultAlias);
		user.setCertificateChain(chain);
		String email = null;
		String xlogin = null;
		String project = null;
		if ((userDefaults.getUserEmail() != null)
				&& (!userDefaults.getUserEmail().equals(""))) {
			email = userDefaults.getUserEmail();
		}
		if ((userDefaults.getXlogin() != null)
				&& (!userDefaults.getXlogin().equals(""))) {
			xlogin = userDefaults.getXlogin();
		}
		if ((userDefaults.getProject() != null)
				&& (!userDefaults.getProject().equals(""))) {
			project = userDefaults.getProject();
		}
		user.setEmailAddress(email);
		user.setXlogin(xlogin);
		user.setProject(project);
	}

	private static void readCacheFromDisk() {
		logger.info("Reading cached information from disk.");
		Thread reader = new Thread() {

			public void run() {
				logger.info("Reading resource cache from file: "
						+ resourceCacheFilename);
				/*
				 * try { XMLObjectReader din = new XMLObjectReader(new
				 * File(resourceCacheFilename)); resourceCache =
				 * (ResourceCache)din.readObjectXML(); } catch(Exception e) {
				 * log.logWrite("Could not read resource cache: " +
				 * e.getMessage()); }
				 */
				resourceCache = (ResourceCache) readObjectFromFile(resourceCacheFilename);
				// if loading cache failed, build new one
				if (resourceCache == null) {
					resourceCache = new ResourceCache();
				}
			}
		};
		reader.start();
	}

	/**
	 * Load current USites into ResourceManager
	 */
	private static Vector parseXML(String page) throws MalformedURLException,
			IOException, SAXException, ParserConfigurationException {
		byte[] xmlBytes = page.getBytes();
		ByteArrayInputStream bis = new ByteArrayInputStream(xmlBytes);
		UsiteSaxHandler handler = new UsiteSaxHandler();
		// Use the validating parser
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		// Parse the input
		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse(bis, handler);
		Vector usites = handler.getUsites();
		// memory leak, if we do not set Vecxtor reference to null
		handler.setUsites(null);
		return usites;
	}

	private static void printVersionInfo() {
		String version = " Running UNICORE client version: ";
		version = version + Version.spcVersion();
		version = version
				+ " Implementation: "
				+ Package.getPackage("com.pallas.unicore.resourcemanager")
						.getImplementationVendor();
		logger.info(version);
		//version = " Running AJO version: ";
		//version =
		//	version
		//		+ Package.getPackage("org.unicore").getSpecificationVersion();
		//version =
		//	version
		//		+ " Implementation: "
		//		+ Package.getPackage("org.unicore.ajo").getImplementationVendor();
		//logger.info(version);
		version = "JVM version: ";
		version = version + System.getProperty("java.vm.version") + " System: "
				+ System.getProperty("os.name");
		logger.info(version);
	}

	private static void initHandler() {
		if (consoleHandler == null) {
			boolean console = false;
			for (int i = 0; i < Logger.getLogger("").getHandlers().length; i++) {
				if (Logger.getLogger("").getHandlers()[i] instanceof ConsoleHandler) {
					consoleHandler = Logger.getLogger("").getHandlers()[i];
					console = true;
					break;
				}
			}
			if (!console) {
				consoleHandler = new ConsoleHandler();
			}
			consoleHandler.setFormatter(new SimpleFormatter());
			consoleHandler.setLevel(logLevels.get(userDefaults.getLogLevel()));
		}
		if (fileXmlHandler == null) {
			try {
				if (fileXml != null) {
					fileXmlHandler = new FileHandler(fileXml.toString());
					fileXmlHandler.setFormatter(new NewXMLFormatter());
				}
			}
			catch (IOException e) {
				logger.log(Level.WARNING, "cannot write log into XML-file", e);
			}
			catch (SecurityException e) {
				logger.log(Level.WARNING, "cannot write log into XML-file", e);
			}
		}
		if (filePlainHandler == null) {
			try {
				if (filePlain != null) {
					filePlainHandler = new FileHandler(filePlain.toString());
					filePlainHandler.setFormatter(new SimpleFormatter());
				}
			}
			catch (IOException e) {
				logger.log(Level.WARNING, "cannot write log into simple-file",
						e);
			}
			catch (SecurityException e) {
				logger.log(Level.WARNING, "cannot write log into simple-file",
						e);
			}
		}
	}

	private static void redirectLogsToConsole(Level level) {
		if (logInto == LOG_TO_CONSOLE) {
			return;
		}
		logger.info("Logging to console");
		consoleHandler.setLevel(level);
		Logger.getLogger("").addHandler(consoleHandler);
		switch (logInto) {
			case LOG_INTO_PLAINFILE :
				Logger.getLogger("").removeHandler(filePlainHandler);
				break;
			case LOG_INTO_FILES :
				Logger.getLogger("").removeHandler(filePlainHandler);
			case LOG_INTO_XMLFILE :
				Logger.getLogger("").removeHandler(fileXmlHandler);
				appendTailToXmlFile(fileXml.toString());
		}
		logInto = LOG_TO_CONSOLE;
	}

	private static void redirectLogsIntoFile(String format) {
		if (UserDefaultsDialog.XML_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_XMLFILE) {
				return;
			}
			logger.info("Logging to XML-file");
			switch (logInto) {
				case LOG_INTO_PLAINFILE :
					Logger.getLogger("").addHandler(fileXmlHandler);
				case LOG_INTO_FILES :
					Logger.getLogger("").removeHandler(filePlainHandler);
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(fileXmlHandler);
					break;
			}
			logInto = LOG_INTO_XMLFILE;
			return;
		}
		if (UserDefaultsDialog.PLAIN_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_PLAINFILE) {
				return;
			}
			logger.info("Logging to simple file");
			switch (logInto) {
				case LOG_INTO_XMLFILE :
					Logger.getLogger("").addHandler(filePlainHandler);
				case LOG_INTO_FILES :
					Logger.getLogger("").removeHandler(fileXmlHandler);
					appendTailToXmlFile(fileXml.toString());
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
			}
			logInto = LOG_INTO_PLAINFILE;
			return;
		}
		if (UserDefaultsDialog.XML_AND_PLAIN_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_FILES) {
				return;
			}
			logger.info("Logging to XML and simple file");
			switch (logInto) {
				case LOG_INTO_PLAINFILE :
					Logger.getLogger("").addHandler(fileXmlHandler);
					break;
				case LOG_INTO_XMLFILE :
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(fileXmlHandler);
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
			}
			logInto = LOG_INTO_FILES;
		}
	}

	private static void appendTailToXmlFile(String fileNameXml) {
		try {
			PrintWriter tailOut = new PrintWriter(new FileOutputStream(
					fileNameXml, true));
			String tail = fileXmlHandler.getFormatter().getTail(fileXmlHandler);
			tailOut.print(tail);
			tailOut.close();
		}
		catch (IOException e) {
			logger.log(Level.WARNING, "Cannot append tail to XML-file ", e);
		}
	}

	/**
	 * Debugging function
	 */
	private void dumpHashtable(Hashtable table) {
		Enumeration e = table.keys();
		Object key;
		Object value;
		while (e.hasMoreElements()) {
			key = e.nextElement();
			value = table.get(key);
			logger.fine("Hash-Key: <" + key + "> value: <" + value + ">");
		}
	}

	private static class UsiteSaxHandler extends DefaultHandler {

		Vector foundSites = new Vector();

		public UsiteSaxHandler() {
		}

		public void setUsites(Vector foundSites) {
			this.foundSites = foundSites;
		}

		public Vector getUsites() {
			return foundSites;
		}

		//===========================================================
		// SAX DocumentHandler methods
		//===========================================================
		public void startDocument() throws SAXException {
		}

		public void endDocument() throws SAXException {
		}

		public void startElement(String namespaceURI, String lName,
				String qName, Attributes attrs) throws SAXException {
			String elementName = lName;
			if ("".equals(elementName)) {
				elementName = qName;
				// namespaceAware = false
			}
			if (elementName.equals("Usite")) {
				if (attrs != null) {
					String host = null;
					String tag = null;
					String port = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);
						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("name")) {
							tag = attrs.getValue(i);
						}
						if (aName.equals("address")) {
							host = attrs.getValue(i);
						}
						if (aName.equals("port")) {
							port = attrs.getValue(i);
						}
					}
					if (host != null && tag != null && port != null) {
						try {
							foundSites.add(new Usite(new URL("http://" + host
									+ ":" + port), tag, Usite.Type.CLASSIC));
						}
						catch (MalformedURLException e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}
				}
			}
			else if (elementName.equals("Registry")) {
				if (attrs != null) {
					String handle = null;
					String description = null;
					String address = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("handle")) {
							handle = attrs.getValue(i);
						}
						if (aName.equals("description")) {
							description = attrs.getValue(i);
						}
						if (aName.equals("address")) {
							address = attrs.getValue(i);
						}
					}
					if (handle != null && description != null
							&& address != null) {
						try {
							foundSites.add(new Usite(new URL(address), handle,
									Usite.Type.REGISTRY));
						}
						catch (MalformedURLException e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}
				}
			}
		}

		public void endElement(String namespaceURI, String sName, String qName)
				throws SAXException {
			// qualified name
		}

		public void characters(char[] buf, int offset, int len)
				throws SAXException {
		}
	}

	/**
	 * The site defaults contain a definition which usites and vsites to contact
	 * form the list of all configured sites
	 * 
	 * @return the definition which sites to contact
	 */
	public static SiteDefaults getSiteDefaults() {
		return siteDefaults;
	}

	public static void setSiteDefaults(SiteDefaults defaults) {
		siteDefaults = defaults;
	}
}