/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.threadpool;

import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Thomas Kentemich
 * @version $Id: GenericObservable.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class GenericObservable extends Object implements IObservable {

	/**
	 * holds all registered observers
	 */
	private Vector myObservers = new Vector();

	/**
	 * Creates new GenericObservable
	 */
	public GenericObservable() {
	}

	/**
	 * add an observer
	 * 
	 * @param theObserver
	 *            The feature to be added to the Observer attribute
	 */
	public void addObserver(IObserver theObserver) {
		this.myObservers.addElement(theObserver);
	}

	/**
	 * delete all registered Observers
	 */
	public void deleteAllObservers() {
		this.myObservers.removeAllElements();
	}

	/**
	 * delete an observer
	 * 
	 *  
	 */
	public void deleteObserver(IObserver theObserver) {
		this.myObservers.removeElement(theObserver);
	}

	/**
	 * iterate through the list of observers and notify them
	 * 
	 *  
	 */
	public void notifyObservers(Object theObserved, Object changeCode) {
		Enumeration myObserversList = this.myObservers.elements();
		while (myObserversList.hasMoreElements()) {
			IObserver theObserver = (IObserver) myObserversList.nextElement();
			theObserver.observableUpdate(theObserved, changeCode);
		}
	}
}