/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.utility;

import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;

/**
 *  This class represents the different logging levels
 *
 *@author     Kirsten Foerster
 *@version    $Id: LoggingLevels.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */

public class LoggingLevels {
	private Vector levels = null;

	private Hashtable levelsTable = null;

	
	public LoggingLevels() {
		this.init();
	}

	/**
	 *  add a Level object
	 *
	 *@param  level  - a new created Level object
	 */
	public void add(Level level) {
		levels.addElement(level);
		this.levelsTable.put(level.toString(), level);
	}

	
	public Level get(String object) {
		return (Level) this.levelsTable.get(object);
	}

	public Vector getLevels() {
		return levels;
	}

	/**
	 * initialize the vector levels and the hashtable levelsTable conisting of
	 * all levels and their belonging strings
	 */
	private void init() {
		this.initLevels();
		this.initLevelsTable(levels);
	}

	/**
	 *  initialize the vector levels
	 */
	private void initLevels() {
		levels = new Vector();

		levels.addElement(Level.ALL);
		levels.addElement(Level.FINEST);
		levels.addElement(Level.FINER);
		levels.addElement(Level.FINE);
		levels.addElement(Level.CONFIG);
		levels.addElement(Level.INFO);
		levels.addElement(Level.WARNING);
		levels.addElement(Level.SEVERE);
		levels.addElement(Level.OFF);
	}

	/**
	 *  initialize the hashtable levelsTable conisting of all levels and their
	 *  belonging strings
	 *
	 *@param  levels  - a vector with Level-objects
	 */
	private void initLevelsTable(Vector levels) {
		this.levelsTable = new Hashtable();
		for (int i = 0; i < levels.size(); i++) {
			this.levelsTable.put(
				((Level) levels.elementAt(i)).toString(),
				(Level) levels.elementAt(i));
		}
	}

}
