/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.utility;

import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @version $Id: UserMessages.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class UserMessages {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.utility");

	public static void error(final String message) {
		if (!Client.getRunningMode()) {
			if (SwingUtilities.isEventDispatchThread()) {
				showError(message);
			}
			else {
				Runnable doUpgrade = new Runnable() {

					public void run() {
						showError(message);
					}
				};
				try {
					SwingUtilities.invokeAndWait(doUpgrade);
				}
				catch (java.lang.reflect.InvocationTargetException e) {
				}
				catch (java.lang.InterruptedException e) {
				}
			}
		}
		logger.severe(message);
	}

	public static void error(String message, String reason) {
		if (reason != null && !reason.equals("")) {
			message += "\nReason: " + reason;
		}
		error(message);
	}

	public static void info(final String message) {
		if (!Client.getRunningMode()) {
			if (SwingUtilities.isEventDispatchThread()) {
				showInfo(message);
			}
			else {
				Runnable doUpgrade = new Runnable() {

					public void run() {
						showInfo(message);
					}
				};
				try {
					SwingUtilities.invokeAndWait(doUpgrade);
				}
				catch (java.lang.reflect.InvocationTargetException e) {
				}
				catch (java.lang.InterruptedException e) {
				}
			}
		}
		logger.info(message);
	}

	public static void info(String message, String reason) {
		if (reason != null && !reason.equals("")) {
			message += "\nReason: " + reason;
		}
		info(message);
	}

	public static void print(String message) {
		logger.fine(message);
	}

	public static void print(String message, String reason) {
		if (reason != null && !reason.equals("")) {
			message += "\nReason: " + reason;
		}
		print(message);
	}

	public static void showError(String message) {
		JOptionPane.showMessageDialog(ResourceManager.getCurrentInstance(),
				message, "UNICORE: Error", JOptionPane.ERROR_MESSAGE);
	}

	public static void showInfo(String message) {
		JOptionPane.showMessageDialog(ResourceManager.getCurrentInstance(),
				message, "UNICORE: Information",
				JOptionPane.INFORMATION_MESSAGE);
	}

	public static void showWarning(String message) {
		JOptionPane.showMessageDialog(ResourceManager.getCurrentInstance(),
				message, "UNICORE: Warning", JOptionPane.WARNING_MESSAGE);
	}

	public static void warning(final String message) {
		if (!Client.getRunningMode()) {
			if (SwingUtilities.isEventDispatchThread()) {
				showWarning(message);
			}
			else {
				Runnable doUpgrade = new Runnable() {

					public void run() {
						showWarning(message);
					}
				};
				try {
					SwingUtilities.invokeAndWait(doUpgrade);
				}
				catch (java.lang.reflect.InvocationTargetException e) {
				}
				catch (java.lang.InterruptedException e) {
				}
			}
		}
		logger.warning(message);
	}

	public static void warning(String message, String reason) {
		if (reason != null && !reason.equals("")) {
			message += "\nReason: " + reason;
		}
		warning(message);
	}

	private UserMessages() {
	}
}