/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.utility;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Kirsten Foerster
 * @version $Id: VectorComparator.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class VectorComparator extends Vector {

	public VectorComparator() {
		super();
	}

	public VectorComparator(Collection c) {
		super(c);
	}

	/**
	 * Comparates the current vector to the elements of the transferred
	 * enumeration.
	 * 
	 * @param enum
	 * @return Returns a vector which elements are the intersection of the
	 *         current vector and the transferred enumeration.
	 */
	public Vector contains(Enumeration enum) {
		Vector result = new Vector();
		if (enum == null) {
			return result;
		}
		while (enum.hasMoreElements()) {
			Object object = enum.nextElement();
			if (this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 * Comparates the current vector to the transferred vector.
	 * 
	 * @param vector
	 * @return Returns a vector which elements are the intersection of both
	 *         vectors.
	 */
	public Vector contains(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return result;
		}
		for (int i = 0; i < vector.size(); i++) {
			Object object = vector.elementAt(i);
			if (this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 * Comparates the current vector to the elements of the transferred
	 * enumeration.
	 * 
	 * @param enum
	 * @return Returns a vector which elements are only components in the
	 *         current vector.
	 */
	public Vector excludes(Enumeration enum) {
		if (enum == null) {
			return this;
		}
		Vector vector = new Vector();
		while (enum.hasMoreElements()) {
			vector.add(enum.nextElement());
		}
		return this.excludes(vector);
	}

	/**
	 * Comparates the current vector to the transferred vector.
	 * 
	 * @param vector
	 * @return Returns a vector which elements are only components in the
	 *         current vector
	 */
	public Vector excludes(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return this;
		}
		for (int i = 0; i < this.size(); i++) {
			Object object = this.elementAt(i);
			if (!vector.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 * Comparates the current vector to the elements of the transferred
	 * enumeration.
	 * 
	 * @param enum
	 * @return Returns a vector which elements are only components in the
	 *         transferred enumeration.
	 */
	public Vector isExcluded(Enumeration enum) {
		Vector result = new Vector();
		if (enum == null) {
			return null;
		}
		while (enum.hasMoreElements()) {
			Object object = enum.nextElement();
			if (!this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 * Comparates the current vector to the transferred vector.
	 * 
	 * @param vector
	 * @return Returns a vector which elements are only components in the
	 *         transferred vector
	 */
	public Vector isExcluded(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return null;
		}
		for (int i = 0; i < vector.size(); i++) {
			Object object = vector.elementAt(i);
			if (!this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}
}