/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.utility;

import java.util.Arrays;
import java.util.Vector;

/**
 * Sort a vector of comparable objects. If objects do not implement the
 * Comparable interface use their String representation for comparision.
 * 
 * @author Ralf Ratering
 * @version $Id: VectorSorter.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class VectorSorter {

	private static boolean ascending;
	private static StringComparator stringComparator = new StringComparator();
	private final static boolean USE_ARRAY_SORT = true;
	private static boolean useStrings;
	private static Vector vector;

	private static void arraySort() {
		Object[] result = vector.toArray();
		if (useStrings) {
			Arrays.sort(result, stringComparator);
		}
		else {
			Arrays.sort(result);
		}
		vector.clear();
		if (ascending) {
			for (int i = 0; i < result.length; i++) {
				vector.add(result[i]);
			}
		}
		else {
			for (int i = 0; i < result.length; i++) {
				vector.add(result[result.length - i - 1]);
			}
		}
	}

	private static int compare(int i, Object object) {
		if (useStrings) {
			int value = vector.elementAt(i).toString().compareTo(
					object.toString());
			return (ascending ? value : -value);
		}
		int value = ((Comparable) vector.elementAt(i)).compareTo(object);
		return (ascending ? value : -value);
	}

	public static void main(String[] args) {
		Vector v = new Vector();
		v.add("afs");
		v.add("sfh");
		v.add("afs");
		v.add("aaatwrzuzthj");
		v.add("dss");
		v.add("afwrths");
		v.add("awrthfs");
		v.add("zhwthtwrhad");
		v.add("aawrha");
		v.add("dswww3456s");
		v.add("afs");
		v.add("afs");
		v.add("zad");
		v.add("gad");
		v.add("fss");
		v.add("afs");
		System.err.println(v);
		VectorSorter.sort(v, false);
		System.err.println(v);
	}

	private static int partition(int l, int r) {
		Object pivot = vector.elementAt((l + r) / 2);
		int i = l - 1;
		int j = r + 1;
		while (true) {
			while (compare(--j, pivot) > 0) {
			}
			while (compare(++i, pivot) < 0) {
			}
			if (i < j) {
				swap(i, j);
			}
			else {
				return j;
			}
		}
	}

	private static void quicksort(int l, int r) {
		if (l < r) {
			int q = partition(l, r);
			quicksort(l, q);
			quicksort(q + 1, r);
		}
	}

	public static Vector sort(Vector v) throws ClassCastException {
		sort(v, true);
		return v;
	}

	public static Vector sort(Vector _vector, boolean _ascending)
			throws ClassCastException {
		if (_vector.isEmpty()) {
			return _vector;
		}
		boolean _useStrings = !(_vector.firstElement() instanceof Comparable);
		return sort(_vector, _ascending, _useStrings);
	}

	public static Vector sort(Vector _vector, boolean _ascending,
			boolean _useStrings) throws ClassCastException {
		vector = _vector;
		ascending = _ascending;
		useStrings = _useStrings;
		if (vector.isEmpty()) {
			return vector;
		}
		if (USE_ARRAY_SORT) {
			arraySort();
		}
		else {
			quicksort(0, vector.size() - 1);
		}
		return vector;
	}

	private static void swap(int i, int j) {
		Object temp = vector.elementAt(i);
		vector.setElementAt(vector.elementAt(j), i);
		vector.setElementAt(temp, j);
	}

	private VectorSorter() {
	}
}