package org.unicore.ajo;

import org.unicore.AJOIdentifier;

/**
 * CopyJob returns a copy of the target AJO.
 * <p>
 * @see org.unicore.outcome.CopyJob_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version CopyJob.java 1.5 09/09/02 
 *
 **/
public final class CopyJob extends QueryService {

    static final long serialVersionUID = -1761318271216313226L;

    /**
     * Create a new CopyJob.
     * <p>
     * @param name The name of the CopyJob
     * @param extra_information Client supplied extra information about the CopyJob
     * @param target_ajo The Identifier ofthe AJO to copy
     *
     **/
    public CopyJob(String name,
                   byte[] extra_information,
                   AJOIdentifier target_ajo
                     ) {
                     
        super(name,extra_information);

        this.target_ajo = target_ajo;
    }

    /**
     * Create a new CopyJob.
     * <p>
     * @param name The name of the CopyJob
     * @param target_ajo The Identifier ofthe AJO to copy
     *
     **/
    public CopyJob(String name, AJOIdentifier target_ajo) {
        this(name, (byte[]) null, target_ajo);
    }

    /**
     * Create a new CopyJob.
     * <p>
     * @param name The name of the CopyJob
     *
     **/
    public CopyJob(String name) {
        this(name, (byte[]) null, (AJOIdentifier) null);
    }

    public CopyJob() {
        this("");
    }

    private AJOIdentifier target_ajo;
    /**
     * Get the identifier of the AJO to copy.
     *
     **/
    public AJOIdentifier getTarget() {return target_ajo;}

    /**
     * Set the identifier of the AJO to copy.
     *
     **/
    public void setTarget(AJOIdentifier target_ajo) {this.target_ajo = target_ajo;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Constructor to public 
//           7 Dec 1998 SvdB Second release to Unicore 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
