package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

import org.unicore.PortfolioIdentifier;

/**
 * Copy a Portfolio to the Outcome of the task. 
 * <p>
 * The contents of the target Portfolio at the time of
 * the execution of the task are copied into
 * the Outcome of this task. This is the way that files 
 * are returned to a client. 
 *<p>
 * The location of the Portfolio is determined by the the
 * Storage resources. These can be either {@link org.unicore.resources.USpace},
 * {@link org.unicore.resources.AlternativeUspace} or {@link org.unicore.resources.Spool}.
 * The default is the root ofthe AJO's Uspace.
 *
 * @see Portfolio
 * @see org.unicore.outcome.CopyPortfolioToOutcome_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version CopyPortfolioToOutcome.java 1.4 09/09/02 
 * 
 **/
public final class CopyPortfolioToOutcome extends FileTransfer {

    static final long serialVersionUID = 0;

    /**
     * Create a new CopyPortfolioToOutcome.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Portfolio to copy to the Outcome.
     *
     **/
    public CopyPortfolioToOutcome( String name,
                              byte[] extra_information,
                              ResourceSet resources,
                              PortfolioIdentifier target
                            ) {
        super(name,extra_information,resources);
        this.target = target;
    }

    /**
     * Create a new CopyPortfolioToOutcome.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Portfolio to copy to the Outcome.
     *
     **/
    public CopyPortfolioToOutcome( String name,
                              ResourceSet resources,
                              PortfolioIdentifier target
                            ) {

        this(name,(byte[]) null, resources, target);

    }

    /**
     * Create a new CopyPortfolioToOutcome with this name
     * and no Portfolio to copy.
     *
     **/
    public CopyPortfolioToOutcome(String name) {
        this(name, (byte[]) null, (ResourceSet) null, (PortfolioIdentifier) null);
    }

    public CopyPortfolioToOutcome() {
        this("");
    }

    private PortfolioIdentifier target;
    /**
     * Return the Portfolio to copy.
     *
     **/
    public PortfolioIdentifier getTarget() {return target;}

    /**
     * Set the Portfolio to copy.
     * <p>
     * This Portfolio must be a Portfolio that is
     * created by a task in the same AJO as this instance
     * of CopyPortfolioToOutcome. The creating task should
     * also have completed execution before this
     * CopyPortfolioToOutcome is executed.
     *
     **/
    public void setTarget(PortfolioIdentifier target) {this.target = target;}

}
//
// Change history:
//          05 Jub 2002 SvdB Made target PortfolioIdentifier (consistent with allowing AUS and Spool)
//          18 Mar 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
