package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Copy the incarnation of a Portfolio to a directory in the Xspace.
 * <p>
 * The source Portfolio is found in the instance of {@link org.unicore.resources.USpace}
 * found in the resources (default is the Uspace root). 
 * <p>
 * The destination is found in the instance of {@link org.unicore.resources.Home}, 
 * {@link org.unicore.resources.Root}, {@link org.unicore.resources.Temp} or
 * {@link org.unicore.resources.StorageServer} found in the resources (default is
 * the root of the Home filesystem).
 * <p>
 * Any directories found in the Portfolio incarnation are copied recursively to
 * the destination.
 *
 * @see org.unicore.outcome.ExportTask_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version ExportTask.java 1.8 09/18/02 
 * 
 **/
public final class ExportTask extends FileTransfer {

    static final long serialVersionUID = -1314928620567473922L;

    /**
     * Create a new ExportTask.
     * <p>
     * @param name The name of the ExportTask
     * @param extra_information Client supplied extra information about the ExportTask
     * @param resources Resources needed to execute the ExportTask
     * @param files The Portfolio to be exported
     * @param overwrite Whether the export should overwrite any existing files.
     * @param xspace_directory The destination directory for the Portfolio's files
     *
     * @deprecated AJO 4.0 The Xspace directory information is now in the path of the Storage resource that
     *             identifies the destination.
     *
     **/
    public ExportTask( String name,
                       byte[] extra_information,
                       ResourceSet resource_limits,
                       Portfolio files, 
                       boolean overwrite, 
                       String xspace_directory
                     ) {

        super(name,extra_information,resource_limits);

        this.files = files;
        this.overwrite = overwrite;
        this.xspace_directory = xspace_directory;
    }

    /**
     * Create a new ExportTask.
     * <p>
     * @param name The name of the ExportTask
     * @param resources Resources needed to execute the ExportTask
     * @param files The Portfolio to be exported
     * @param overwrite Whether the export should overwrite any existing files.
     * @param xspace_directory The destination directory for the Portfolio's files
     *
     * @deprecated AJO 4.0 The Xspace directory information is now in the path of the Storage resource that
     *             identifies the destination.
     *
     **/
    public ExportTask( String name,
                       ResourceSet resource_limits,
                       Portfolio files, 
                       boolean overwrite, 
                       String xspace_directory
                     ) {

        this(name,(byte[]) null,resource_limits, files, overwrite, xspace_directory);
    }

    /**
     * Create a new ExportTask.
     * <p>
     * @param name The name of the ExportTask
     * @param extra_information Client supplied extra information about the ExportTask
     * @param resources Resources needed to execute the ExportTask
     * @param files The Portfolio to be exported
     * @param overwrite Whether the export should overwrite any existing files.
     *
     **/
    public ExportTask( String name,
                       byte[] extra_information,
                       ResourceSet resource_limits,
                       Portfolio files, 
                       boolean overwrite 
                     ) {

        super(name,extra_information,resource_limits);

        this.files = files;
        this.overwrite = overwrite;
    }

    /**
     * Create a new ExportTask.
     * <p>
     * @param name The name of the ExportTask
     * @param resources Resources needed to execute the ExportTask
     * @param files The Portfolio to be exported
     * @param overwrite Whether the export should overwrite any existing files.
     *
     **/
    public ExportTask( String name,
                       ResourceSet resource_limits,
                       Portfolio files, 
                       boolean overwrite
                     ) {

        this(name,(byte[]) null,resource_limits, files, overwrite);
    }

    /**
     * Create a new ExportTask.
     * <p>
     * @param name The name of the ExportTask
     *
     **/
    public ExportTask(String name) {
        this(name,(byte[]) null, (ResourceSet) null, (Portfolio) null, false);
    }

    public ExportTask() {
        this("");
    }

    private String xspace_directory;
    /**
     * Return the <em>directory</em> into which the files will
     * be placed.
     *
     * @deprecated AJO 4.0 The Xspace directory information is now in the path of the Storage resource that
     *             identifies the destination.
     *
     **/
    public String getXspaceDirectory() {return xspace_directory;}

    /**
     * Set the <em>directory</em> into which the files will
     * be placed.
     *
     * @deprecated AJO 4.0 The Xspace directory information is now in the path of the Storage resource that
     *             identifies the destination.
     *
     **/
    public void setXspaceDirectory(String xspace_directory) {this.xspace_directory = xspace_directory;}

    private Portfolio files;
    /**
     * Return the Portfolio to be exported.
     *
     **/
    public Portfolio getPortfolio() {return files;}

    /**
     * Set the  Portfolio to be exported.
     *
     **/
    public void setPortfolio(Portfolio files) {this.files = files;}

    private boolean overwrite;
    /**
     * Overwrite existing destination files?
     * <p>
     * If no overwriting is allowed (this method returns false), then the export will
     * fail if it would overwrite an existing file. As the destination is a directory,
     * the test will be applied to each file in the directory (that
     * is, writing files into existing directories is always allowed).
     * <p>
     * Overwriting existing files or writing into existing directories is always
     * subject to permissions on the existing files or directories and the export
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set the overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, etc
//          16 Nov 2000 SvdB Make overwrite method naming consistent with CopyFile and RenameFile
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//          12 Jun 1998 SvdB Initial release to Unicore
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
