package org.unicore.ajo;

import org.unicore.Unicore;

/**
 * Option represent token/value pairs.
 * <p>
 * These are used for either the environment variables or cpp defines/undefines
 *
 * @see FortranTask#getDefines
 * @see FortranTask#getUndefines
 * @see ExecuteTask#getEnvironmentVariables
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version Option.java 1.7 09/09/02 
 * 
 **/
public final class Option extends Unicore {

    static final long serialVersionUID = -2046853465658039934L;


    public Option() {
        this("","");
    }

    /**
     * Create an Option with just the token set.
     *
     **/
    public Option(String token) {
        this(token,"");
    }

    /**
     * Create an Option mapping a token to a name.
     *
     **/
    public Option(String token, String value) {
        setToken(token);
        this.value = value;
    }

    private String token;
    /**
     * Return the token.
     *
     **/
    public String getToken() {return token;}

    /**
     * Change the token.
     *
     **/
    public void setToken(String token) {
        if(token == null) {
            this.token = "null_token";
        }
        else if(token.equals("")) {
            this.token = "empty_token";
        }
        else {
            this.token = token;
        }
    }

    private String value;
    /**
     * Return the value.
     * <p>
     * Empty String implies no value
     *
     **/
    public String getValue() {return value;}

    /**
     * Change the value.
     *
     **/
    public void setValue(String value) {this.value = value;}

}
//
// Change history:
//          20 May 2002 SvdB Tokens never null
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
