package org.unicore.ajo;

import org.unicore.AAIdentifier;
import org.unicore.Unicore;
import org.unicore.sets.ResourceSet;

/**
 * This class encapsulates information about resources required by a particular
 * {@link org.unicore.ajo.AbstractTask}.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.6
 *
 * @version TaskResource.java 1.3 05/13/02
 * 
 **/
public class TaskResource extends Unicore {

    static final long serialVersionUID = 6326682031992490739L;

    private AAIdentifier taskID;    
    private ResourceSet resourceSet;
        
    /**
     * Creates new TaskResource object.
     *
     * @param taskID Abstract Action Identifier
     * @param resourceSet Set of resources required for the execution of the Abstract Action.
     *
     **/
    public TaskResource(AAIdentifier taskID, ResourceSet resourceSet) {
        this.taskID = taskID;
        this.resourceSet  = resourceSet;
    }

    public TaskResource() {
        this(null,null);
    }

    /**
     * Returns the Identifier for this Abstract Action
     *
     **/
    public AAIdentifier getTaskID(){
        return this.taskID;
    }

    /** 
     *
     * Returns a {@link org.unicore.sets.ResourceSet } set of resource required by this Abstract Action.
     *
     **/
    public ResourceSet getResourceSet(){
        return this.resourceSet;
    }
}
//
// Change history:
//          18 Mar 200s SvdB V4 no arg constructor
//           6 Dec 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
