package org.unicore.outcome;

import java.util.Hashtable;

import org.unicore.Unicore;
import org.unicore.AAIdentifier;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.ajo.AbstractAction;

/**
 * A ConditionalAction_Outcome contains the results of both the
 * branches of the ConditionAction
 *
 * @see org.unicore.ajo.ConditionalAction
 * @see org.unicore.ajo.If
 * @see org.unicore.ajo.IfSuccess
 *
 * @author David F. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version ConditionalAction_Outcome.java 1.4 07/11/02
 * 
 **/
public abstract class ConditionalAction_Outcome extends Outcome {

    static final long serialVersionUID = 0;

    public ConditionalAction_Outcome() {
        this((AbstractAction)null, null);
    }

    /**
     * Create a new ConditionalAction_Outcome.
     * <p>
     * @param source The AbstractAction for which this Outcome will hold the results.
     * @param initial The status of the AbstractAction at the time that the Outcome is created
     *
     **/
    public ConditionalAction_Outcome(AbstractAction source, AbstractActionStatus initial) {
      super (source, initial);
    }

    /**
     * Create a new ConditionalAction_Outcome.
     * <p>
     * @param source_id The AAIdentifier of the ConditionalAction for which this Outcome will hold the results.
     * @param initial The status of the ConditionalAction at the time that the Outcome is created
     *
     **/
    public ConditionalAction_Outcome(AAIdentifier source_id, AbstractActionStatus initial) {
      super (source_id, initial);
    }

    protected Outcome true_branch_outcome;
    /**
     * Return the Outcome of the branch that will be executed if the
     * Conditional Action's test evaluates to true.
     *
     **/
    public Outcome getTrueBranchOutcome() {
        return true_branch_outcome;
    }

    public void setTrueBranchOutcome(Outcome outcome) {
        true_branch_outcome = outcome;
    }

    protected Outcome false_branch_outcome;
    /**
     * Return the Outcome of the branch that will be executed if the
     * Conditional Action's test evaluates to false.
     *
     **/
    public Outcome getFalseBranchOutcome() {
        return false_branch_outcome;
    }

    public void setFalseBranchOutcome(Outcome outcome) {
        false_branch_outcome = outcome;
    }

    protected Outcome taken_outcome;
    /**
     * Return the Outcome of the branch that was executed.
     * <p>
     * This will be the same as one of the true or false branches
     * or null if the ConditionalAction has not been executed.
     *
     **/
    public Outcome getTakenOutcome() {
        return taken_outcome;
    }

    public void setTakenOutcome(Outcome outcome) {
        taken_outcome = outcome;
    }

}

//
// Change histroy:
//           8 jul 2002 SvdB Keep both Outcomes, have method to return taken Outcome
//          18 Mar 2002 SvdB V4 no arg constructor
//          30 Jan 2001  DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
