package org.unicore.outcome;

import java.util.Hashtable;

import org.unicore.AAIdentifier;
import org.unicore.ajo.ControlAction;

/**
 * The result of applying a {@link org.unicore.ajo.ControlAction} to an AbstractAction.
 * <p>
 * If the AbstractAction is an ActionGroup, then we need to report
 * the result of applying the control to all child AbstractActions.
 * If the target AbstractAction has no children, then the status 
 * as returned by {@link Outcome#getStatus()} is the result of applying
 * the ControlAction. Otherwise, one should use {@link #getStatus(AAIdentifier)}
 * for each child AbstractAction.
 *
 * @see org.unicore.ajo.ControlAction
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version ControlAction_Outcome.java 1.6 09/09/02 
 * 
 **/
public class ControlAction_Outcome extends Outcome {

    static final long serialVersionUID = 6616823391568421088L;

    protected Hashtable statuses;

    public ControlAction_Outcome() {
        this(null, null);
    }

    /**
     * Create a new ControlAction_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus a breakdown of the result of applying the ControlAction
     * to any child AbstractActions of the target.
     *
     * @param source The ControlAction for which this Outcome will hold the results.
     * @param initial The status of the ControlAction at the time that the Outcome is created.
     *
     **/
    public ControlAction_Outcome(ControlAction        source,
                                AbstractActionStatus initial_status) {
        super(source, initial_status);

        statuses = new Hashtable(11);
    }
    

    /**
     * Add the result of applying the control
     * to the AbstractAction.
     * <p>
     * <b>Pre:</b> There is no Status for the AbstractAction
     *               in this ControlActionOutcome
     * <br>
     * <b>Post:</b> The Status is in this ControlActionOutcome
     * <p>
     **/
    public void addStatus(AAIdentifier id, AbstractActionStatus status) {
        statuses.put(id,status);
    }

    /**
     * Returns the result of applying the ControlAction to an AbstractAction.
     * <p>
     * <b>Pre:</b> true
     * <br>
     * <b>Post:</b> Returns the Status or null if there is no
     *              Status for this AbstractAction
     * <p>
     **/
    public AbstractActionStatus getStatus(AAIdentifier id) {
        return (AbstractActionStatus) statuses.get(id);
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          20 Dec 2000 SvdB Removed clone method
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
