package org.unicore.outcome;

import org.unicore.ajo.ExportTask;

/**
 * The status and logs of executing an {@link org.unicore.ajo.ExportTask}.
 * <p>
 * As a Portfolio's contents may not be known until run time and
 * as the actual path to a Storage Server's sub-space is not
 * obvious from its name, this contains a list of the names
 * of the actual files written to the Xspace by the execution
 * of the ExportTask.
 *
 * @see org.unicore.ajo.ExportTask
 *
 * @author S. van den Berghe (fecit)
 * 
 * @since AJO 2
 *
 * @version ExportTask_Outcome.java 1.6 05/13/02 
 * 
 **/
public class ExportTask_Outcome extends AbstractTask_Outcome {

    static final long serialVersionUID = 5475276878561863292L;

    public ExportTask_Outcome() {
        this(null,null);
    }

    /**
     * Create a new ExportTask_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}.
     *
     * @param source The ExportTask for which this Outcome will hold the results.
     * @param initial The status of the ExportTask at the time that the Outcome is created.
     *
     **/
    public ExportTask_Outcome( ExportTask           source,
                               AbstractActionStatus initial_status
                        ) {
        super(source, initial_status); 
    }

    String[] written_files;

    /**
     * Get the list of file names 
     * written by the ExportTask.
     *
     * The list contains the full path to each
     * of the files exported.
     *
     **/
    public String[] getWrittenFiles() {return written_files;}

    /**
     * Set the list of file names 
     * written by the ExportTask.
     *
     **/
    public void setWrittenFiles(String[] files) {written_files = files;}

    /**
     * Add a file name to the list of files
     * written by the ExportTask.
     *
     **/
    public void addWrittenFile(String file) {
        if(written_files == null) {
            written_files = new String[1];
            written_files[0] = file;
        }
        else {
            String[] temp = new String[written_files.length+1];
            System.arraycopy(written_files,0,temp,0,written_files.length);
            temp[written_files.length] = file;
            written_files = temp;
        }
    }


}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
