package org.unicore.resources;

/**
 * This class describes a Priority capability available at a site. 
 * <p>
 * Each priority level supported by a Vsite
 * described by a separate instance of this resource in the Resource Description.
 * A Resource Description can contain 0 or more instances of Priority.
 * <p>
 * Only one instance of a Priority is expected in a Resource Request.
 * No Priority in the Resource Request implies the default for the Vsite. 
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version Priority.java 1.11 05/13/02 
 *
 **/
public final class Priority extends CapabilityResource {

    static final long serialVersionUID = 387098581445482792L;

    /**
     * Create a new Priority
     *
     * @param description A description of this Priority
     * @param value The value for this Priority
     * 
     **/
    public Priority (String description, PriorityValue value) {
        super (description);
        this.value = value;
    }
    /**
     * Create a new Priority
     *
     * The PriorityValue is set to WHENEVER
     *
     **/
    public Priority () {
        this("undescribed Priority", PriorityValue.WHENEVER);
    }

    private PriorityValue value = PriorityValue.WHENEVER;
    /**
     * Get the value of this priority resource.
     *
     **/
    public  PriorityValue getValue ()                    { return value; }

    /**
     * Set the value of this priority resource.
     *
     **/
    public void setValue (PriorityValue value) { 
        if(value == null) {
            this.value = PriorityValue.WHENEVER;
        }
        else {
            this.value = value; 
        }
    }

    /**
     * Priorities are equal if their PriorityValues are the same. 
     *
     **/
    public boolean equals(Object o) {
        if(o == null) return false;
        return ( o.getClass() == getClass() && getValue().equals(((Priority)o).getValue()) ); // value is NEVER null
    }

    /**
     * Return a hash code for this Priority. 
     * <p>
     * Returns the hash code
     * of the value (since the value determines equality).
     *
     **/
    public int hashCode() {
        return getValue().hashCode(); // value is NEVER null
    }

    /**
     * Returns the PriorityValue as a String.
     *
     **/
    public String toString() {
        return getValue().toString();
    }

}
//
// Change history:
//          20022002    SvdB value is never null
//          13 Jul 2000 SvdB override equals
//          21 Jan 2000 DFS  First release with V3.0 resource model. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
