package org.unicore.resources;

/**
 * Resource for describing the time that a Task will be allowed to
 * occupy the system. 
 * <p>
 * The elapsed run time, but not the CPU time.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version RunTime.java 1.11 09/09/02
 *
 **/
public final class RunTime extends CapacityResource {

    static final long serialVersionUID = 538454300605567856L;

    public RunTime () {
        this("", 0.0, 0.0, 0.0);
    }

    /**
     * Create a new RunTime.
     * <p>
     * Sets the units to "Seconds"
     *
     * @param description A description of the RunTime
     * @param maxrequest Maximum amount of resource allowed
     * @param defaultrequest Amount of resource granted if no specific request received
     * @param minrequest Minimum amount of resource allowed
     *
     **/
    public RunTime (String description,
                    double maxrequest,
                    double defaultrequest,
                    double minrequest) {
        super(description, maxrequest, defaultrequest, minrequest);
        this.units = "Seconds";
    }

    public String toString() {
        return "RunTime";
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor
//          14 Aug 2000 DFS Added minimum request.
//          21 Jan 2000  DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
