package org.unicore.sets;

import java.util.Vector;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.outcome.*;
import org.unicore.resources.*;
import org.unicore.upl.*;

/**
 * <tt>ActionGroup_OutcomeList</tt> provides an ordered list of <tt>ActionGroup_Outcome</tt> elements. 
 *
 * @author M. Drescher
 *
 * @since AJO 1
 *
 * @version listTemplate 1.11 09/09/02 
 *
 */
public class ActionGroup_OutcomeList extends Unicore {

    private Vector list;

    public ActionGroup_OutcomeList() {
        list = new Vector();
    }
        
    public ActionGroup_OutcomeList(ActionGroup_Outcome first) {
        list = new Vector();
        add(first);
    }
        
    /**
     * Test if a <tt>ActionGroup_Outcome</tt> is in the list.
     * <p>
     * <b>Pre:</b>  True.  
     * <b>Post:</b> Return true iff the <tt>ActionGroup_Outcome</tt> is in the list
     * <p>
     *
     * @return Return true iff the <tt>ActionGroup_Outcome</tt> is in the list
     *
     **/
    public boolean isIn(ActionGroup_Outcome thing) { return list.contains(thing); }

    /**
     * Return the number of elements in the ActionGroup_OutcomeList.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> Return the number of elements
     *
     * @return the number of elements in the ActionGroup_OutcomeList
     *
     **/
    public int size() { return list.size();}

    /**
     * Append a <tt>ActionGroup_Outcome</tt> to this list.
     * <p>
     * <b>Pre:</b>  true
     * <b>Post:</b> If the <tt>ActionGroup_Outcome</tt> was non-null, then the <tt>ActionGroup_Outcome</tt> is at the end of the list.
     * <p>
     *
     * @param thing A <tt>ActionGroup_Outcome</tt>.
     */
    public void add(ActionGroup_Outcome thing) { if(thing != null) list.addElement(thing); }

    /**
     * Sets a <tt>ActionGroup_Outcome</tt> at the specified position <tt>pos</tt>.
     * The <tt>ActionGroup_Outcome</tt> occupying this position is discarded. 
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> If the <tt>ActionGroup_Outcome</tt> was non-null, then the <tt>ActionGroup_Outcome</tt> is in the list at pos and
     *              the <tt>ActionGroup_Outcome</tt> that was at pos in not in the list
     *
     * @param thing A <tt>ActionGroup_Outcome</tt>
     * @param pos An index in the list
     */
    public void setAt(ActionGroup_Outcome thing, int pos) {
        if(thing != null) {
            try{
                 list.setElementAt(thing,pos);
            }
            catch(ArrayIndexOutOfBoundsException ex) {
                System.err.println("This must not happen in org.unicore.sets.ActionGroup_OutcomeList (pre-condition)"+ex);
            }
        }
    }
            
    /**
     * Insert a <tt>ActionGroup_Outcome</tt> at the specified position.
     *
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> If the <tt>ActionGroup_Outcome</tt> was non-null, then the <tt>ActionGroup_Outcome</tt> is in the list at pos 
     *
     * @param thing A <t>ActionGroup_Outcome</tt>
     * @param pos An index in the list
     */
    public void insertAt(ActionGroup_Outcome thing, int pos) {
        if(thing != null) {
            try{
                 list.insertElementAt(thing,pos);
            }
            catch(ArrayIndexOutOfBoundsException ex) {
                System.err.println("This must not happen in org.unicore.sets.ActionGroup_OutcomeList (pre-condition)"+ex);
            }
        }
    }

    /**
     * Removes the <tt>ActionGroup_Outcome</tt> from the list.
     *
     * <p>
     * <b>Pre:</b>  True. 
     * <b>Post:</b> The <tt>ActionGroup_Outcome</tt> is not in the list
     * <p>
     *
     * @param thing A <tt>ActionGroup_Outcome</tt>
     */
    public void remove(ActionGroup_Outcome thing) { list.removeElement(thing);}

    /**
     * Removes the ActionGroup_Outcome at the specified position.
     *
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> the <tt>ActionGroup_Outcome</tt> that was at pos in not in the list
     * <p>
     *
     * @param pos An index in the list
     */
    public void removeAt(int pos) {
        try{
             list.removeElementAt(pos);
        }
        catch(ArrayIndexOutOfBoundsException ex) {
            System.err.println("This cannot happen (pre-condition)"+ex);
        }
    }

    /**
     * Removes all objects in the list.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> The list is empty
     *
     **/
    public void removeAll() {list.removeAllElements();}

    /**
     * Returns the position of the <tt>ActionGroup_Outcome</tt> in the list. 
     *
     * <p>
     * <b>Pre:</b> The <tt>ActionGroup_Outcome</tt> is in the list 
     * <b>Post:</b> Returns the position of the <tt>ActionGroup_Outcome</tt>
     * <p>
     *
     * @param thing The <tt>ActionGroup_Outcome</tt> to locate
     * @return The position of the <tt>ActionGroup_Outcome</tt>
     *
     **/
    public int pos(ActionGroup_Outcome thing) { return list.indexOf(thing);}

    /**
     * Returns the <tt>ActionGroup_Outcome</tt> at the specified position.
     *
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> the <tt>ActionGroup_Outcome</tt> at pos
     * <p>
     *
     * @param pos An index in the list
     * @return A <tt>ActionGroup_Outcome</tt>
     *
     **/
    public ActionGroup_Outcome getAt(int pos) {
        try{
             return (ActionGroup_Outcome) list.elementAt(pos);
        }
        catch(ArrayIndexOutOfBoundsException ex) {
            System.err.println("This must not happen in org.unicore.sets.ActionGroup_OutcomeList (pre-condition)"+ex);
            return null;
        }
    }

    /**
     * Returns a <tt>ActionGroup_OutcomeEnumeration</tt>.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> the enumeration (in no particular order)
     * <p>
     *
     *
     * @return A <tt>ActionGroup_OutcomeEnumetaion</tt>
     */
    public ActionGroup_OutcomeEnumeration elements() {return new ActionGroup_OutcomeEnumeration(list);}

    /**
     * Return a shallow clone of the list.
     *
     * The clone is shallow to match the style
     * of the java.util collections.
     *
     **/
    public ActionGroup_OutcomeList cloneList() {
        // Do not use clone as we do not return an Object
        // so compiler will complain (Object sits poorly with rest)
        ActionGroup_OutcomeList result = new ActionGroup_OutcomeList(); 
        result.list = (Vector) list.clone();
        return result;
    }

    static final long serialVersionUID = 2L;
}
//
// Change history
//          31 Jan 2001 SvdB Rejects null elements
//           4 Oct 2001 SvdB Unfinalised
//           5 May 2000 SvdB V3.0 
//          11 Mar 1999 SvdB Includes serialVersionUID 
//          22 Dec 1998 SvdB SvdB, Added constructor with initial element 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB SvdB Added clone method 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
