package org.unicore.sets;

import java.util.Vector;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.outcome.*;
import org.unicore.resources.*;
import org.unicore.upl.*;

/**
 * A type-safe container class for <tt>Resource</tt>s.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version  setTemplate 1.11 09/09/02 
 * 
 **/

public class ResourceSet extends Unicore {

    private Vector set;

    public ResourceSet() {
        set = new Vector();
    }
    
    public ResourceSet(Resource first) {
        set = new Vector();
        add(first);
    }
    
    /**
     * Test if a <tt>Resource</tt> is in the set.
     * <p>
     * <b>Pre:</b>  True.  
     * <b>Post:</b> Return true iff the <tt>Resource</tt> is in the set
     * <p>
     *
     * @return Return true iff the <tt>Resource</tt> is in the set
     *
     **/
    public boolean isIn(Resource thing) { return set.contains(thing); }


    /**
     * Add a <tt>Resource</tt> to the set.
     * <p>
     * <b>Pre:</b>  The <tt>Resource</tt> is not in the set and non-null
     * <b>Post:</b> The <tt>Resource</tt> is in the set  
     * <p>
     *
     * @param thing The <tt>Resource</tt> to be added.
     *
     **/
    public void add(Resource thing) { if(thing != null) set.addElement(thing); }

    /**
     * Remove the <tt>Resource</tt> from the set.
     * <p>
     * <b>Pre:</b>  true 
     * <p>
     * <b>Post:</b> The <tt>Resource</tt> is not in the set.
     * <p>
     * @param thing The <tt>Resource</tt> to be removed.
     *
     **/
    public void remove(Resource thing) { set.removeElement(thing);}

    /**
     * Remove all the <tt>Resource</tt>s from the set.
     * <p>
     * <b>Pre:</b>  true 
     * <p>
     * <b>Post:</b> The ResourceSet is empty
     * <p>
     *
     **/
    public void removeAll() { set.clear();}

    /**
     *
     * Create and return an enumeration over the elements in
     * the <tt>ResourceSet</tt>.
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> the enumeration (in no particular order)
     * <p>
     *
     * @return The enumeration.
     *
     **/
    public ResourceEnumeration elements() {return new ResourceEnumeration(set);}

    /**
     * Return a shallow clone of the set.
     *
     * The clone is shallow to match the style
     * of the java.util collections.
     *
     **/
    public ResourceSet cloneSet() {
        // Do not use clone as we do not return an Object
        // so compiler will complain (Object sits poorly with rest)
        ResourceSet result = new ResourceSet(); 
        result.set = (Vector) set.clone();
        return result;
    }

    static final long serialVersionUID = 3L;

}
//
// Change history:
//
//          21 Feb 2002 SvdB Added removeAll method
//          31 Jan 2001 SvdB Do not add null objects
//           4 Oct 2001 SvdB Unfinalised
//           5 May 2000 SvdB V3.0 
//          11 Mar 1999 SvdB Includes serialVersionUID 
//          22 Dec 1998 SvdB SvdB, Added constructor with initial element 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB SvdB Added clone method 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
