package org.unicore.sets;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.outcome.*;
import java.util.*;

/**
 * This class contains a map from Vsites to the TaskIDStatus sets.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 * @author Sven van den Berghe (fecit)
 *
 * @since AJO 3.6
 *
 * @version  mapTemplate 1.8 11/25/02
 * 
 **/

public class VsiteTaskIDStatusMap extends Unicore {

    private Map map;
   
    /**
     * Creates a new VsiteTaskIDStatusMap with a 1st Vsite to TaskIDStatus set entry.
     *
     **/
    public VsiteTaskIDStatusMap (Vsite vsite, TaskIDStatusSet tcset) {
        map = new HashMap (); 
        map.put (vsite, tcset);
    }
    
    /**
     * Create an empty VsiteTaskIDStatusMap.
     *
     **/
    public VsiteTaskIDStatusMap () {
        map = new HashMap (); 
    }
    
    /**
     * Adds a new record associating a Vsite and TaskIDStatusSet .
     *
     * Any previous records for the Vsite are overwritten.
     *
     * Both must be non-null
     *
     *
     **/
    public void addTaskIDStatusSet (Vsite vsite, TaskIDStatusSet tcset) {
        if(vsite != null && tcset != null) map.put (vsite, tcset);
    }

    /**
     * Remove the record for a Vsite.
     *
     * @param vsite Unicore Vsite
     *
     **/
    public void removeTaskIDStatusSet (Vsite vsite) {
        map.remove (vsite);
    }

    /**
     * Removes all records in the Map.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> The map is empty
     *
     **/
    public void removeAll() {map.clear();}

    /**
     * Returns the TaskIDStatusSet for this Vsite.
     *
     **/
    public TaskIDStatusSet getTaskIDStatusSet (Vsite vsite) {
        return (TaskIDStatusSet) map.get (vsite); 
    }
    
    /**
     * Returns an Enumeration over the Vsites for which an entry is held. 
     *
     **/
    public VsiteEnumeration getVsiteEnumeration () {
        return new VsiteEnumeration(new Vector(map.keySet()));
    }

     static final long serialVersionUID = 400L;
}
//
// Change history:
//          25 Nov 2002 SvdB Removed deprecation of getVsiteEnumeration
//          19 Jul 2002 SvdB Retains Vsites (now that we have equals())
//          28 May 2002 SvdB Constructor matches set type
//          21 Feb 2002 SvdB Added removeAll method
//          12 Dec 2001 DFS Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
