package org.unicore.upl;

import org.unicore.Unicore;

import java.util.ArrayList;

/**
 * Identifies a UPL reply sent by a Unicore server to a Unicore
 * client.
 * <p>
 * Every {@link org.unicore.upl.Request} is matched by a Reply.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9 (AJO 2 in other versions)
 *
 * @version Reply.java 1.5 05/13/02  
 * 
 **/
public class Reply extends Unicore {

    /**
     * Create a new Reply.
     *
     **/
    public Reply() {
        trace = new ArrayList(11);
    }

    private ArrayList trace;
    /**
     * Get a trace of all the responses added to this Reply..
     *
     **/
    public UnicoreResponse[] getTrace() {
        UnicoreResponse[] result = new UnicoreResponse[trace.size()];
        return (UnicoreResponse[]) trace.toArray(result);
    }

    /**
     * Add a response to the end of the trace. 
     *
     * @param entry The latest entry
     *
     **/
    public void addTraceEntry(UnicoreResponse entry) {
        trace.add(entry);
    }

    /**
     * Add an array of responses to the end of the current trace.
     *
     **/
    public void addTraceEntries(UnicoreResponse[] entries) {
        for(int i=0; i< entries.length; i++) {
            addTraceEntry(entries[i]);
        }
    }

    /**
     * Get the last response added to the list.
     * <p>
     * When the client receives this Reply this should 
     * be the the final UnicoreResponse for the request -
     * and so contain the overall status for the Request's
     * processing.
     *
     **/
    public UnicoreResponse getLastEntry() {
        if(trace.isEmpty()) {
            return null;
        }
        else {
            return (UnicoreResponse) trace.get(trace.size()-1);
        }
    }

    static final long serialVersionUID = -6806315233582826576L;

}
//
// Change history:
//          17 Oct 2000 SvdB Added get last response
//           4 Feb 2000 SvdB Unicore-Plus, renamed from RequestReply,
//                           deleted refs to other packages 
//          15 Mar 1998 SvdB Made concrete for (very) exceptional replies 
//          16 Dec 1998 SvdB Finished 
//           7 Dec 1998 SvdB Second release to Unicore 
//           3 Dec 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
