package org.unicore.upl;

import org.unicore.Vsite;
import org.unicore.AJOIdentifier;

/**
 * A RetrieveOutcomeAck says that a client has
 * no more need of the Outcome of an AJO.
 * <p>
 * An NJS should retain the Outcome of an AJO until
 * it receives a RetrieveOutcomeAck for it. Following this
 * the NJS is free to delete the AJO's Outcome.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9 (AJO 2 in other versions)
 *
 * @version RetrieveOutcomeAck.java 1.5 06/19/02  
 * 
 **/
public final class RetrieveOutcomeAck extends ServerRequest {

    /**
     * Create an empty RetrieveOutcomeAck.
     *
     **/
    public RetrieveOutcomeAck() {
        this(null,null,null);
    }

    /**
     * Create a RetrieveOutcomeAck.
     *
     * @param target The AJO whose Outcome can be deleted
     * @param vsite The Vsite where the AJO was executed
     *
     **/
    public RetrieveOutcomeAck(AJOIdentifier target, Vsite vsite) {
        this(target,vsite,null);
    }

    /**
     * Create a RetrieveOutcomeAck.
     *
     * @param target The AJO whose Outcome can be deleted
     * @param vsite The Vsite where the AJO was executed
     * @param sso Security information required by the Vsite
     *
     **/
    public RetrieveOutcomeAck(AJOIdentifier target, Vsite vsite, byte[] sso) {
        super(target,vsite,sso);
    }

    static final long serialVersionUID = -7626451436775243347L;

}
//
// Change history:
//          15 Mar 2002, SvdB V4
//           4 Feb 2000, SvdB Modifications for Unicore-Plus AJO  
//          16 Dec 1998, SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
