package org.unicore.upl;

import org.unicore.Unicore;

import java.util.Date;

/**
 * A UnicoreResponse is a logging entry that
 * is returned in a <tt>Reply</tt>. 
 * <p>
 * It is timestamped and has fields for comments and a return code.
 * <p>
 * This is intended to log the processing of a {@link Request},
 * the execution of an AJO is logged in its {@link org.unicore.outcome.Outcome}.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9 (AJO 2 in other versions)
 *
 * @version UnicoreResponse.java 1.5 05/13/02  
 * 
 **/
public class UnicoreResponse extends Unicore {

    /**
     * Create an empty UnicoreResponse
     *
     **/
    public UnicoreResponse() {
    }

    /**
     * Create a UnicoreResponse (time stamped at the moment this was called)
     *
     * @param return_code The return code.
     * @param comment The reason for this response
     *
     **/
    public UnicoreResponse( int return_code,
                            String comment ) {
        this(return_code, comment, new Date());
    }

    /**
     * Create a UnicoreResponse
     *
     * @param return_code The return code.
     * @param comment The reason for this response
     * @param time_stamp Time of this response
     *
     **/
    public UnicoreResponse( int return_code,
                            String comment,
                            Date time_stamp ) {

        this.return_code = return_code;
        this.comment = comment;
        this.time_stamp = time_stamp;
    }

    protected int return_code;
    /**
     * Get the return code for this response.
     * <p>
     * 0 is success, anything else is a problem
     * processing the request.
     *
     **/
    public int getReturnCode() {return return_code;}

    /**
     * Set the return code for this response.
     *
     **/
    public void setReturnCode(int return_code) {this.return_code = return_code;}

    protected String comment;

    /**
     * Return the comment for this response.
     *
     **/
    public String getComment() {return comment;}

    /**
     * Set the comment for this response.
     *
     **/
    public void setComment(String comment) {this.comment = comment;}

    protected Date time_stamp;
    /**
     * Return the time stamp for this response.
     *
     **/
    public Date getTimeStamp() {return time_stamp;}

    /**
     * Set the time stamp for this response.
     *
     **/
    public void setTimeStamp(Date time_stamp) {this.time_stamp = time_stamp;}

    static final long serialVersionUID = 9016531790079529364L;

}
//
// Change history:
//          15 Aug 2000 SvdB Not final
//           4 Feb 2000 SvdB Changes for Unicore-Plus 
//          17 Jan 1999 SvdB More friendly constructor 
//          11 Jan 1999 SvdB Not abstract 
//          16 Dec 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
